#!/usr/bin/python

import	sys
import	os
import	string
import	re

from	xml.dom.ext.reader.Sax2	import FromXmlStream
from	xml.dom.ext		import PrettyPrint
from	xml.dom.minidom		import DOMImplementation


keywords	= {}
db		= None
manual		= 'rekall'

def findChild (node, name) :

    child = node.firstChild
    while child is not None :
        if child.nodeType == child.ELEMENT_NODE and child.tagName == name :
            return child
        child = child.nextSibling
    return None


def findTitleAny (node) :

    try    : return node.firstChild.data
    except : pass
    try    : return node.firstChild.firstChild.data
    except : pass
    return None

def getChildrenByTagName (node, name) :

    elements = []
    child    = node.firstChild
    while child is not None :
        if child.nodeType == child.ELEMENT_NODE and child.tagName == name :
            elements.append (child)
        child = child.nextSibling
    return elements


def findTitle (node) :

    title = findTitleAny (node)
    if title is None : return None

    title = string.replace (title, '\n', '')
    title = string.replace (title, '\r', '')
    title = string.replace (title, '\t', ' ')
    title = re.sub ('  +',  ' ', title)
    title = re.sub ('\( +', '(', title)
    title = re.sub (' +\)', ')', title)

    return string.strip(title)


def scanForSections (doc, tags, pid, elem) :

    for node in doc.getElementsByTagName (tags[0]) :
        id      = node.getAttribute('id')
        section = elem.ownerDocument.createElement('section')
        if tags[0] == 'chapter' :
            section.setAttribute('ref', '%s/%s.html' % (manual, id))
        if tags[0] == 'appendix' :
            section.setAttribute('ref', '%s/%s.html' % (manual, id))
        if tags[0] == 'sect1'   :
            section.setAttribute('ref', '%s/%s.html' % (manual, id))
        if tags[0] == 'sect2'   :
            section.setAttribute('ref', '%s/%s.html#%s' % (manual, pid, id))
        title   = findChild (node, 'title')
        ttext   = findTitle (title)
        if ttext is not None :
            ttext = re.sub ('\n',   '',   ttext)
            ttext = re.sub ('  *',  ' ',  ttext)
            ttext = re.sub ('\( *', '(',  ttext)
            ttext = re.sub (' *\)', ')',  ttext)
            ttext = re.sub (' *\(', ' (', ttext)
            ttext = re.sub (',  *', ', ', ttext)
            section.setAttribute ('title', ttext)
        elem.appendChild(section)
        if len(tags) > 1 :
            scanForSections (node, tags[1:], id, section)


def extractKeywords (index, file) :

    for word in string.split(open('manual/%s/%s' % (manual, file)).read()) :

        word = string.lower(word)
        word = re.sub ('[^A-Za-z0-9_]', '', word)
        if word == '' : continue

        try :
            w = keywords[word]
        except :
            w = []
            keywords[word] = w

        if not index in w : w.append (index)


def makeRichText (srce) :

    srce = open(srce).read()
    patt = re.compile ('.*<body>(.*)</body>.*', re.M|re.I|re.S)
    m    = patt.match (srce)
    if m : srce = m.group(1)

    patt = re.compile ('(.*?)<img src="(.*?)">(.*)', re.M|re.I|re.S)
    dest = ''

    m    = patt.match (srce)
    while m is not None :

        dest = dest + m.group(1) + '<img src="'
        name = m.group(2)

        if db is not None :

            file = 'manual/%s/%s' % (manual, name)

            try :
                img  = open(file).read()
                name = string.replace (name, '/',    '_')
                name = string.replace (name, '.png',  '')
                name = 'man_' + name

                db.cursor().execute \
                (   "insert into __RekallObjects (Name, Type, Definition, SaveDate, Extension) values (%s, %s, %s, %s, %s)",
                    [   name,
                        'graphic',
                        img,
                        '20050430120000',
                        'png'
                    ]
                )

            except :
                print file, '**Missing**'

        dest = dest + name + '.png"/>'
        srce = m.group(3)
        m    = patt.match (srce)

    dest = dest + srce

    patt = re.compile ('(.*?)<a href="(.*?)">(.*)', re.M|re.I|re.S)
    srce = dest
    dest = ''

    m    = patt.match (srce)
    while m is not None :
        dest = dest + m.group(1) + '<a href="link:///'
        dest = dest + m.group(2)
        dest = dest + '">'
        srce = m.group(3)
        m    = patt.match (srce)

    dest = dest + srce
    return '<qt>' + dest + '</qt>'


def makeMainFile (doc, index) :

    main     = DOMImplementation().createDocument ('main', 'help', None)
    contents = main.createElement('contents')

    try :
        bookinfo = findChild (doc.documentElement, 'bookinfo')
        title    = findChild (bookinfo, 'title')
        contents.setAttribute ('title', title.firstChild.data)
    except :
        contents.setAttribute ('title', 'None')

    main.documentElement.appendChild (contents)

    scanForSections (doc, [ 'chapter',  'sect1', 'sect2' ], None, contents)
    scanForSections (doc, [ 'appendix', 'sect1', 'sect2' ], None, contents)

    filecnt  = 0
    keyfiles = main.createElement  ('keyfiles')
    main.documentElement.appendChild (keyfiles)

    for file in os.listdir ('manual/rekall') :

        if file[-5:] != '.html' : continue

        keyfile = main.createElement ('keyfile')
        keyfile.setAttribute ('id',  '%d' % (filecnt))
        keyfile.setAttribute ('ref',  '%s/%s' % (manual, file))
        keyfiles.appendChild (keyfile)

        extractKeywords (filecnt, file)
        filecnt = filecnt + 1

    keylist  = main.createElement ('keywords')
    main.documentElement.appendChild (keylist)

    for key in keywords.keys() :
        keyitem = main.createElement ('keyword')
        keyitem.setAttribute ('word', key)
        keylist.appendChild (keyitem)
        for occur in keywords[key] :
            keyoccurs = main.createElement ('keyoccurs')
            keyoccurs.setAttribute ('id', '%d' % (occur))
            keyitem.appendChild (keyoccurs)

    idxelem = main.createElement ('index')
    main.documentElement.appendChild (idxelem)

    keys = index.keys()
    keys.sort()
    for key in keys :
        i     = index[key]
        item1 = main.createElement ('item')
        idxelem.appendChild  (item1)
        item1  .setAttribute ('title', i[0])
        item1  .setAttribute ('ref',   i[1])
        for j in i[2] :
            item2 = main.createElement ('item')
            item1.appendChild  (item2)
            item2.setAttribute ('title', j[0])
            item2.setAttribute ('ref',   j[1])
        
    return main

def makeSections (chapter, index, chapno) :

    sectno = 0

    for node in getChildrenByTagName (chapter, 'sect1') :

        sectno = sectno + 1
        id     = node.getAttribute('id')
        sect   = open ('/tmp/rm_%s.xml' % (id), 'w')
        sect.write  ("<?xml version='1.0' encoding='UTF-8'?>\n")
        sect.write  ('<section>\n')
        child = node.firstChild
        while child is not None :
            PrettyPrint (child, sect)
            child = child.nextSibling
        sect.write  ('</section>\n')
        sect.close  ()
        os.system   ('xsltproc m_section.xsl /tmp/rm_%s.xml > manual/%s/%s.html' % (id, manual, id))

        title = findChild (node, 'title')
        ttext = findTitle (title)
        entry = [ ttext, '%s/%s.html' % (manual, id), [] ]
        index[ttext] = entry

        if db is not None :
            db.cursor().execute \
            (   "insert into manual (chapno, secno, tag, title, body) values (%s, %s, %s, %s, %s)",
                [   chapno,
                    sectno,
		    '%s.html' % (id),
                    ttext,
                    makeRichText('manual/%s/%s.html' % (manual, id))
                ]
            )

        for sect2 in node.getElementsByTagName ('sect2') :
            title = findChild (sect2, 'title')
            ttext = findTitle (title)
            tag   = sect2.getAttribute('id')
            entry[2].append ([ ttext, '%s.html#%s' % (id, tag)])


def makeChapters (doc) :

    index  = {}
    chapno = 0

    for node in getChildrenByTagName (doc.documentElement, 'chapter') + getChildrenByTagName (doc.documentElement, 'appendix') :

        chapno = chapno + 1
        id     = node.getAttribute('id')
        chap   = open ('/tmp/rm_%s.xml' % (id), 'w')
        chap.write  ("<?xml version='1.0' encoding='UTF-8'?>\n")
        chap.write  ('<chapter>\n')
        child = node.firstChild
        while child is not None :
            PrettyPrint (child, chap)
            child = child.nextSibling
        chap.write  ('</chapter>\n')
        chap.close  ()
        os.system   ('xsltproc m_chapter.xsl /tmp/rm_%s.xml > manual/%s/%s.html' % (id, manual, id))

        if db is not None :

            title = findChild (node, 'title')
            ttext = findTitle (title)

            db.cursor().execute \
            (   "insert into manual (chapno, secno, tag, title, body) values (%s, %s, %s, %s, %s)",
                [   chapno,
                    0,
		    '%s.html' % (id),
                    ttext,
                    makeRichText('manual/%s/%s.html' % (manual, id))
                ]
            )

        makeSections(node, index, chapno)

    return index


def mkman () :

    source = open ('index.xml', 'r')
    doc	   = FromXmlStream (source)

    index  = makeChapters (doc)
    main   = makeMainFile (doc, index)

    out	   = open ('manual/rekall.xml', 'w')
    PrettyPrint (main, out)
    out.close   ()


try :
    import MySQLdb
    db = MySQLdb.connect(db = 'RekallWebsite', user = 'website', passwd = 'website')
    db.cursor().execute ("delete from manual") ;
    db.cursor().execute ("delete from __RekallObjects where Type = 'graphic' and Name like 'man_%'") ;
except :
    db = None

#import profile
#profile.run ('mkman()')

mkman()

