/***************************************************************************
    file		 : kb_main_emb.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>

#include	<qpe/qpeapplication.h>
#include	"tkstyle.h"

#include	<qglobal.h>
#include	<qfileinfo.h>
#include	<qlibrary.h>
#include	<qpaintdevice.h>


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_splash.h"
#include	"kb_optionsdlg.h"
#include	"kbase.h"

#include	<qsettings.h>

#if		__KB_RUNTIME
#define		APPNAME		"tkcRekallrt"
#define		APPTITLE	"tkcRekallRT"
#else
#define		APPNAME		"tkcRekall"
#define		APPTITLE	"tkcRekall"
#endif




extern	KBaseApp*kbaseApp	;
extern	bool	__useMDI	;
extern	bool	__useSDI	;
extern	bool	__noNoteDB	;
extern	bool	__noLastDB	;
extern	QString	__initDEB	;
static	bool	__noSplash	;
static	QString	__language	;


QPEApplication	*qpeApp		;

/*  HACK ALERT:								*/
/*									*/
/*  Under QTopia, applications have a specified main widget; if the	*/
/*  user switches to another application and back then the main widget	*/
/*  appears, irrespective of the original top-level widget. This is not	*/
/*  what is wanted for tkcRekall; we want to maintain the original top-	*/
/*  level widget.							*/
/*									*/
/*  So, we need to call QPEApplication::showMainWidget each time that	*/
/*  the user changes it (opens a form, etc). The best way to maintain	*/
/*  it would be to pick up WindowActivated events, but these seem to go	*/
/*  missing. We therefore do it my steam, catching all cases that can	*/
/*  affect the visible widget, and passing control to the following two	*/
/*  routines.								*/


static	QPtrList<QWidget>		windowList	;

/*  setActiveWindow							*/
/*		: Make a specifed widget the active one			*/
/*  widget	: QWidget *	: Widget in question			*/
/*  (returns)	: void		:					*/

void	setActiveWindow
	(	QWidget		*widget
	)
{
	if (widget != 0)
	{
		/* Move the widget to the head of the window list and	*/
		/* set it as the active one.				*/
		fprintf
		(	stderr,
			"setActiveWindow: [%p]\n",
			(void *)widget
		)	;

		windowList.removeRef	(widget) ;
		windowList.prepend	(widget) ;
		qpeApp->showMainWidget	(widget) ;
	}
}

/*  dropActiveWindow							*/
/*		: Drop the currently active window if it closes		*/
/*  widget	: QWidget *	: Widget in question			*/
/*  (returns)	: void		:					*/

void	dropActiveWindow
	(	QWidget		*widget
	)
{
	fprintf
	(	stderr,
		"dropActiveWindow: w=%p a=%p\n",
		(void *)widget,
		(void *)windowList.at(0)
	)	;

	/* First case is just a paranoia check in case we get called	*/
	/* with a null widget pointer.					*/
	if (widget == 0)
	{
		return	;
	}

	/* If the widget is not at the head of the window list then	*/
	/* simply remove it from the list (if indeed it is even there).	*/
	if (widget != windowList.at(0))
	{
		windowList.removeRef (widget) ;
		return ;
	}

	windowList.removeRef (widget) ;

	/* OK, widget is the currently active one. We need to find the	*/
	/* next one down the list. The one now at the head should do,	*/
	/* but we are being paranoid here and we check on the top-level	*/
	/* window list.							*/
	QWidgetList   *tlw = QApplication::topLevelWidgets() ;

	if (tlw != 0)
		while (windowList.count() > 0)
		{
			QWidget	*head = windowList.at(0) ;

			if (tlw->findRef (head) < 0)
			{
				fprintf
				(	stderr,
					"dropActiveWindow: lost [%p], removing ...\n",
					(void *)head
				)	;
				windowList.removeFirst () ;
				continue ;
			}

			/* OK, the head widget is a top-level widget,	*/
			/* use this one.				*/
			head->show		()	;
			head->raise		()	;
			head->setActiveWindow	()	;
			setActiveWindow    	(head)	;
			return	;
		}

	/* Final fall back case, use the main application window.	*/
	if (kbaseApp != 0)
	{
		kbaseApp->show		  () ;
		kbaseApp->raise		  () ;
		kbaseApp->setActiveWindow () ;
		setActiveWindow    	  (kbaseApp) ;
	}
}

/*  rekallMain	: Main routine						*/
/*  argc	: int		: Argument count			*/
/*  argv	: char *[]	: Argument vector			*/
/*  (returns)	: int		: Exit code				*/

int	rekallMain
	(	int	argc,
		char	*argv[],
		AUTH	*authGbl
	)
{
	QPEApplication	app (argc, argv) ;
	qpeApp	= &app	;

	for (int idx = 0 ; idx < argc ; idx += 1)
		fprintf	(stderr, "---->[%s]\n", argv[idx]) ;

	TKConfig	*config	= TKConfig::getConfig () ;
	config->setApplication	(APPNAME     )	;

	int		ptr1	= 1	;
	int		ptr2	= 1	;
	QString		initDB	;

	while (ptr1 < argc)
	{
		if (strcmp (argv[ptr1], "--useMDI"  ) == 0)
		{	__useMDI = true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--useSDI"  ) == 0)
		{	__useSDI = true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--skipNoteDB") == 0)
		{	__noNoteDB=true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--skipLastDB") == 0)
		{	__noLastDB=true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--errLocn" ) == 0)
		{	KBError::errDebug = 1 ;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--errTrace") == 0)
		{	KBError::errDebug = 2 ;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--skipSplash") == 0)
		{	__noSplash = true ;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "-qcop") == 0)
		{
			ptr1	+= 2	;
			continue	;
		}
		argv[ptr2] = argv[ptr1] ;
		ptr1	  += 1 ;
		ptr2	  += 1 ;
	}

	argc	   = ptr2 ;
	argv[argc] = 0	  ;

	if ((argc > 1) && (argv[argc-1][0] != '-')) initDB = argv[argc-1] ;

	extern	void	setAppdataDir (const QString &) ;
	setAppdataDir	(locateDir ("appdata", "gui/rekallui_null.gui")) ;

#if	! __KB_RUNTIME
	KBOptionsDlg::setupWizard (config) ;
#endif	// __KB_RUNTIME

	if (!__noSplash)
	{
		QString	release = RELEASE BUILD ;
		new KBSplash ("rekall", release, 3000, Qt::red) ;
	}



	new KBaseApp (initDB) ;

	Config qpeConfig   ("qpe" ) ;
	qpeConfig.setGroup ("Date") ;

	switch ((DateFormat::Order)qpeConfig.readNumEntry("ShortOrder", DateFormat::DayMonthYear))
	{
		case DateFormat::DayMonthYear :
			fprintf	(stderr, "Date Order: DMY\n") ;
			KBDateTime::setDateOrder (KBDateTime::DayMonthYear) ;
			break	;

		case DateFormat::MonthDayYear :
			fprintf	(stderr, "Date Order: MYY\n") ;
			KBDateTime::setDateOrder (KBDateTime::MonthDayYear) ;
			break	;

		default	:
			fprintf	(stderr, "Date Order: def\n") ;
			KBDateTime::setDateOrder (KBDateTime::DayMonthYear) ;
			break	;
	}


	app.setStyle	   (new TKStyle) ;
	app.setName	   (APPNAME    ) ;

	QWidget	*sw = kbaseApp->showWidget() ;
	::setActiveWindow  (sw) ;


	/* If the database is not runnable then set a single shot timer	*/
	/* to close it. We have to do it this way in the embedded	*/
	/* version since it seems we *have* to close from inside the	*/
	/* event loop, otherwise the icon is left for quick-start.	*/
	if (!kbaseApp->isRunable())
		QTimer::singleShot (0, kbaseApp, SLOT(close())) ;

	return	app.exec() ;
}  
