/***************************************************************************
    file		 : kb_main_kde.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>

#include	<qglobal.h>
#include	<qfileinfo.h>
#include	<qlibrary.h>
#include	<qpaintdevice.h>


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_splash.h"
#include	"kb_optionsdlg.h"
#include	"kb_dprintf.h"
#include	"kbase.h"

#include	"rk_dcopbase.h"


#if		__KB_RUNTIME
#define		APPTITLE	"RekallRT"
#else
#define		APPTITLE	"Rekall"
#endif

#ifdef		__KB_RKLITE
#define		SPLASHIMG	"rekall_lite"
#else
#define		SPLASHIMG	"rekall"
#endif

#include	<klocale.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>

#include	"rk_application.h"


static	const	KCmdLineOptions	options[] =
{
	{	"useMDI",		I18N_NOOP("Run in MDI mode"		),	0	},
	{	"useSDI",		I18N_NOOP("Run in SDI mode"		),	0	},
	{	"skipNoteDB",		I18N_NOOP("Don't remember last database"),	0	},
	{	"skipLastDB",		I18N_NOOP("Don't open last database"	),	0	},
	{	"errLocn",		I18N_NOOP("Show error locations"	),	0	},
	{	"errTrace",		I18N_NOOP("Trace errors"		),	0	},
	{	"errFile <file>",	I18N_NOOP("Write error output to file"	),	0	},
	{	"skipSplash",		I18N_NOOP("Don't show the splash screen"),	0	},
	{	"skipSetup",		I18N_NOOP("Don't show the setup wizard"	),	0	},
	{	"debug <lang>",		I18N_NOOP("Load language debugger"	),	0	},
	{	"language <lang>",	I18N_NOOP("Set KLocale language"	),	0	},
	{	"country <cntry>",	I18N_NOOP("Set KLocale country"		),	0	},
	{	"rkdcop <access>",	I18N_NOOP("Set RKDCOP access"		),	0	},
	{	"+[database]",		I18N_NOOP("Database file"		),	0	},
	{	0,		0,							0	}
}	;


extern	KBaseApp*kbaseApp	;
extern	bool	__useMDI	;
extern	bool	__useSDI	;
extern	bool	__noNoteDB	;
extern	bool	__noLastDB	;
extern	QString	__initDEB	;
static	bool	__noSplash	;
static	bool	__noSetupWiz	;
static	QString	__errFile	;

extern	void	setRootDir	(const QString& _rootdir) ;

/*  setActiveWindow							*/
/*		: Make a specifed widget the active one			*/
/*  widget	: QWidget *	: Widget in question			*/
/*  (returns)	: void		:					*/

void	setActiveWindow
	(	QWidget		*widget
	)
{
	/* Dummy stub under KDE						*/
}

/*  dropActiveWindow							*/
/*		: Drop the currently active window if it closes		*/
/*  widget	: QWidget *	: Widget in question			*/
/*  (returns)	: void		:					*/

void	dropActiveWindow
	(	QWidget		*widget
	)
{
	/* Dummy stub under KDE						*/
}


/*  rekallMain	: Main routine						*/
/*  argc	: int		: Argument count			*/
/*  argv	: char *[]	: Argument vector			*/
/*  (returns)	: int		: Exit code				*/

int	rekallMain
	(	int	argc,
		char	*argv[]
	)
{
	const char *rd	= getenv("REKALL_ROOTDIR") ;
	if (rd != 0) setRootDir (rd) ;

	KAboutData about (__KB_APPNAME,
			  I18N_NOOP(APPTITLE),
			  __KB_BUILD_RELEASE,
			  I18N_NOOP(APPTITLE ": A KDE database front end"),
			  KAboutData::License_Unknown,
			  "(c) Mike Richardson, and John Dean",
			  0,
			  "http://www.rekallrevealed.org",
			  "mike@quaking.demon.co.uk"
			 ) ;
	about.addAuthor  ("Mike Richardson", 	 0, "mike@quaking.demon.co.uk") ; 
	about.addAuthor  ("John Dean", 	     	 0, "john@totalrekall.co.uk") ; 
	about.addAuthor  ("Abel Deuring",      	 0, "") ; 
	about.addAuthor  ("Brian Harring",       0, "") ; 
	about.addAuthor  ("Richard Shaw",        0, "") ; 
	about.addAuthor  ("Tim Southerwood",     0, "") ; 

	KCmdLineArgs::init		(argc, argv, &about) ;
	KCmdLineArgs::addCmdLineOptions	(options) ;

	RKApplication app ;

	TKConfig *config = TKConfig::getConfig() ;
	config->setApplication	(__KB_APPNAME  ) ;
	config->setGroup	("Development" ) ;
	config->writeEntry	("CheckAccelerators", "F12") ;
	config->writeEntry	("AutoCheckAccelerators", "false") ;
	config->writeEntry	("AlwaysShowCheckAccelerators", "false") ;

	about.setLicenseTextFile (locateFile ("appdata", "LICENSE")) ;

	extern	void	loadRekallPlugins () ;
	loadRekallPlugins () ;

#if	__KB_PAGESWITCHLOADER
	extern	void	pageSwitchLoader  () ;
	pageSwitchLoader  () ;
#endif
	if (app.isRestored ())
	{
		RESTORE (KBaseApp(QString())) ;
	}
	else 
	{
		QString		__language	;
		QString		__country	;

		KCmdLineArgs	*args	= KCmdLineArgs::parsedArgs () ;
		cchar		*eumdi	= getenv ("REKALL_USEMDI")    ;
		cchar		*eusdi	= getenv ("REKALL_USESDI")    ;
		QString		initDB	;

		if (args->isSet("useMDI"  ))
			__useMDI = true ;
		else if ((eumdi != 0) && (strcasecmp (eumdi, "YES") == 0))
			__useMDI = true ;

		if (args->isSet("useSDI"  ))
			__useSDI = true ;
		else if ((eusdi != 0) && (strcasecmp (eusdi, "YES") == 0))
			__useSDI = true ;

		if (args->isSet("errLocn" ))
			KBError::errDebug = 1 ;

		if (args->isSet("errTrace"))
			KBError::errDebug = 2 ;

		if (args->isSet("skipNoteDB"))
			__noNoteDB = true ;

		if (args->isSet("skipLastDB"))
			__noLastDB = true ;

		if (args->isSet("skipSplash"))
			__noSplash = true ;

		if (args->isSet("skipSetup"))
			__noSetupWiz = true ;

		if (args->isSet("rkdcop"))
			RKDCOPBase::setAccess
			(	args->getOption("rkdcop").toInt(),
				args->getOption("rkdcop")[0] == '+'
			)	;

		__initDEB  = args->getOption ("debug"   ) ;
		__language = args->getOption ("language") ;
		__country  = args->getOption ("country" ) ;
		__errFile  = args->getOption ("errFile" ) ;

		if (!__errFile.isEmpty())
			kbDPrintfSetStream (__errFile.latin1()) ;

		if (args->count() > 0)
			initDB	= args->url(0).path() ;

#if	! __KB_RUNTIME
		if (!__noSetupWiz)
			KBOptionsDlg::setupWizard (config) ;
#endif
		if (!__language.isNull())
			if (!KGlobal::locale()->setLanguage (__language))
				fprintf
				(	stderr,
					"Failed to set language [%s]\n",
					(cchar *)__language
				)	;
		if (!__country .isNull())
			if (!KGlobal::locale()->setCountry  (__country ))
				fprintf
				(	stderr,
					"Failed to set country [%s]\n",
					(cchar *)__country
				)	;

		if (!__noSplash)
			showSlashScreen (SPLASHIMG, 7000, Qt::white) ;

		new KBaseApp (initDB) ;
	}

	KApplication::dcopClient () ;

	/* Check if rekall is runnable. This handles things like the	*/
	/* run-time version with nothing to open up.			*/
	if (!kbaseApp->isRunable())
		return	1 ;

	return	app.exec() ;
}  

