/***************************************************************************
    file	         : kb_locator_win.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


/*  ------------------------------------------------------------------  */
/*  QT3/Win32								*/
/*  ------------------------------------------------------------------  */


#include	"libcommon_exports.h"

#if		__KB_TKC && defined(Q_OS_WIN32)

#include	<qstring.h>
#include 	<qdir.h>
#include	<qfileinfo.h>

#include	"kb_classes.h"
#include	"tk_messagebox.h"

#define		__DEBUG_LOCATOR		(0)
#if		__DEBUG_LOCATOR	
#include 	"kb_dprintf.h"
#endif

#ifndef		__KB_COMMONDIR
#define		__KB_COMMONDIR	"\\common"
#endif

static	QString instdir = QDir::convertSeparators(QDir::currentDirPath()) ;
static	QString rootdir = QDir::convertSeparators(QDir::currentDirPath() + __KB_COMMONDIR) ;

/*  locateFile	: Locate a file of specified type			*/
/*  type	: const char *	  : File type				*/
/*  name	: const QString & : File name				*/
/*  (returns)	: QString	  : Path				*/

LIBCOMMON_API
	QString	locateFile
	(	const char	*type,
		const QString	&name
	)
{
	DPRINTF
	((	"locateFile: %s: %s (with %s)\n",
		type,
		name   .latin1(),
		rootdir.latin1()
	))	;

	QString	path	;

	if	(qstrcmp (type, "appdata") == 0) path = rootdir + __KB_COMMONDIR "\\data\\" + name ;
	else if (qstrcmp (type, "doc"    ) == 0) path = rootdir + __KB_COMMONDIR "\\doc\\"  + name ;

	if (path != 0)
		if (!QFileInfo(path).exists())
			path	= QString::null ;

	return	path	;
}

/*  locateDir	: Locate a directory of specified type			*/
/*  type	: const char *	  : Directory type			*/
/*  name	: const QString & : Directory name			*/
/*  (returns)	: QString	  : Path				*/

LIBCOMMON_API
	QString	locateDir
	(	const char	*type,
		const QString	&
	)
{
	/* The name is never used here. The argument exists since under	*/
	/* KDE there seems to be no way to locate an application	*/
	/* directory without naming a file that will be found in it.	*/
	return	locateFile   (type, "") ;
}

/*  setRootDir	: Set different root directory				*/
/*  _rootdir	: const QString & : New root directory name		*/
/*  (returns)	: void		  :					*/

LIBCOMMON_API
	void	setRootDir
	(	const QString	&_rootdir
	)
{
	DPRINTF
	((	"setRootDir(%s)\n",
		_rootdir.latin1()
	))	;

	instdir = _rootdir ;
	rootdir	= _rootdir ;

	if (locateDir ("appdata", "services").isNull())
		TKMessageBox::sorry
		(	0,
			"Rekall has set a root directory that does not "
			"appear to have the right contents. Rekall will "
			"probably not run",
			"Root Directory Error"
		)	;
}

/*  getRootDir	: Get root directory					*/
/*  (returns)	: QString & 	: Directory name			*/

LIBCOMMON_API
	QString	getRootDir ()
{
	return	rootdir	;
}

/*  getInstDir	: Get installation directory				*/
/*  (returns)	: const QString & : Directory name			*/

LIBCOMMON_API
	QString	getInstDir ()
{
	return	instdir	;
}

#endif
