/***************************************************************************
    file	         : kb_methdict.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qdict.h>
#include	<qdir.h>
#include	<qregexp.h>
#include	<qmessagebox.h>


#include	"kb_classes.h"
#include	"kb_methdict.h"
#include	"kb_locator.h"
#include	"kb_dom.h"

#define		__DEBUG_METHDICT	(1)
#if		__DEBUG_METHDICT
#include	"kb_dprintf.h"
#endif

#if		__KB_KDE

#include	<klocale.h>
#include	<kglobal.h>

#endif		// __KB_KDE


KBMethDictArg::KBMethDictArg
	(	const QDomElement	&elem
	)
{
	m_name		= elem.attribute("name"   ) ;
	m_type		= elem.attribute("type"   ) ;
	m_comment	= elem.attribute("comment") ;
	m_opt		= elem.attribute("opt"	  ) == "Yes" ;

	DPRINTF
	((	"KBMethDictArg::KBMethDictArg: for [%s]\n",
		m_name.latin1()
	))	;
}

/*  ------------------------------------------------------------------	*/

QString	KBMethDictEntry::asText
	(	bool		withReturn
	)
{
	QString	 d ;

	if (withReturn)
	{
		d.append (m_return) ;
		d.append (" ") ;
	}

	d.append (m_name) ;
	d.append ("(") ;

	int optcnt = 0 ;
	for (uint idx = 0 ; idx < m_args.count() ; idx += 1)
	{
		const KBMethDictArg &arg = m_args[idx] ;
		d.append (idx > 0 ? ", " : "") ;
		if (arg.m_opt)
		{	d.append ("[")	;
			optcnt += 1	;
		}
		d.append (arg.m_type)	;
		d.append (" ")		;
		d.append (arg.m_name)	;
	}
	while (optcnt > 0)
	{	d.append ("]")	;
		optcnt -= 1	;
	}
	d.append (")")	;
	return	 d	;		

}

QString	KBMethDictEntry::description ()
{
	return	asText (true ) ;
}

QString	KBMethDictEntry::substitute ()
{
	return	asText (false) ;
}

/*  ------------------------------------------------------------------	*/

QPtrList<KBMethDictEntry>
	KBMethDict::getMethods
	(	const QString	&method
	)
{
	QPtrList<KBMethDictEntry>	matched	;
	QRegExp			pattern	(method) ;

	for
	(	KBMethDictEntry	*entry = m_entries.first() ;
				entry != 0 ;
				entry  = m_entries.next()
	)
	{
		DPRINTF
		((	"KBMethDict::getMethods: [%s][%s]\n",
			(cchar *)(method),
			(cchar *)(entry->m_class + "::" + entry->m_name)
		))	;
		if (pattern.exactMatch(entry->m_class + "::" + entry->m_name))
			matched.append (entry) ;
	}

	DPRINTF
	((	"KBMethDict::getMethods: matched %d\n",
		matched.count()
	))	;

	return	matched	;
}
	
/*  KBMethDict								*/
/*  KBMethDict	: Dictionary constructor				*/
/*  dName	: QString	: Dictionary name			*/
/*  (returns)	: KBMethDict	:					*/

KBMethDict::KBMethDict
	(	QString		dName
	)
{
	QString	dPath	;
	QDir	dDir	;

	dPath	= locateDir ("appdata", QString("dict/%1.dict") .arg(dName)) ;
	dPath  += "/dict";

	DPRINTF
	((	"KBMethDict::KBMethDict: [%s] -> [%s]\n",
		(cchar *)dName,
		(cchar *)dPath
	))	;

	dDir.setPath	   (dPath) ;
	dDir.setNameFilter (QString("%1*.dict").arg(dName)) ;
	dDir.setFilter     (QDir::Files|QDir::Readable) ;
	dDir.setSorting    (QDir::Name) ;

	const QFileInfoList   *info = dDir.entryInfoList() ;
	if (info == 0)
	{
		QMessageBox::warning(0, "Location Error", "No dictionary directory found!!\n");
		return	;
	}

	QFileInfoListIterator dIter (*info) ;
	QFileInfo *fi ;

#if	__KB_KDE
	KLocale	*locale	= KGlobal::locale () ;
	DPRINTF
	((	"KBMethDict::KBMethDict: [%s] language [%s]\n",
		(cchar *)dName,
		(cchar *)locale->language()
	))	;
#endif

	while ((fi = dIter.current()) != 0)
	{
		QString	name	= fi->filePath() ; 
		loadFile (name)	;

#if	__KB_KDE
		QString	name2	= fi->dirPath () + "/" + locale->language() + "/" + fi->fileName() ;
		if (QFileInfo(name2).exists()) loadFile (name2) ;
#endif
		dIter += 1 ;
	}
}

/*  KBMethDict								*/
/*  loadFile	: Load a dictionary file				*/
/*  name	: const QString & : File name				*/
/*  (returns)	: void		  :					*/

void	KBMethDict::loadFile
	(	const QString	&name
	)
{
	DPRINTF
	((	"KBMethDict::loadFile: [%s]\n",
		(cchar *)name
	))	;

	KBDomDocument	dXML	;
	if (!dXML.loadFile (name, 0, 0))
	{
		dXML.lastError().DISPLAY() ;
		return	;
	}

	DOMITER_BEGIN(dXML.documentElement(), "method", method)
	{
		KBMethDictEntry *entry = new KBMethDictEntry   ;

		entry->m_class   = method.attribute("class"  ) ;
		entry->m_name	 = method.attribute("name"   ) ;
		entry->m_comment = method.attribute("comment") ;

		DPRINTF
		((	"KBMethDict::loadFile: for [%s.%s]\n",
			(cchar *)entry->m_class,
			(cchar *)entry->m_name
		))	;

		DOMITER_BEGIN(method, "return", rtn)
		{
			entry->m_return = rtn.attribute("type") ;
		}
		DOMITER_END
		DOMITER_BEGIN(method, "arg",    arg)
		{
			entry->m_args.append (KBMethDictArg(arg)) ;
		}
		DOMITER_END

		m_entries.append (entry) ;
	}
	DOMITER_END
}
