#include	"rk_widget.moc"



/*  RKWidget								*/
/*  RKWidget	: RKWidget constructor					*/
/*  gtkWidget	: GtkWidget *	: Underlying GTK widget			*/
/*  parent	: RKWidget *	: Parent widget				*/
/*  name	: const char *	: Optional name				*/
/*  flags	: WFlags	: Flags					*/
/*  (returns)	: QWidget	:					*/

RKWidget::RKWidget
	(	GtkWidget	*gtkWidget,
		RKWidget	*parent,
		const char	*name,
		WFlags		flags
	)
	:
	QObject		(parent, name),
	m_gtkWidget	(gtkWidget),
	m_parent	(parent),
	m_flags		(flags)
{
	m_minWidth	= -1 ;
	m_minHeight	= -1 ;

	/* If there is a parent widget then add the new GTK widget into	*/
	/* the parent containter.					*/
	if (m_parent != 0)
		m_parent->gtkInsertChild (m_gtkWidget) ;

	g_object_set_data (G_OBJECT(m_gtkWidget), "RKWidget", this) ;

	g_signal_connect
	(	G_OBJECT(m_gtkWidget),
		"destroy",
		G_CALLBACK(gtk_widget_destroyed),
		&m_gtkWidget
	)	;

	show	() ;
}


RKWidget::RKWidget
	(	RKWidget	*parent,
		const char	*name,
		WFlags		flags
	)
	:
	QObject		(parent, name),
	m_gtkWidget	(gtk_label_new("")),
	m_parent	(parent),
	m_flags		(flags)
{
	m_minWidth	= -1 ;
	m_minHeight	= -1 ;

	/* If there is a parent widget then add the new GTK widget into	*/
	/* the parent containter.					*/
	if (m_parent != 0)
		m_parent->gtkInsertChild (m_gtkWidget) ;

	g_object_set_data (G_OBJECT(m_gtkWidget), "RKWidget", this) ;

	g_signal_connect
	(	G_OBJECT(m_gtkWidget),
		"destroy",
		G_CALLBACK(gtk_widget_destroyed),
		&m_gtkWidget
	)	;

	show	() ;
}


RKWidget::~RKWidget ()
{
	if (m_gtkWidget != 0)
		gtk_widget_destroy (m_gtkWidget) ;
}

void	RKWidget::show ()
{
	gtk_widget_show (m_gtkWidget) ;
}

void	RKWidget::hide ()
{
	gtk_widget_hide (m_gtkWidget) ;
}

void	RKWidget::setEnabled
	(	bool		enabled
	)
{
	gtk_widget_set_sensitive (m_gtkWidget, enabled) ;
}

void	RKWidget::setMinimumWidth
	(	int		minWidth
	)
{
	setMinimumSize (minWidth, m_minHeight) ;
}

void	RKWidget::setMinimumHeight
	(	int		minHeight
	)
{
	setMinimumSize (m_minWidth, minHeight) ;
}

void	RKWidget::setMinimumSize
	(	int		minWidth,
		int		minHeight
	)
{
	m_minWidth  = minWidth  ;
	m_minHeight = minHeight ;
	gtk_widget_set_usize  (m_gtkWidget, m_minWidth, m_minHeight) ;
}


void	RKWidget::reparent
	(	RKWidget	*child,
		WFlags		flags,
		const QPoint	&at,
		bool		showIt
	)
{
	if (child->parentWidget() != 0)
		 gtk_container_remove (GTK_CONTAINER(child->parentWidget()->gtkWidget()), child->gtkWidget()) ;

	gtkInsertChild (child->gtkWidget()) ;
}

void	RKWidget::reparent
	(	RKWidget	*child,
		const QPoint	&at,
		bool		showIt
	)
{
	reparent (child, 0, at, showIt) ;
}

void	RKWidget::setPalette
	(	const QPalette	&pal
	)
{
}

void	RKWidget::gtkInsertChild
	(	GtkWidget	*
	)
{
	fprintf
	(	stderr,
		"RKWidget::gtkInsertChild: attempt to insert into raw widget\n"
	)	;
}
