/***************************************************************************
    file	         : main_kde.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qapplication.h>
#include	<qstringlist.h>
#include	<qsettings.h>
#include	<qdir.h>
#include	<qguardedptr.h>

#include	<kapp.h>
#include	<klocale.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>

#include	"viewer.h"
#include	"tk_config.h"
#include	"kb_locator.h"

#include	"images.cpp"

QStringList	xmlFiles	;
QString		mwHelpDir	;
QString		mwHelpKey	;
uint		mwHelpPort	;

static	const	KCmdLineOptions	options[] =
{
{	"helpdir <helpdir>",	I18N_NOOP("Help files directory"),	0	},
{	"helpkey <key>",	I18N_NOOP("Initial help key"),		0	},
{	"port <port>",		I18N_NOOP("Communications port"),	0	},
{	0,			0,						0	}
}	;

int	main
	(	int	argc,
		char	**argv
	)
{
	KAboutData about
		   (	__KB_APPNAME "Manual",
			"Rekall Manual",
			"0.1",
			I18N_NOOP("Rekall Manual" ": Rekall Manual Viewer"),
			KAboutData::License_GPL,
			"(c) Mike Richardson, and John Dean",
			0,
			"http://www.rekallrevealed.org",
			"mike@quaking.demon.co.uk"
		   )	;
	about.addAuthor  ("Mike Richardson", 	 0, "mike@quaking.demon.co.uk") ; 
	about.addAuthor  ("John Dean", 	     	 0, "john@totalrekall.co.uk") ; 

	KCmdLineArgs::init		(argc, argv, &about) ;
	KCmdLineArgs::addCmdLineOptions	(options) ;

	KApplication app ;

	KCmdLineArgs *a	= KCmdLineArgs::parsedArgs () ;
	mwHelpDir	= a->getOption ("helpdir") ;
	mwHelpKey	= a->getOption ("helpkey") ;
	mwHelpPort	= a->getOption ("port"   ).toInt() ;

	/* HACK ALERT ....						*/
	/*								*/
	if (mwHelpDir.isEmpty())
	{
		const char *paths [] =
		{
			"/usr/share/apps/"       __KB_APPNAME "/manual/",
			"/opt/kde3/share/apps/"  __KB_APPNAME "/manual/",
			"/usr/local/share/apps/" __KB_APPNAME "/manual/",
			0
		}	;
		for (const char **p = &paths[0] ; *p != 0 ; p += 1)
			if (QDir(*p).exists())
			{	mwHelpDir = *p ;
				break	;
			}
	}

	fprintf
	(	stderr,
		"FTHelp: cwd     =%s\n"
		"        helpdir =%s\n"
		"        helpkey =%s\n"
		"        helpport=%d\n",
		QDir::current().absPath().latin1(),
		mwHelpDir.latin1(),
		mwHelpKey.latin1(),
		mwHelpPort
	)	;

	QDir		dir	(mwHelpDir) ;
	QStringList	xml	= dir.entryList("*.xml", QDir::Files, QDir::Name) ;

	for (unsigned int idx = 0 ; idx < xml.count() ; idx += 1)
		xmlFiles.append (dir.absFilePath(xml[idx])) ;

	QMimeSourceFactory::defaultFactory()->addFactory (factory) ;

	QGuardedPtr<SOHelpViewer> mw  = new SOHelpViewer
					(	mwHelpPort,
						0,
						about.programName(),
						Qt::WDestructiveClose
					)	;

	TKConfig *config = TKConfig::getConfig () ;

	if (config->readBoolEntry ("Maximized", false))
		mw->showMaximized() ;
	else	mw->show	 () ;

	qApp->processEvents	 () ;

	if (!mw) return 0  ;

	if (!mwHelpKey.isEmpty())
		mw->setHelpKey (mwHelpKey) ;

	app.connect (&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit())) ;

	return	app.exec() ;
}
