import	os
import	sys
import	re
import	string
import	types
import	time

import	rkConfig

from	SCons.Builder		import	Builder
from	SCons.Environment	import	Environment



buildQT3		= 1		# Linux QT3-Only
buildKDE		= 2		# Linux KDE3
buildGTK		= 3		# Linux GTK (development :)
buildWIN		= 5		# Win32
buildMINGW		= 6		# Win32 cross-compile
buildMAC		= 7		# Mac OS/X
#buildSet		= 999

typeSharedLib		= 1		# Linked shared library (.so/.dll)
typeLoadedLib		= 2		# Dynamically loaded library (.so/.dll)
typeProgram		= 3		# Normal program
typePythonLib		= 4		# Python dynamic library
typeConsole		= 5		# Console program under windows



#  ------------------------------------------------------------------
#
#  Stuff used by windows to generate program .rc and .ico files
#  automatically. See code in rkBuilder.build()
#
def rcBuilderFN  (target, source, env) :
    bits = string.split (str(target[0]), '\\')
    name = bits[-1]
    skel = open("support/windows/skel.rc" ).read()
    skel = string.replace (skel, 'skel.ico', name[:-3] + '.ico')
    open(str(target[0]),"w").write(skel)

def icoBuilderFN (target, source, env) :
    open(str(target[0]),"w").write(open("support/windows/skel.ico").read())


rcBuilder  = Builder (action = rcBuilderFN,  suffix = '.rc',  src_suffix = '.rc' )
icoBuilder = Builder (action = icoBuilderFN, suffix = '.ico', src_suffix = '.ico')


#  ------------------------------------------------------------------
#
#  Builder used to handle library versioning for Linux (ie., libname.so.N).
#  This generates the required symlink.
#
def soBuilderFN (target, source, env) :
    srcbits = string.split(str(source[0]), '/')
    os.symlink (srcbits[-1], str(target[0]))

soBuilder  = Builder (action = soBuilderFN,  suffix = '.so',  src_suffix = '.so.' + rkConfig.libver)


def laBuilderFN (target, source, env) :

    tgtname	= str(target[0])
    srcname	= str(source[0])

    m = __la_mapper.match(srcname)
    if m : srcname = m.group(1) + '.so'

    if verbose() :
        sys.stderr.write ("Generating libtool .la file for " + srcname + "\n")

    l = open (tgtname, 'w')
    l.write  ("dlname='%s.%s'" % (srcname, rkConfig.libver) + "\n")
    l.write  ("library_names='%s.%s %s'" % (srcname, rkConfig.libver, srcname) + "\n")
    l.write  ("old_library=''" + "\n")
    l.write  ("dependency_libs=''\n")
    l.write  ("dlopen=''\n")
    l.write  ("dlpreopen=''\n")
    l.write  ("libdir='%s'\n" % (rkConfig.kde3LibDir))
    l.close  ()
    return   None


def la_builder () :

    global __la_builder
    if __la_builder is None :
        __la_builder  = Builder (action = laBuilderFN, suffix = '.la', src_suffix = '.so')
    return __la_builder


#  ------------------------------------------------------------------


def toList (s) :

    if type(s) != types.ListType :
        return [ s ]
    return s



#  ------------------------------------------------------------------


try    : destDir	= os.environ['DESTDIR'] + '/'
except : destDir 	= ''

if rkConfig.mode in [ 'win', 'mingw' ] :
    instDir		= os.path.join (os.getcwd(), 'RekallWin')
    binDir		= destDir + instDir
    libDir		= destDir + instDir
    pyLibDir		= destDir + instDir
    pluginDir		= destDir + instDir
    appDir		= destDir + os.path.join (instDir, 'data')
    rekallServicesDir	= os.path.join (appDir,  'services')
    rekallGUIDir	= os.path.join (appDir,  'gui')

if rkConfig.mode in [ 'mac' ] :
    instDir		= rkConfig.appName + '.app'
    binDir		= destDir + os.path.join (instDir, 'Contents/MacOS')
    libDir		= destDir + os.path.join (instDir, 'Contents/Frameworks')
    pyLibDir		= destDir + os.path.join (instDir, 'Contents/Frameworks')
    pluginDir		= destDir + os.path.join (instDir, 'Contents/Plugins')
    appDir		= destDir + os.path.join (instDir, 'Contents/Resources/Data')
    rekallServicesDir	= os.path.join (appDir,  'services')
    rekallGUIDir	= os.path.join (appDir,  'gui')

if rkConfig.mode in [ 'qt3' ] :
    instDir		= rkConfig.prefix
    binDir		= destDir + os.path.join (instDir, 'bin')
    libDir		= destDir + os.path.join (instDir, 'lib')
    pyLibDir		= destDir + os.path.join (rkConfig.pythonLibDir, rkConfig.pythonLib, 'site-packages')
    pluginDir		= destDir + os.path.join (instDir, 'lib')
    appDir		= destDir + os.path.join (instDir, 'share', 'apps', rkConfig.appName)
    rekallServicesDir	= os.path.join (appDir,  'services')
    rekallGUIDir	= os.path.join (appDir,  'gui')

if rkConfig.mode in [ 'kde' ] :
    instDir		= rkConfig.prefix
    binDir		= destDir + os.path.join (instDir, 'bin')
    libDir		= destDir + os.path.join (instDir, 'lib')
    pyLibDir		= destDir + os.path.join (rkConfig.pythonLibDir, rkConfig.pythonLib, 'site-packages')
    pluginDir		= destDir + os.path.join (instDir, 'lib')
    appDir		= destDir + os.path.join (instDir, 'share', 'apps', rkConfig.appName)
    rekallServicesDir	= os.path.join (appDir,  'services')
    rekallGUIDir	= os.path.join (appDir,  'gui')

#  ------------------------------------------------------------------

__installer		= None
__la_builder		= None
__la_mapper		= re.compile (r'.*/([^/]+)\.so')

__install		= 0
__arguments		= {}
__argsSet		= 0

#  ------------------------------------------------------------------


def setCommandLine (targets, arguments) :

    global __install
    global __arguments
    global __argsSet

    if not __argsSet :
        if 'install' in targets : __install = 1
        __arguments = arguments
        __argsSet   = 1


def argument (name) :

    global __arguments
    try    : return __arguments[name]
    except : return None


def verbose () :

    return True
    try    : return __arguments['verbose']
    except : return None




def installer () :

    global __installer
    if __install and __installer is None :
        import SCons.Environment
        __installer = SCons.Environment.Environment()
    return __installer




def fixSourcePath (dir) :

    return os.path.join(rkConfig.topDir, 'build', rkConfig.mode, dir)


#  getSourcePath
#  -------------
#  When doing variant builds, there does not seem to be a way to find
#  the original source directory, so we get the current directory and
#  zap the variant-build-specific bit.
#
def getSourcePath () :

    import os
    path = os.getcwd()
    path = string.replace(path, os.path.join ('build', rkConfig.mode), '')
    path = string.replace(path, '_rt', '')
    return path


def isRooted (path) :

    try :
        if path[0] == '/'  : return True
        if path[0] == '\\' : return True
        if path[1] == ':'  : return True
    except :
        pass

    return False
    

# listAllMatching
# ---------------
# List all files matching a pattern.
#
def listAllMatching (pattern) :

    files = []
    for file in os.listdir(getSourcePath()) :
        if re.match(pattern, file) :
            files.append (file)
    return files


# listAllCPP
# ----------
# Most of the time we build all .cpp files in a directory, so use this
# method when we are passed an empty list of source files.
#
def listAllCPP () :

    cpp = []
    for file in os.listdir(getSourcePath()) :
        if file[-4:] == '.cpp' :
            if file[:2] == '.#' :
                print "Warning: skipping dead CVS file:", file
                continue
            if file[:4] == 'moc_' :
                continue
            cpp.append (file)

    return cpp


# listAllH
# --------
# List all the header files in a directory.
#
def listAllH (allcode) :

    h = []
    for file in os.listdir(getSourcePath()) :
        if file[-2:] == '.h' :
            if file[:2] == '.#' :
                print "Warning: skipping dead CVS file:", file
                continue
            if allcode is not None and string.find (allcode, file) < 0 :
                continue
            h.append (file)

    return h


#  ------------------------------------------------------------------
#
#  rkBuilder
#  ---------
#  Main class for the Rekall build system.
#
class rkBuilder :

    global listAllH
    global listAllCPP

    # init
    # ----
    # Constructor for rekall builder object. Target is the normal target
    # name; type indicates what sort of thing is being build.
    #
    def __init__ (self, target, type = typeSharedLib) :

        self.m_defines  = {}
        self.m_cflags	= []
        self.m_includes = []
        self.m_libpath  = []
        self.m_libs     = []
        self.m_lflags   = []
        self.m_depends  = []
        self.m_fixed    = 0

        self.m_prefix	= None # prefix
	self.m_target	= target
	self.m_type	= type
        self.m_build    = None
        self.m_env	= None
        self.m_dir	= os.getcwd()

        self.m_rt       = ''

        self.m_includes.append ('.')

        if rkConfig.mode == 'kde'   : self.setupKDE   ()
        if rkConfig.mode == 'qt3'   : self.setupQT3   ()
        if rkConfig.mode == 'gtk'   : self.setupGTK   ()
        if rkConfig.mode == 'win'   : self.setupWIN   ()
        if rkConfig.mode == 'mingw' : self.setupMINGW ()
        if rkConfig.mode == 'mac'   : self.setupMAC   ()


    # setupWIN
    # --------
    # Set up for a WIN build. Flag that this is the case and set the
    # __KB_KDE and __KB_TKC flags as appropriate
    #
    def setupWIN (self) :

        self.m_build	= buildWIN
        self.setDefine ('__KB_KDE',	 	'0')
        self.setDefine ('__KB_TKC',	 	'1')
        self.setDefine ('__KB_AUTOMOC',  	'1')
        self.setDefine ('__KB_PLAT_WIN32',  	'1')

        self.setDefine ('__WIN32__',		'' )
        self.setDefine ('_WIN32',		'' )

    # setupMINGW
    # ----------
    # Set up for a MinGW cross-compile build. Flag that this is the case
    # and set the __KB_KDE and __KB_TKC flags as appropriate
    #
    def setupMINGW (self) :

        self.m_build	= buildMINGW
        self.setDefine ('__KB_KDE',	 	'0')
        self.setDefine ('__KB_TKC',	 	'1')
        self.setDefine ('__KB_AUTOMOC',  	'1')
        self.setDefine ('__KB_PLAT_WIN32',	'1')

        if self.m_type == typeSharedLib :
            self.addLFlags ('-Wl,--out-implib,%s/%s.a' % (self.m_dir, self.target()))


    # setupQT3
    # --------
    # Set up for a QT3 build. Flag that this is the case and set the
    # __KB_KDE and __KB_TKC flags as appropriate
    #
    def setupQT3  (self) :

        self.m_build	= buildQT3
        self.setDefine ('__KB_KDE',	 	'0')
        self.setDefine ('__KB_TKC',	 	'1')
        self.setDefine ('__KB_AUTOMOC',  	'1')
        self.setDefine ('__KB_PLAT_LINUX',	'1')


    # setupKDE
    # --------
    # Set up for a KDE build. Flag that this is the case and set the
    # __KB_KDE and __KB_TKC flags as appropriate
    #
    def setupKDE (self) :

        self.m_build	= buildKDE
        self.setDefine ('__KB_KDE',	 	'1')
        self.setDefine ('__KB_TKC',	 	'0')
        self.setDefine ('__KB_AUTOMOC', 	'1')
        self.setDefine ('__KB_PLAT_LINUX',	'1')


    # setupGTK
    # --------
    # Set up for a GTK build. Flag that this is the case and set the
    # __KB_KDE and __KB_TKC flags as appropriate
    #
    def setupGTK (self) :

        self.m_build	= buildGTK
        self.setDefine ('__KB_KDE',		'0')
        self.setDefine ('__KB_TKC',		'1')
        self.setDefine ('__KB_GTK',		'1')
        self.setDefine ('__KB_PLAT_LINUX',	'1')


    # setupMAC
    # --------
    # Set up for a MAC OS/X build. Flag that this is the case and set the
    # __KB_KDE and __KB_TKC flags as appropriate
    #
    def setupMAC (self) :

        self.m_build	= buildMAC
        self.setDefine ('__KB_KDE',		'0')
        self.setDefine ('__KB_TKC',		'1')
        self.setDefine ('__KB_AUTOMOC',		'1')
        self.setDefine ('__KB_PLAT_MACX',	'1')


    def setBuildStamp (self) :

        self.setDefine ('__KB_BUILD_TIME',    '\\"' + rkConfig.buildTime + '\\"')
        self.setDefine ('__KB_BUILD_RELEASE', '\\"' + rkConfig.release + '\\"')
        self.setDefine ('__KB_BUILD_BUILD',   '\\"' + rkConfig.build   + '\\"')


    # setRuntime
    # ----------
    # Set as a runtime build
    #
    def setRuntime (self) :

        self.m_rt = 'rt'


    # fixup
    # -----
    # This method is called before returning any settings, and adds stuff
    # like includes and libraries. This is done as late as possible so
    # that the "system" includes and libraries appear last.
    #
    def fixup (self) :

        if self.m_fixed :
            return

        try :
            if rkConfig.restrict['RKLITE'] :
                self.setDefine ('__KB_RKLITE', '1')
        except :
            pass

        self.addInclude(rkConfig.topDir   )
        self.addInclude(rkConfig.qt3IncDir)
        self.addLibPath(rkConfig.qt3LibDir)
        self.addLib    (rkConfig.qt3LibLib)
	self.setDefine ('__KB_APPNAME', '\\"' + rkConfig.appName + self.m_rt + '\\"')

        if self.m_type == typeLoadedLib :
            if self.m_build in [ buildKDE, buildQT3 ] :
                self.addLFlags ('-Wl,-soname,' + self.target() + '.so.' + rkConfig.libver)
            self.setDefine ('__KB_INITNAME', 'init_' + self.target())

        if self.m_type == typeSharedLib :
            if self.m_build in [ buildKDE, buildQT3 ] :
                self.addLFlags ('-Wl,-soname,' + self.target() + '.so.' + rkConfig.libver)

        if self.m_build not in [ buildMAC, buildWIN ] :
            self.addLFlags ('-Wl,--no-undefined')
            self.addCFlags ('-Wall')

        if self.m_build in [ buildMAC ] :
            if self.m_type == typeSharedLib :
                self.addLFlags ('-Wl,-install_name,@executable_path/../Frameworks/%s.dylib' % (self.target()))
            if self.m_type == typeLoadedLib :
                self.addLFlags ('-Wl,-install_name,@executable_path/../Plugins/%s.dylib'    % (self.target()))

        if self.m_build in [ buildKDE ] :
            self.addInclude(rkConfig.kde3IncDir)
            self.addLibPath(rkConfig.kde3LibDir)
            self.addLib    ('kparts')
            self.addLib    ('kdeui')
            self.addLib    ('kdecore')
            self.addLib    ('kio')
            self.addLib    ('kdeprint')
            self.addLib    ('DCOP')
            self.setDefine ('__KB_KDEVERSION', rkConfig.kdeverno)

        if self.m_build in [ buildGTK ] :
            self.addInclude(rkConfig.glibIncDir1)
            self.addInclude(rkConfig.glibIncDir2)
            self.addInclude(rkConfig.gtkIncDir  )
            self.addInclude(rkConfig.gdkIncDir  )
            self.addInclude(rkConfig.pangoIncDir)
            self.addInclude(rkConfig.atkIncDir  )
            self.addLib    ('gtk-x11-2.0')
            self.addLib    ('gobject-2.0')
            self.addLib    ('glib-2.0')

        if self.m_build in [ buildQT3, buildGTK, buildMAC ] :
	    self.setDefine ('ROOTDIR', '"\\"%s"\\"' % (rkConfig.prefix))

        if self.m_build in [ buildWIN ] :
	    self.setDefine ('ROOTDIR', '"\\"%s"\\"' % (rkConfig.prefix))
            self.addCFlags ('/MD')
            self.addCFlags ('/EHsc')
            self.setDefine ('QT_DLL')
            self.setDefine ('QT_THREAD_SUPPORT')
            self.setDefine ('QT_NO_DEBUG')
            self.setDefine ('UNICODE')
            self.addLFlags ('/INCREMENTAL:NO')
            if self.m_type == typeProgram :
                self.addLFlags  ('/FORCE:MULTIPLE')
                self.addLFlags  ('/SUBSYSTEM:WINDOWS')
            if self.m_type == typeConsole :
                self.addLFlags  ('/FORCE:MULTIPLE')
                self.addLFlags  ('/SUBSYSTEM:CONSOLE')

        if self.m_build in [ buildMINGW ] :
	    self.setDefine ('ROOTDIR', '"\\"%s"\\"' % (rkConfig.prefix))
            self.setDefine ('UNICODE')
            self.setDefine ('QT_DLL')
            self.setDefine ('QT_THREAD_SUPPORT')
            self.addCFlags ('-fno-exceptions')
            self.addCFlags ('-fno-rtti')
            self.addLFlags ('-Wl,-subsystem,windows')
            self.addLib    ('qt-mt')
            self.addLib    ('qtmain')

        if self.m_rt != '' :
               self.setDefine ('__KB_RUNTIME', '1')
        else : self.setDefine ('__KB_RUNTIME', '0')

        if rkConfig.debug :
            if self.m_build in [ buildWIN ] :
                self.addCFlags ('/Yd')
                self.addCFlags ('/Zi')
                self.addCFlags ('/GZ')
                self.addLFlags ('/DEBUG')
            else :
                self.addCFlags ('-g')
                self.addLFlags ('-g')

        if rkConfig.dprintf :
               self.setDefine ('__KB_DPRINTF', '1')
        else : self.setDefine ('__KB_DPRINTF', '0')

        try :
            for line in open(rkConfig.topDir + '/scons/extra.defines').readlines() :
                line = string.replace(line, '\n', '')
                if line[0] in [ '#', ';' ] :
                    continue
                bits = string.split  (line, '=')
                if len(bits) == 2 :
                    self.setDefine (bits[0], bits[1])
        except :
            pass

#       self.m_build = buildSet
        self.m_fixed = 1


    # target
    # ------
    # Return the target name
    #
    def target (self) :

        t = self.m_target

        if self.m_type == typeLoadedLib or self.m_type == typeSharedLib :
            if self.m_prefix is None :
                if t == '' :
                       t = 'lib' + rkConfig.libPrefix + self.m_rt
                else : t = 'lib' + rkConfig.libPrefix + self.m_rt + '_' + t
            else :
                t = 'lib' + t

        if self.m_type == typeProgram and rkConfig.mode in [ 'win', 'mingw' ] :
            t = t + '.exe'

        if self.m_type == typeConsole and rkConfig.mode in [ 'win', 'mingw' ] :
            t = t + '.exe'

        if self.m_type == typePythonLib :
            pass

        return t


    def appDir (self) :

        return appDir


    # setDefine
    # ---------
    # Set a compiler define. These are stored in a dictionary so that
    # they can be overridden.
    #
    def setDefine (self, define, value = '') :

        self.m_defines[define] = value


    #  winDefine
    #  ---------
    #  Set a compiler define if and only if this is a windows build.
    #
    def winDefine (self, define, value = '') :

        if rkConfig.mode in [ 'win', 'mingw' ] :
            self.setDefine (define, value)


    # defines
    # -------
    # Get the set of compiler definitions.
    #
    def defines (self) :

        self.fixup()

        result = []
        for key in self.m_defines.keys () :
            result.append ('-D%s=%s' % (key, self.m_defines[key]))
        return result


    # addCFlags
    # ---------
    # Add a compiler flag
    #
    def addCFlags (self, flag) :

        self.m_cflags.append (flag)


    # cFlags
    # ------
    # Retrieve list of compiler flags
    #
    def cFlags (self) :

        self.fixup()
        return self.m_cflags


    # addInclude
    # ----------
    # Add an include directory.
    #
    def addInclude (self, path) :

        if path is not None :
            if not isRooted(path) : path = os.path.join (rkConfig.topDir, path)
            self.m_includes.append (path)


    # includes
    # --------
    # Get list of all include directories
    #
    def includes (self) :

        self.fixup()
        return self.m_includes


    # addLibPath
    # ----------
    # Add a library search directory
    #
    def addLibPath (self, path) :

        if path is not None :
            if not isRooted(path) : path = os.path.join (rkConfig.topDir, 'build', rkConfig.mode, path)
            self.m_libpath.append (path)


    # libPath
    # -------
    # Get list of all library search directories
    #
    def libPath (self) :

        self.fixup()
        return self.m_libpath


    # addLib
    # ------
    # Add a library which must be linked against
    #
    def addLib (self, lib) :

        if self.m_build in [ buildWIN ] :
            if lib[:len(rkConfig.libPrefix)] == rkConfig.libPrefix :
                lib = 'lib' + lib

        self.m_libs.append (lib)


    # addPython
    # ---------
    # Add include and library path for python
    #
    def addPython (self) :

        if self.m_build in [ buildMAC ] :
            self.addInclude (rkConfig.pythonIncDir)
            for flag in string.split (rkConfig.pythonLink, ' ') :
                self.addLFlags  (flag)
            return

        self.addInclude (rkConfig.pythonIncDir)
        self.addLibPath (rkConfig.pythonLibDir)
        self.addLib     (rkConfig.pythonLib   )

        if rkConfig.mode not in [ 'win', 'mingw' ] :
            self.addLib	   ('dl')
            self.addLib	   ('pthread')
            self.addLib	   ('util')


    # addPorting
    # ----------
    # Add porting headers and libraries
    #
    def addPorting (self) :

        if self.m_build in [ buildKDE ] :
            self.addAll (os.path.join ('ports', 'kde'), '_kde')

        if self.m_build in [ buildQT3 ] :
            self.addAll (os.path.join ('ports', 'qt3'), '_kde')
            self.addLib ('dl')

        if self.m_build in [ buildGTK ] :
            self.addAll (os.path.join ('ports', 'gtk'), '_kde')
            self.addLib ('dl')

        if self.m_build in [ buildWIN, buildMINGW, buildMAC ] :
            self.addAll     (os.path.join ('ports', 'qt3'), '_kde')

        if self.m_build in [ buildWIN, buildMINGW ] :
            self.addInclude ('support/windows')

        #  For windows programs, add the common library DLLs needed.
        #  Saves adding them every time.
        #
        if self.m_build in [ buildWIN, buildMINGW ] :
            if self.m_type in [ typeProgram, typeConsole ] :
                self.addLib ('wsock32' )
                self.addLib ('user32'  )
                self.addLib ('advapi32')
                self.addLib ('qtmain'  )

        self.addAll ('ports/rkdcop', '_rkdcop')


    def addCPPUnit (self) :

        self.addLib     (rkConfig.cppunitLibLib)
        self.addLibPath (rkConfig.cppunitLibDir)
        self.addInclude (rkConfig.cppunitIncDir)

        if rkConfig.cppunitLibLibQT and rkConfig.cppunitLibDirQT :
            self.addLib     (rkConfig.cppunitLibLibQT)
            self.addLibPath (rkConfig.cppunitLibDirQT)
            self.setDefine  ('__KB_CPPUNITQT', '1')
        else :
            self.setDefine  ('__KB_CPPUNITQT', '0')


    # libs
    # ----
    # Get list of libraries to be linked against.
    #
    def libs (self) :

        self.fixup()
        return self.m_libs


    def addLFlags (self, flag) :

        self.m_lflags.append (flag)


    def lFlags (self) :

        return self.m_lflags


    # allAll
    # ------
    # This is a convenience routine which adds a path to the include and
    # library search path lists, and adds a named library to link against.
    #
    def addAll (self, path, name, rt = False) :

        self.addInclude (path)

        if rt :
               self.addLibPath (path + '_rt')
        else : self.addLibPath (path)

        if rt :
               self.addLib     (rkConfig.libPrefix + 'rt' + name)
        else : self.addLib     (rkConfig.libPrefix + name)

        if self.m_build in [ buildKDE, buildQT3 ] :
            if rt :
                   self.m_depends.append (fixSourcePath(path) + '_rt/lib' + rkConfig.libPrefix + 'rt' + name + '.so')
            else : self.m_depends.append (fixSourcePath(path) +    '/lib' + rkConfig.libPrefix + name + '.so')


    # environment
    # -----------
    # Create an instance of the build environment on demand. This method
    # is provided so that used can access and modify the environment
    # before the build runs.
    #
    def environment (self) :

        if self.m_env is None :
            import SCons.Environment
            import os

	    self.m_env = SCons.Environment.Environment \
			 (
				ENV		= os.environ,
				CPPPATH		= self.includes(),
				CPPFLAGS	= self.defines () + self.cFlags(),
				LIBPATH		= self.libPath (),
				LIBS		= self.libs    (),
				LINKFLAGS	= self.lFlags  ()
			 )

            if rkConfig.mode in [ 'mingw' ] :
                self.m_env.Replace (CXX = '/usr/local/bin/i386-mingw32-g++')
		self.m_env.Replace (CC  = '/usr/local/bin/i386-mingw32-gcc')

        return self.m_env


    # build
    # -----
    # Call this method to generate the build environment. Souces is the
    # list of source files, or None to specify all .cpp files. Desktop
    # is the desktop file name if one is used.
    #
    def build (self, sources = None, desktop = None, gui = None, noinst = None, depends = None) :

        build = self.m_build
        type  = self.m_type

        ins   = installer()
        env   = self.environment()
        env.SourceCode (".", None)

        #  For windows programs, we auto-generate the .rc and .ico files
        #  from stock files.
        #
        if build in [ buildWIN, buildMINGW ] :
            if type in [ typeProgram, typeConsole ] :
                env.Append (BUILDERS = { 'rcBuilder'  : rcBuilder  } )
                env.Append (BUILDERS = { 'icoBuilder' : icoBuilder } )
                env.rcBuilder  (target = self.m_target + '.rc',  source = os.path.join (rkConfig.topDir, 'support', 'windows', 'skel.rc' )) 
                env.icoBuilder (target = self.m_target + '.ico', source = os.path.join (rkConfig.topDir, 'support', 'windows', 'skel.ico'))
                env.Depends    (target = self.m_target + '.rc',  dependency = self.m_target + '.ico')

        if sources is None :
            sources = listAllCPP ()

        #  Shared library, this one is straightforward
        #
        if self.m_type == typeSharedLib :
            if verbose() :
                sys.stderr.write ("Create shared library " + self.target() + "\n")
            if build in [ buildKDE, buildQT3 ] :
                target  = env.SharedLibrary (self.target(), sources, SHLIBSUFFIX = '.so.' + rkConfig.libver)
                env.Append    (BUILDERS = { 'soBuilder'  : soBuilder  } )
                env.soBuilder (target = self.target() + '.so', source     = self.target() + '.so.' + rkConfig.libver)
                env.Depends   (target = self.target() + '.so', dependency = self.target() + '.so.' + rkConfig.libver)
            else :
                target  = env.SharedLibrary (self.target(), sources)
            if ins and not noinst :
                ins.Alias ('install', env.Install (libDir, target))

        #  Loaded library (ie., QLibrary/KLibrary), a smidgen more complicated
        #  since we may need to generate a stub .la file.
        #
        if self.m_type == typeLoadedLib :
            if verbose() :
                sys.stderr.write ("Create loaded library " + self.target() + "\n")
            if build in [ buildKDE, buildQT3 ] :
                target  = env.SharedLibrary (self.target(), sources, SHLIBSUFFIX = '.so.' + rkConfig.libver)
                env.Append    (BUILDERS = { 'soBuilder'  : soBuilder  } )
                env.soBuilder (target = self.target() + '.so', source     = self.target() + '.so.' + rkConfig.libver)
                env.Depends   (target = self.target() + '.so', dependency = self.target() + '.so.' + rkConfig.libver)
            else :
                target  = env.SharedLibrary (self.target(), sources)
            if build in [ buildKDE, buildQT3 ] :
                env.Append (BUILDERS = { 'La' : la_builder() })
                env.La (self.target())
            if ins and not noinst :
                ins.Alias ('install', env.Install (pluginDir, target))
                if build in [ buildKDE, buildQT3 ] :
                    ins.Alias ('install', env.Install (libDir, self.target() + '.la'))

        #  Python library, this is the same as a shared library. The
        #  install stage will fugde the name.
        #
        if self.m_type == typePythonLib :
            if verbose() :
                sys.stderr.write ("Create python library " + self.target() + "\n")
            target  = env.SharedLibrary (self.target(), sources)
            if ins and not noinst :
                ins.Alias ('install', env.InstallAs (os.path.join (pyLibDir, self.target() + '.so'), target))

        #  Program (distinguised for windows as normal or console). In
        #  the case of windows add the .rc file.
        #
        if self.m_type == typeProgram or self.m_type == typeConsole :
            if verbose() :
                sys.stderr.write ("Create program " + self.target() + "\n")
            if rkConfig.mode == 'win' :
                sources = toList (sources)
                sources = sources + [ self.m_target + '.res' ]
                env.RES (self.m_target + '.rc')
            target = env.Program (self.target(), sources)
            if ins and not noinst :
                ins.Alias ('install', env.Install (binDir, target))

        #  If there is an associated desktop file and installing, get this
        #  into the services directory.
        #
        if desktop is not None :
            if ins and not noinst :
                ins.Alias ('install', env.Install (rekallServicesDir, desktop))

        if gui is not None :
            if ins and not noinst :
                ins.Alias ('install', env.Install (rekallGUIDir, gui))

#       print self.target(), '->', self.m_depends
        for d in self.m_depends :
            env.Depends (target, d)

        return target
