import	imp
import	os
import	sys
import	string
import	re
import	time

import	rkGlobal
import	rkLocator


makeSkeleton = """
#
# Skeleton makefile auto-generated by scons configure framework. Do not
# edit this file, it will be recreated at the next reconfigure.
#
all:
	( cd %s ; REKALL_MODE=%s PYTHONPATH=scons scons -Q indir=%s )

install:
	( cd %s ; REKALL_MODE=%s PYTHONPATH=scons scons -Q install indir=%s )

clean:
	( cd %s ; REKALL_MODE=%s PYTHONPATH=scons scons -Q --clean indir=%s )
"""

makeSkeletonWin = """
#
# Skeleton makefile auto-generated by scons configure framework. Do not
# edit this file, it will be recreated at the next reconfigure.
#
# Requires GNU make to run, not NMAKE.
#
SHELL=cmd.exe
export PYTHONPATH=%s
export REKALL_MODE=win
all:
	( cd %s ; .\\bin\\make.bat -Q indir=%s )

install:
	( cd %s ; .\\bin\\make.bat -Q install indir=%s )

clean:
	( cd %s ; .\\bin\\make.bat -Q --clean indir=%s )
"""


class rkWalker :

    def __init__ (self, dir, pattern = None) :

        self.m_stack = [ dir ]
        self.m_files = []
        self.m_index = 0

        if pattern : self.m_pattern = re.compile (pattern)
        else       : self.m_pattern = None

    def __getitem__ (self, index) :

        while 1 :

            try :
                file = self.m_files[self.m_index]
                self.m_index = self.m_index + 1

            except IndexError :
                self.m_dir   = self.m_stack.pop ()
                self.m_files = os.listdir (self.m_dir)
                self.m_index = 0

            else :
                name = os.path.join(self.m_dir, file)
                if file[0:3] == "CVS" :
                    continue
		if file[0:3] == ".hg" :
                    continue
                if os.path.isdir (name) :
                    if not os.path.islink (name) :
                        self.m_stack.append (name)
                    continue
                if self.m_pattern :
                    if not self.m_pattern.match (file) :
                        continue

                return [self.m_dir, file]


#  rkSettings
#  ----------
#  Class used to handle settings configuration for the build process.
#
class rkSettings :

    def __init__ (self) :

        self.m_settings = {}
        self.m_fatal    = []
        self.m_warning  = []


    #  set
    #  ---
    #  Set a key to a specified value
    #
    def set (self, key, value) :

        self.m_settings[key] = value


    #  lookup
    #  ------
    #  Look up a key value. If found and the "subdir" argument is set
    #  then treat the value as a path name and append the "subdir"
    #  argument.
    #
    def lookup (self, key, subdir = None, default = None) :

        try    :
            value = self.m_settings[key]
            if subdir : value = os.path.join (value, subdir)
            return value
        except :
            if default is not None :
                return default
            return None

    #  env
    #  ---
    #  Look up an environment value. Processes value with a "subdir"
    #  argument as for lookup above.
    #
    def env (self, key, subdir = None) :

        try    :
            value = os.environ[key]
            if subdir : value = os.path.join (value, subdir)
            return value
        except :
            return None


    #  mode
    #  ----
    #  Return the mode. This is just a convenience method.
    #
    def mode (self) :

        return self.lookup ('mode')


    #  make
    #  ----
    #  Generate a wrapper Makefile for the specified directory and
    #  tag.
    #
    def make (self, dir) :

        scons	 = dir + '/scons'
        topDir	 = os.getcwd()
        makefile = open (os.path.join (dir, 'Makefile.' + self.mode()), 'w')

        if self.mode() == 'win' :
               makefile.write  (makeSkeletonWin % (scons, topDir, dir, topDir, dir, topDir, dir))
        else : makefile.write  (makeSkeleton    % (       topDir, self.mode(), dir, topDir, self.mode(), dir, topDir, self.mode(), dir))

        makefile.close  ()


    #  disabled
    #  --------
    #  Check if the component is to be disabled. If so returns
    #  configuration options for disabled.
    #
    def disabled (self, tag, build, *args) :

        if string.lower(self.lookup ('enable-' + tag, default = 'yes')) == 'no' :

            text = ''
            for arg in args :
                text = text + '%s = None\n' % (arg)
            text = text + '%s = 0\n' % (build)

            return text

        return None


    #  enabled
    #  -------
    #  As for "disabled" above except that the default state is disabled.
    #
    def enabled (self, tag, build, *args) :

        if string.lower(self.lookup ('enable-' + tag, default = 'no')) != 'yes' :

            text = ''
            for arg in args :
                text = text + '%s = None\n' % (arg)
            text = text + '%s = 0\n' % (build)

            return text

        return None


    #  warning
    #  -------
    #  Append a warning message
    #
    def warning (self, msg) :

        self.m_warning.append (msg)


    #  fatal
    #  -----
    #  Append a fatal error message
    #
    def fatal (self, msg) :

        self.m_fatal.append (msg)


    #  report
    #  ------
    #  Print fatal and warning messages
    #
    def report (self) :

        for msg in self.m_warning :
            print "Warning    :", msg
        for msg in self.m_fatal :
            print "Fatal error:", msg


    def log (self, legend, *values) :

        if len(legend) < 40 :
            legend = legend + ' ' * (40 - len(legend))
        import sys
        sys.stdout.write (legend)
        sys.stdout.write (':')
        for value in values :
            sys.stdout.write (' ')
            sys.stdout.write (value)
        sys.stdout.write ('\n')

    #  escapePath
    #  ----------
    #  Escape backslash characters in string, used to prevent windows
    #  paths getting mangled. Note that if a path is written to a
    #  config file then the string should be like r'...'
    #
    def escapePath (self, path) :

        if self.mode() == 'win' :
            path = string.replace (path, '/', '\\')

        return path


    #  optionOrList
    #  ------------
    #  If the option is set then return its value (as a single element
    #  list), else return the list.
    #
    def optionOrList (self, option, list) :

        value = self.lookup(option)
        if value is not None and value != '' :
            return [ value ]
        return list



def writeEnable (config, what, default = None) :

    if settings.lookup ('enable-' + what, default = default) == 'yes' :
           config.write (what + ' = 1\n')
    else : config.write (what + ' = 0\n')


#  loadSpecFile
#  ------------
#  Load a specification file containing name=value pairs, storing these
#  into the settings object.
#
def loadSpecFile (name) :

    for line in open(name).readlines() :
        line = string.replace(line, '\n', '')
        if len(line) == 0 or line[0] in [ '#', ';' ] :
            continue
        bits = string.split  (line, '=')
        if len(bits) == 2 :
            settings.set (string.strip(bits[0]), string.strip(bits[1]))


def checkExec (name, dirs) :

    dirs = [ d for d in dirs ] + string.split(os.environ['PATH'], ':')
    path = rkLocator.locateFile (name, dirs)
    if path is not None :
        return True
    return False


index	 = 0
text	 = ''
help	 = 0
make	 = 0
settings = rkSettings ()


for arg in sys.argv :

    if arg == '--help' :
        help = 1
        continue

    if arg == '--make' :
        make = 1
        continue

    if arg[:2] == '--' :
        bits = string.split (arg[2:], '=')
        if len(bits) != 2 : continue

        if bits[0] == 'spec' :
            loadSpecFile (bits[1])

        settings.set (bits[0],  bits[1])



for [dir, file] in rkWalker('.', 'rkScan(|_.*).py') :

    path   = os.path.join (dir, file)
    code   = open (path)

    name   = 'Scan_%04x' % (index)
    index  = index + 1

    module = imp.new_module (name)
    exec code in module.__dict__

    if help :
        try    : module.Help (settings)
        except : pass
        continue

    if make :
        try    : module.Make (settings, dir)
        except : pass


    try :
        module.Scan
        scan = 1
    except :
        scan = 0

    if scan :
        text = text + module.Scan (settings)


if not help :

    libPrefix = settings.lookup ('libprefix', default = 'kbase' )
    appName   = settings.lookup ('appname',   default = 'rekall')
    mode      = settings.lookup ('mode',      default = 'kde'   )
    scache    = settings.lookup ('scache',    default = None	)
    cygdir    = settings.lookup ('cygdir',    default = None	)

    config    = open ('scons/rkConfig_%s.py' % (mode), 'w')

    config.write ('mode = "%s"\n' % (mode))


    writeEnable (config, 'dprintf'	) 
    writeEnable (config, 'nongpl'	) 
    writeEnable (config, 'debug'	) 
    writeEnable (config, 'tests'	) 
    writeEnable (config, 'harness'	) 
    writeEnable (config, 'rt',   'no'	) 
    writeEnable (config, 'full', 'yes'	)


    if scache is None :
           config.write ('scache = None\n')
    else : config.write ('scache = r"%s"\n' % (scache))

    if cygdir is None :
           config.write ('cygdir = None\n')
    else : config.write ('cygdir = r"%s"\n' % (cygdir))


    config.write  ("topDir = r'%s'\n"	% (os.getcwd()))

    config.write  ("libPrefix = '%s'\n" % (libPrefix))
    config.write  ("appName = '%s'\n"   % (appName  ))

    release = settings.lookup ('release',  default = rkGlobal.release)
    build   = settings.lookup ('build',    default = rkGlobal.build  )
    libver  = string.replace  (release, '.', '')

    config.write  ("release = '%s'\n"   % (release))
    config.write  ("build = '%s'\n"     % (build  ))
    config.write  ("buildTime = '%s'\n" % (time.strftime ('%H:%M %d-%m-%Y GMT', time.gmtime(time.time()))))
    config.write  ("libver = '%s'\n"	% (libver ))

    config.write  (text)
    config.close  ()


    if mode != 'win' :
        if not checkExec ('flex',  []) :
            settings.fatal ('command not found: flex' )
        if not checkExec ('bison', []) :
            settings.fatal ('command not found: bison')

    settings.report ()
