  <class name="FormBlock" title="Form Block Methods" super="Block">
    <method name="gotoQueryRow" comment="Go to row by row number">
      <return type="void" comment=""/>
      <arg name="row" type="number" comment="Row number" opt="No"/>
      <description>
          Focus moves to the row specified, where the row number applies to
          the data returned by the database. The row number should be in
          the range 0 ... N where N is the number of rows in the block; moving
          to row N moves to a new, empty record, logically just beyond
          the extant records. The displayed data will be
          scrolled to bring the specified row into view.
      </description>
    </method>
    <method name="gotoQueryRowByKey" comment="Go to row by primary key">
      <return type="number" comment="Row number or negative on error"/>
      <arg name="key" type="any" comment="Target key" opt="No"/>
      <description>
        Moves focus to the row corresponding to the specified key, where the
        key corresponds to the unique column for the query (typically,
        a table primary key column). Should
        only be used where the records are returned from a single table
        (ie., not via a query which joins multiple tables), since with
        multiple tables it is ambiguous which unique column to use.
      </description>
    </method>
    <method name="doAction" comment="Perform block level action">
      <return type="void" comment=""/>
      <arg name="action" type="number" comment="Action code" opt="No"/>
      <description>
        The specified block action is performed, where the action is
        identified by the code argument. Actions are numbers, but are
        best accessed via <emphasis>RekallMain</emphasis> (for instance,
        <emphasis>RekallMain.actNext</emphasis>) or via a block
        object (for instance, <emphasis>block.actNext</emphasis>). In
        practice, it will almost always be more convenient to use
        on of the action methods (for instance,
        <emphasis>block.firstRecord()</emphasis>). The available actions
        are listed below:
        <table cols="2">
          <tr><td>actFirst</td><td>Go to first record</td></tr>
          <tr><td>actPrevious</td><td>Go to previous record</td></tr>
          <tr><td>actNext</td><td>Go to next record</td></tr>
          <tr><td>actLast</td><td>Go to last record</td></tr>
          <tr><td>actAdd</td><td>Add a new record</td></tr>
          <tr><td>actSave</td><td>Save current record</td></tr>
          <tr><td>actDelete</td><td>Delete current record</td></tr>
          <tr><td>actQuery</td><td>Start a query</td></tr>
          <tr><td>actExecute</td><td>Execute a query</td></tr>
          <tr><td>actCancel</td><td>Cancel a quert</td></tr>
          <tr><td>actReload</td><td>Reload data</td></tr>
          <tr><td>actSaveAsNew</td><td>Save data as a new record</td></tr>
          <tr><td>actSaveUnchecked</td><td>Save data without checks</td></tr>
        </table>
      </description>
    </method>
    <method name="firstRecord" comment="Go to first record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the first record in the block. This is equivalent
          to <emphasis>block.doAction(block.actFirst)</emphasis>.
      </description>
    </method>
    <method name="previousRecord" comment="Go to previous record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the previous record in the block. If focus is
          already at the first record then no action occurs.
      </description>
    </method>
    <method name="nextRecord" comment="Go to next record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the next in the block. If focus is already at the
          last record, then focus moves to a new empty record (which can be
          filled in and inserted into the database). If focus is already at
          this new record then no action takes place.
      </description>
    </method>
    <method name="lastRecord" comment="Go to last record">
      <return type="bool" comment="Success"/>
      <description>
          Moves focus to the last record. This is the last record returned
          by the database, and not a new empty record.
      </description>
    </method>
    <method name="addRecord" comment="Add a new record">
      <return type="bool" comment="Success"/>
      <description>
          Focus is moved to a new, empty record. The record is not actually
          added at this point; it will be inserted into the database when
          subsequently saved.
      </description>
    </method>
    <method name="saveRecord" comment="Save current record">
      <return type="bool" comment="Success"/>
      <description>
          Changes made to the current record are saved to the database.
      </description>
    </method>
    <method name="deleteRecord" comment="Delete current record">
      <return type="bool" comment="Success"/>
      <description>
          The current record is deleted from the database, and focus moves to the
          next record. Note that after a deletion, records are in effect
          renumbered to account for the deleted record.
      </description>
    </method>
    <method name="startQuery" comment="Start query">
      <return type="bool" comment="Success"/>
      <description>
          The block is set into query mode, equivalent to the user clicking
          the <emphasis>start query</emphasis> tool button. All displayed
          controls for the current row are cleared, and the user can
          then enter search terms prior to executing the query.
      </description>
    </method>
    <method name="executeQuery" comment="Execute query">
      <return type="bool" comment="Success"/>
      <description>
          Executes a query started with the <emphasis>startQuery()</emphasis>
          method (or via the <emphasis>start query</emphasis> tool button or
          menu entry).
      </description>
    </method>
    <method name="cancelQuery" comment="Cancel query">
      <return type="bool" comment="Success"/>
      <description>
          Cancels a query. All controls in the current row are returned
          to their state before the query was started.
      </description>
    </method>
    <method name="saveAsNewRecord" comment="Save current record as new record">
        <return type="void" comment=""/>
    </method>
    <method name="invalidControls" comment="Get list of invalid controls">
      <return type="list" comment="List of invalid controls"/>
      <arg name="row" type="number" comment="Row number" opt="No"/>
      <arg name="required" type="bool" comment="True to return list if no controls set" opt="Yes"/>
      <arg name="recurse" type="bool" comment="True to enter nested containers" opt="Yes"/>
    </method>
  </class>
