  <class name="Item" title="Item Methods and Properties" super="Object">
    <property name="value" access="rw" comment="Current value">
      <description>
        The <emphasis>value</emphasis> property accesses the value
        of the control for the current row, and can be used to
        access or update the value.
        <emphasis>ctrl.value</emphasis> is equivalent to
        <emphasis>ctrl.getValue(ctrl.getBlock().getQueryRow())</emphasis>
        or <emphasis>ctrl.setValue(ctrl.getBlock().getQueryRow(), value)</emphasis>
        for access or update respectively. It can only be used to
        access the current row.
      </description>
    </property>
    <method name="setValue" comment="Update value in control">
      <return type="void" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="value" type="string" comment="Value" opt="No"/>
      <description>
         Set the value of a control. The row number does
         <emphasis>not</emphasis> count displayed rows, rather the
         row number in the query. Use the block method
         <emphasis>getQueryRow()</emphasis> to retrieve the
         current row number. The value for the current row can be
         accessed directly using the <emphasis>value</emphasis> property
         described above.
      </description>
    </method>
    <method name="getValue" comment="Get value from control">
      <return type="string" comment="Value"/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <description>
         Fetches the control value for a specified (query) row
         number.  The value for the current row can be
         accessed directly using the <emphasis>value</emphasis> property
         described above. If the specified row is not visible, returns
         an empty value.
      </description>
    </method>
    <method name="clearValue" comment="Clear control value">
      <return type="void" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <description>
         Clears the control value for a specified (query) row
         number; in effect this sets the value to null. If the specified row
         is not visible, the call has no effect.
      </description>
    </method>
    <method name="getRowValue" comment="Get value from underlying query">
      <return type="string" comment="Value"/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <description>
         This is similar to <emphasis>getValue</emphasis>, but the value
         is returned from the underlying query. Values may be retrieved from
         any row (not only those for visible records), but may differ from
         the displayed value if the user has changed but not saved a
         value.
      </description>
    </method>
    <method name="setTabOrder" comment="Set control tab ordering">
      <return type="void" comment=""/>
      <arg name="order" type="number" comment="Tab order number" opt="No"/>
      <description>
      </description>
    </method>
    <method name="setEnabled" comment="Enable or disable control">
      <return type="void" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="enable" type="bool" comment="True to enable" opt="No"/>
      <description>
      </description>
    </method>
    <method name="setVisible" comment="Show or hide control">
      <return type="void" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="show" type="bool" comment="True to show" opt="No"/>
      <description>
      </description>
    </method>
    <method name="isEnabled" comment="Test if control is enabled">
      <return type="bool" comment="True if enabled"/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <description>
      </description>
    </method>
    <method name="isVisible" comment="Test if control is visible">
      <return type="bool" comment="True if visible"/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <description>
      </description>
    </method>
    <method name="isValid" comment="Test if control contents are valid">
      <return type="bool" comment="True if valid"/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <description>
      </description>
    </method>
    <method name="errorText" comment="Get the error text for the control">
      <return type="string" comment="Error text"/>
      <description>
        The error text for a data control can be set in the properties for
        the control. If it not set then text based on the control and and
        expression is used.
      </description>
    </method>
  </class>
