  <class name="LinkTree" title="Link/Tree Methods and Properties" super="Item">
    <method name="setUserFilter" comment="Set filter on link query">
      <return type="void" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="filter" type="string" comment="Filter expression" opt="No"/>
      <description>
          Set a filter expression to be used when the link or tree is next
          reloaded (see the <i>reload</i> method below). This
          should be a valid SQL <i>order by</i> expression, and will be used in
          the query in addition to (and after) any other filtering expressions.
          Note that when used, this method must be called whenever the
          record display is scrolled, and would typically be
          invoked with a block <i>onDisplay</i> event.
      </description>
    </method>
    <method name="setUserSorting" comment="Set sorting on link query">
      <return type="void" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="sorting" type="string" comment="Sorting expression" opt="No"/>
      <description>
          Set an ordering expression to be used when the link or tree is next
          reloaded (see the <i>reload</i> method below). This
          should be a valid SQL <i>order by</i> expression, and will be used in
          the query in addition to (and after) any other ordering expressions.          
          Note that when used, this method must be called whenever the
          record display is scrolled, and would typically be
          invoked with a block <i>onDisplay</i> event.
      </description>
    </method>
    <method name="reload" comment="Reload link query">
      <return type="void" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <description>
          This method reloads the link or tree, using the query specified
          at design time, plus a user filter or sorting expression is
          set (see <i>setUserFilter</i> and <i>setUserSorting</i>).
          Note that when used, this method must be called whenever the
          record display is scrolled, and would typically be
          invoked with a block <i>onDisplay</i> event.
      </description>
    </method>
    <method name="getExtra" comment="Get extra value">
      <return type="string" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="extra" type="number" comment="Extra value index" opt="No"/>
      <description>
          Returns an extra value for the control for the specified row.
          Extra values are specified in the control setup, and name
          additional columns (or, generally, SQL expressions) that are
          retrieved from the linked table (or query). If the specified
          row is not visible, returns an empty value.
      </description>
    </method>
    <method name="getRowExtra" comment="Get extra value from underlying query">
      <return type="string" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="extra" type="number" comment="Extra value index" opt="No"/>
      <description>
          Similar to <emphasis>getExtra</emphasis>, but the value is
          retrieved from the underlying query. Any row can be accessed
          (not only those for visible rows). The value may not correspond
          to the currently displayed value if the user has changed the
          control but not saved the record.
      </description>
    </method>
    <method name="getDisplay" comment="Get display value">
      <return type="string" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="index" type="number" comment="Display value index" opt="No"/>
      <description>
          Returns the display value, rather than the actual data value. Where
          values from multiple columns are displayed, the
          <emphasis>index</emphasis> argument specifies which (so would
          be zero in the usual case where only a single column is displayed).
      </description>
    </method>
    <method name="getRowDisplay" comment="Get display value from underlying query">
      <return type="string" comment=""/>
      <arg name="rowNum" type="number" comment="Query row number" opt="No"/>
      <arg name="index" type="number" comment="Display value index" opt="No"/>
      <description>
          Similar to <emphasis>getDisplay</emphasis>, but the value is
          retrieved from the underlying query. Any row can be accessed
          (not only those for visible rows). The value may not correspond
          to the currently displayed value if the user has changed the
          control but not saved the record.
      </description>
    </method>
  </class>
