/***************************************************************************
    file	         : alltests.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/TestResult.h>
#include	<cppunit/TestRunner.h>
#include	<cppunit/extensions/TestFactoryRegistry.h>
#include	<cppunit/ui/text/TextTestRunner.h>
#include	<cppunit/ui/qt/QtTestRunner.h>

#include	<qfileinfo.h>
#include	<qapplication.h>
#include	<qmap.h>

#if		__KB_KDE
#include	<kapp.h>
#include	<klocale.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>
#endif

#include	"kb_classes.h"
#include	"kb_desktop.h"
#include	"kb_locator.h"
#include	"kb_libloader.h"

#ifdef		Q_OS_LINUX
#include	<dlfcn.h>
#define		DLERROR()	dlerror()
#endif

#ifdef		Q_OS_WIN32
#define		DLERROR() 	""
#endif

#if		__KB_RUNTIME
#define		DTPREFIX	"rekallrt"
#define		DTTYPE		"RekallRT/Test"
#else
#define		DTPREFIX	"rekall"
#define		DTTYPE		"Rekall/Test"
#endif

#if		defined(__WIN32__)
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		EXPORT_API
#endif

#define		__DEBUG_ALLTESTS	(0)
#if		__DEBUG_ALLTESTS
#include	"kb_dprintf.h"
#endif


class	KBBuildTest : public CppUnit::TestCase
{
public	:

	KBBuildTest	() ;
	KBBuildTest	(std::string name) ;

	void			testBuild	() ;
	static CppUnit::Test	*suite		() ;
}	;

KBBuildTest::KBBuildTest ()
{
}

KBBuildTest::KBBuildTest
	(	std::string	name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBBuildTest::testBuild ()
{
}

CppUnit::Test	EXPORT_API
		*testLoader
		(	const QMap<QString,QString> *settings
		)
{
	CppUnit::TestSuite	*suite	= new CppUnit::TestSuite ("AllTests") ;
	KBLibLoader		*loader	= KBLibLoader::self() ;

	/* Scan the local desktop files which descrive the rekall	*/
	/* "services". These are the same files as used for the KTrader	*/
	/* version, but the decoding in KBDesktop is much simplified.	*/
	QString		 dtDir	= locateDir
				  (	"appdata",
					"services/" DTPREFIX "_table.desktop"
				  )	;

	XPRINTF
	((	"loadTestPlugins: [%s]\n",
		dtDir.latin1()
	))	;

	QPtrList<KBDesktop> dtDefs		;

	KBDesktop::scan (dtDir + "/services", DTPREFIX "_", dtDefs) ;

	for (uint idx = 0 ; idx < dtDefs.count() ; idx += 1)
	{
		KBDesktop *desktop = dtDefs.at(idx) ;

		if (desktop->property ("ServiceTypes") != DTTYPE)
			continue	;

#if		__DEBUG_ALLTESTS
		desktop->print () ;
#endif

		QString	libname	 = desktop->property("X-KDE-Library") ;
		KBLibrary *lib	 = loader->getLibrary(libname) ;

		if (lib == 0)
		{	
			XPRINTF
			((	"loadRekallPlugins: %s: no library: %s\n", 
				libname.latin1(),
				loader->lastErrorMessage().latin1()
			))	;
			continue;
		}

		KBFactory	*factory = (KBFactory *)lib->factory() ;
		if (factory == 0)
		{	
			XPRINTF
			((	"loadRekallPlugins: %s: cannot create factory\n",
				libname.latin1()
			))	;
			continue;
		}

		/* Call the plugin factory's ::create method, but note	*/
		/* that we do not expect any results.			*/
		factory->create (0, (const char *)suite, (const char *)settings) ;

		XPRINTF
		((	"loadRekallPlugins: %s: returned from factory\n",
			libname.latin1()
		))	;
	}

	CppUnit::TestSuite *s = new CppUnit::TestSuite("Build") ;
	s->addTest
	(	new CppUnit::TestCaller<KBBuildTest>
		(	"BUILD:" __KB_BUILD_RELEASE __KB_BUILD_BUILD ":" __KB_BUILD_TIME,
			&KBBuildTest::testBuild
	)	)	;
	suite->addTest (s) ;

	return	suite	;
}
