/***************************************************************************
    file	         : format.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBFormatTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	KBField			*getField	(const char *) ;
	KBControl		*getControl	(KBField    *) ;
	KBControl		*getControl	(KBField    *, uint, uint) ;
	RKLineEdit		*getWidget	(KBControl  *) ;

public	:

	KBFormatTest	() ;
	KBFormatTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testFormat0	() ;
	void			testFormat1	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

static	const char *values_date[] =
{
	"One",
	"Two",
	"Three",
	"Four",
	"Five",
	"Six",
	"Seven",
	"Eight",
	"Nine"
}	;

KBFormatTest::KBFormatTest ()
{
}

KBFormatTest::KBFormatTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBFormatTest::setUp ()
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"format: %s: cannot open: %s\n",
			"API.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenForm (dbInfo, "api", "Format", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBFormatTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBFormatTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBFormatTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBField
	*KBFormatTest::getField
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBField")) ;

	KBField *field = (KBField *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(field->getName())
	)	;

	return	field	;
}

KBControl
	*KBFormatTest::getControl
	(	KBField		*field
	)
{
	KBControl *control = KBTestAccessor::getControl (field) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	return	  control  ;
}

KBControl
	*KBFormatTest::getControl
	(	KBField		*field,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (field, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKLineEdit
	*KBFormatTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKLineEdit")) ;
	return	(RKLineEdit *)widget ;
}

void	KBFormatTest::testFormat0 ()
{
	KBField *field1  = getField  ("date_1") ;
	KBField *field2  = getField  ("date_2") ;
	KBField *field3  = getField  ("date_3") ;

	KBControl  *control1 = getControl (field1  ) ;
	RKLineEdit *widget1  = getWidget  (control1) ;
	KBControl  *control2 = getControl (field2  ) ;
	RKLineEdit *widget2  = getWidget  (control2) ;
	KBControl  *control3 = getControl (field3  ) ;
	RKLineEdit *widget3  = getWidget  (control3) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("2006-01-01")),
		_STDSTR(widget1->text())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01-Jan-06")),
		_STDSTR(widget2->text())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01/01/06")),
		_STDSTR(widget3->text())
	)	;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("2006-01-01")),
		_STDSTR(control1->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01-Jan-06")),
		_STDSTR(control2->getValue().getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01/01/06")),
		_STDSTR(control3->getValue().getRawText())
	)	;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("2006-01-01")),
		_STDSTR(field1->getValue(0).getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01-Jan-06")),
		_STDSTR(field2->getValue(0).getRawText())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01/01/06")),
		_STDSTR(field3->getValue(0).getRawText())
	)	;

}

void	KBFormatTest::testFormat1 ()
{
	KBField *field1  = getField  ("date_1") ;
	KBField *field2  = getField  ("date_2") ;
	KBField *field3  = getField  ("date_3") ;

	RKLineEdit *widget1  = getWidget  (getControl (field1  )) ;
	RKLineEdit *widget2  = getWidget  (getControl (field2  )) ;
	RKLineEdit *widget3  = getWidget  (getControl (field3  )) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("2006-01-01")),
		_STDSTR(widget1->text())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01-Jan-06")),
		_STDSTR(widget2->text())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("01/01/06")),
		_STDSTR(widget3->text())
	)	;
	CPPUNIT_ASSERT (!field1->changed(0)) ;
	CPPUNIT_ASSERT (!field2->changed(0)) ;
	CPPUNIT_ASSERT (!field3->changed(0)) ;
}

CppUnit::Test
	*KBFormatTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Format") ;

	if (rekallTestSkip ("Format")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBFormatTest> ("noDisplay", &KBFormatTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBFormatTest> ("withGUI",   &KBFormatTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBFormatTest>
			(	"format0",
				&KBFormatTest::testFormat0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFormatTest>
			(	"format1",
				&KBFormatTest::testFormat1
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *formatTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBFormatTest::suite (__settings) ;
}
