/***************************************************************************
    file	         : listboxes.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_listbox.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_listbox.h"
#include	"kb_ctrl.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBListBoxTest : public KBAPITest
{
public	:

	KBListBoxTest	() ;
	KBListBoxTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testListboxes0	() ;
	void			testListboxes1	() ;
	void			testListboxes2	() ;
	void			testListboxes3	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;


static	const char *stringList1 [] =
{
	"",
	"A",
	"B",
	"C",
	"D"
}	;

static	const char *stringList2 [] =
{
	"",
	"1",
	"2",
	"3",
	"4"
}	;

KBListBoxTest::KBListBoxTest ()
{
}

KBListBoxTest::KBListBoxTest
	(	std::string		name
	)
	:
	KBAPITest	(name)
{
}

void	KBListBoxTest::setUp ()
{
	openForm ("ListBoxes")	;
}

void	KBListBoxTest::tearDown ()
{
	rekallTestCloseForm() ;
}


void	KBListBoxTest::testListboxes0 ()
{
	KBListBox	*listbox0 = getListBox  ("ListBox_0", "ListBox_0") ;
	KBControl	*control0 = getControl  (listbox0) ;
	RKListBox	*combo0	  = getRKListBox(control0) ;

	QStringList	strings	= listbox0->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		5,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 5 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList1[idx1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 5 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList1[idx2])),
			_STDSTR(combo0->text(idx2))
		)	;
}

void	KBListBoxTest::testListboxes1 ()
{
	KBListBox	*listbox1 = getListBox  ("ListBox_1", "ListBox_1") ;
	KBControl	*control1 = getControl  (listbox1 ) ;
	RKListBox	*combo1	  = getRKListBox(control1) ;

	QStringList	strings	  = listbox1->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		4,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 4 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList1[idx1+1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 4 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList1[idx2+1])),
			_STDSTR(combo1->text(idx2))
		)	;
}

void	KBListBoxTest::testListboxes2 ()
{
	KBListBox	*listbox0 = getListBox  ("ListBox_0", "ListBox_0") ;
	KBControl	*control0 = getControl  (listbox0 ) ;
	RKListBox	*combo0	  = getRKListBox(control0) ;

	QStringList	strings	  ;

	for (uint idx0 = 1 ; idx0 < 5 ; idx0 += 1)
		strings.append (stringList2[idx0]) ;
	listbox0->setValues (strings) ;

	strings	  = listbox0->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		5,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 5 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList2[idx1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 5 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList2[idx2])),
			_STDSTR(combo0->text(idx2))
		)	;
}

void	KBListBoxTest::testListboxes3 ()
{
	KBListBox	*listbox1 = getListBox  ("ListBox_1", "ListBox_1") ;
	KBControl	*control1 = getControl  (listbox1 ) ;
	RKListBox	*combo1	  = getRKListBox(control1) ;

	QStringList	strings	  ;

	for (uint idx0 = 1 ; idx0 < 5 ; idx0 += 1)
		strings.append (stringList2[idx0]) ;
	listbox1->setValues (strings) ;

	strings	  = listbox1->getValues () ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	"Unexpected entry count",
		4,
		(int)strings.count()
	)	;

	for (uint idx1 = 0 ; idx1 < 4 ; idx1 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected text at offset %1").arg(idx1)),
			_STDSTR(QString(stringList2[idx1+1])),
			_STDSTR(strings[idx1])
		)	;

	for (uint idx2 = 0 ; idx2 < 4 ; idx2 += 1)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control value at offset %1").arg(idx2)),
			_STDSTR(QString(stringList2[idx2+1])),
			_STDSTR(combo1->text(idx2))
		)	;
}

CppUnit::Test
	*KBListBoxTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("ListBoxes") ;

	if (rekallTestSkip ("ListBoxes")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBListBoxTest> ("noDisplay", &KBListBoxTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBListBoxTest> ("withGUI",   &KBListBoxTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBListBoxTest>
			(	"listboxes0",
				&KBListBoxTest::testListboxes0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBListBoxTest>
			(	"listboxes1",
				&KBListBoxTest::testListboxes1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBListBoxTest>
			(	"listboxes2",
				&KBListBoxTest::testListboxes2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBListBoxTest>
			(	"listboxes3",
				&KBListBoxTest::testListboxes3
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *listboxTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBListBoxTest::suite (__settings) ;
}
