/***************************************************************************
    file	         : nested.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBNestedTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	KBField			*getField	(const char *) ;
	KBControl		*getControl	(KBField    *) ;
	KBControl		*getControl	(KBField    *, uint, uint) ;
	RKLineEdit		*getWidget	(KBControl  *) ;

public	:

	KBNestedTest	() ;
	KBNestedTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testNested0	() ;
	void			testNested1	() ;
	void			testNested2	() ;
	void			testNested3	() ;
	void			testNested4	() ;
	void			testNested5	() ;
	void			testNested6	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

static	const char *values_t[] =
{
	"One",
	"Two",
	"Three",
	"Four",
	"Five",
	"Six",
	"Seven",
	"Eight",
	"Nine"
}	;

KBNestedTest::KBNestedTest ()
{
}

KBNestedTest::KBNestedTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBNestedTest::setUp ()
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"nested: %s: cannot open: %s\n",
			"API.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenForm (dbInfo, "api", "Nested", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBNestedTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBNestedTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBNestedTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBField
	*KBNestedTest::getField
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getGlobalNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBField")) ;

	KBField *field = (KBField *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(field->getName())
	)	;

	return	field	;
}

KBControl
	*KBNestedTest::getControl
	(	KBField		*field
	)
{
	KBControl *control = KBTestAccessor::getControl (field) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	return	  control  ;
}

KBControl
	*KBNestedTest::getControl
	(	KBField		*field,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (field, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKLineEdit
	*KBNestedTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKLineEdit")) ;
	return	(RKLineEdit *)widget ;
}

void	KBNestedTest::testNested0 ()
{
	KBField *field0  = getField  ("Field_0") ;
	KBField *field1  = getField  ("Field_1") ;
	KBField *field2  = getField  ("Field_2") ;
	KBField *field3  = getField  ("Field_3") ;

	CPPUNIT_ASSERT ( field0->isVisible(0)) ;
	CPPUNIT_ASSERT ( field0->isEnabled(0)) ;
	CPPUNIT_ASSERT ( field1->isVisible(0)) ;
	CPPUNIT_ASSERT (!field1->isEnabled(0)) ;
	CPPUNIT_ASSERT (!field2->isVisible(0)) ;
	CPPUNIT_ASSERT ( field2->isEnabled(0)) ;
	CPPUNIT_ASSERT (!field3->isVisible(0)) ;
	CPPUNIT_ASSERT (!field3->isEnabled(0)) ;

	KBControl  *control0 = getControl (field0  ) ;
	RKLineEdit *widget0  = getWidget  (control0) ;
	KBControl  *control1 = getControl (field1  ) ;
	RKLineEdit *widget1  = getWidget  (control1) ;
	KBControl  *control2 = getControl (field2  ) ;
	RKLineEdit *widget2  = getWidget  (control2) ;
	KBControl  *control3 = getControl (field3  ) ;
	RKLineEdit *widget3  = getWidget  (control3) ;

	CPPUNIT_ASSERT ( widget0->isVisible()) ;
	CPPUNIT_ASSERT ( widget0->isEnabled()) ;
	CPPUNIT_ASSERT ( widget1->isVisible()) ;
	CPPUNIT_ASSERT (!widget1->isEnabled()) ;
	CPPUNIT_ASSERT (!widget2->isVisible()) ;
	CPPUNIT_ASSERT ( widget2->isEnabled()) ;
	CPPUNIT_ASSERT (!widget3->isVisible()) ;
	CPPUNIT_ASSERT (!widget3->isEnabled()) ;

	KBControl  *drow00   = getControl (field0, 0, 0) ;
	KBControl  *drow10   = getControl (field1, 0, 0) ;
	KBControl  *drow20   = getControl (field2, 0, 0) ;
	KBControl  *drow30   = getControl (field3, 0, 0) ;

	CPPUNIT_ASSERT (drow00 == control0) ;
	CPPUNIT_ASSERT (drow10 == control1) ;
	CPPUNIT_ASSERT (drow20 == control2) ;
	CPPUNIT_ASSERT (drow30 == control3) ;
}

void	KBNestedTest::testNested1 ()
{
	KBField *field   = getField  ("Field_0") ;
	uint	drows	 = m_form->m_form->rowsInBlock() ;

	for (uint row = 0 ; row < drows ; row += 1)
	{
		KBControl  *control = getControl (field, row, row) ;
		RKLineEdit *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(widget->text())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected field text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(field  ->getValue(row).getRawText())
		)	;
	}

}

void	KBNestedTest::testNested2 ()
{
	KBField *field   = getField  ("Field_1") ;
	uint	drows	 = m_form->m_form->rowsInBlock() ;

	for (uint row = 0 ; row < drows ; row += 1)
	{
		KBControl  *control = getControl (field, row, row) ;
		RKLineEdit *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget text: row %1").arg(row)),
			_STDSTR(QString(values_t[row])),
			_STDSTR(widget->text())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString(values_t[row])),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected field text: row %1").arg(row)),
			_STDSTR(QString(values_t[row])),
			_STDSTR(field  ->getValue(row).getRawText())
		)	;
	}

}

void	KBNestedTest::testNested3 ()
{
	KBField *field   = getField  ("Field_2") ;
	uint	drows	 = m_form->m_form->rowsInBlock() ;

	for (uint row = 0 ; row < drows ; row += 1)
	{
		KBControl  *control = getControl (field, row, row) ;
		RKLineEdit *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(widget->text())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected field text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(field  ->getValue(row).getRawText())
		)	;
	}

}

void	KBNestedTest::testNested4 ()
{
	KBField *field   = getField  ("Field_3") ;
	uint	drows	 = m_form->m_form->rowsInBlock() ;

	for (uint row = 0 ; row < drows ; row += 1)
	{
		KBControl  *control = getControl (field, row, row) ;
		RKLineEdit *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("Unexpected widget text: row %1").arg(row)),
			widget->text().isEmpty()
		)	;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			control->getValue()   .getRawText().isEmpty()
		)	;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("Unexpected field text: row %1").arg(row)),
			field  ->getValue(row).getRawText().isEmpty()
		)	;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("Unexpected field text: row %1").arg(row)),
			field  ->getValue(row).isEmpty()
		)	;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("Unexpected field text: row %1").arg(row)),
			field  ->getValue(row).isNull()
		)	;
	}
}

void	KBNestedTest::testNested5 ()
{
	KBField *field   = getField  ("Field_0") ;
	int	width	 = m_form->m_widget->width () ;
	int	height	 = m_form->m_widget->height() ;

	rekallGUIPause	   ("testNested5") ;
	uint	drows1	 = m_form->m_form->rowsInBlock() ;

	m_form->m_widget->resize (width, height + 20 * 2) ;
	rekallGUIPause	   ("testNested5") ;

	uint	drows2	 = m_form->m_form->rowsInBlock() ;

//	fprintf
//	(	stderr,
//		"KBNestedTest::testNested2: %d->%d\n",
//		drows1,
//		drows2
//	)	;

	for (uint row = 0 ; row < drows2 ; row += 1)
	{
		KBControl  *control = getControl (field, row, row) ;
		RKLineEdit *widget  = getWidget  (control) ;

		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected widget text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(widget->text())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected control text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(control->getValue().getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected field text: row %1").arg(row)),
			_STDSTR(QString::number(row + 1)),
			_STDSTR(field  ->getValue(row).getRawText())
		)	;
	}
}

void	KBNestedTest::testNested6 ()
{
	KBField *field0  = getField  ("Field_0") ;
	KBField *field1  = getField  ("Field_1") ;
	KBField *field2  = getField  ("Field_2") ;
	KBField *field3  = getField  ("Field_3") ;

	field0->setVisible (0, false) ;
	field0->setEnabled (0, false) ;
	field1->setVisible (1, false) ;
	field1->setEnabled (1,  true) ;
	field2->setVisible (2,  true) ;
	field2->setEnabled (2, false) ;
	field3->setVisible (3, true ) ;
	field3->setEnabled (3, true ) ;

	CPPUNIT_ASSERT (!field0->isVisible(0)) ;
	CPPUNIT_ASSERT (!field0->isEnabled(0)) ;
	CPPUNIT_ASSERT (!field1->isVisible(1)) ;
	CPPUNIT_ASSERT ( field1->isEnabled(1)) ;
	CPPUNIT_ASSERT ( field2->isVisible(2)) ;
	CPPUNIT_ASSERT (!field2->isEnabled(2)) ;
	CPPUNIT_ASSERT ( field3->isVisible(3)) ;
	CPPUNIT_ASSERT ( field3->isEnabled(3)) ;

	KBControl  *control0 = getControl (field0, 0, 0) ;
	RKLineEdit *widget0  = getWidget  (control0) ;
	KBControl  *control1 = getControl (field1, 1, 1) ;
	RKLineEdit *widget1  = getWidget  (control1) ;
	KBControl  *control2 = getControl (field2, 2, 2) ;
	RKLineEdit *widget2  = getWidget  (control2) ;
	KBControl  *control3 = getControl (field3, 3, 3) ;
	RKLineEdit *widget3  = getWidget  (control3) ;

	CPPUNIT_ASSERT (!widget0->isVisible()) ;
	CPPUNIT_ASSERT (!widget0->isEnabled()) ;
	CPPUNIT_ASSERT (!widget1->isVisible()) ;
	CPPUNIT_ASSERT ( widget1->isEnabled()) ;
	CPPUNIT_ASSERT ( widget2->isVisible()) ;
	CPPUNIT_ASSERT (!widget2->isEnabled()) ;
	CPPUNIT_ASSERT ( widget3->isVisible()) ;
	CPPUNIT_ASSERT ( widget3->isEnabled()) ;
}

CppUnit::Test
	*KBNestedTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Nested") ;

	if (rekallTestSkip ("Nested")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBNestedTest> ("noDisplay", &KBNestedTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBNestedTest> ("withGUI",   &KBNestedTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBNestedTest>
			(	"nested0",
				&KBNestedTest::testNested0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBNestedTest>
			(	"nested1",
				&KBNestedTest::testNested1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBNestedTest>
			(	"nested2",
				&KBNestedTest::testNested2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBNestedTest>
			(	"nested3",
				&KBNestedTest::testNested3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBNestedTest>
			(	"nested4",
				&KBNestedTest::testNested4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBNestedTest>
			(	"nested5",
				&KBNestedTest::testNested5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBNestedTest>
			(	"nested6",
				&KBNestedTest::testNested6
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *nestedTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBNestedTest::suite (__settings) ;
}
