/***************************************************************************
    file	         : records.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBRecordsTest : public CppUnit::TestCase
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;

	KBField			*getField	(const char *) ;
	KBControl		*getControl	(KBField    *) ;
	KBControl		*getControl	(KBField    *, uint, uint) ;
	RKLineEdit		*getWidget	(KBControl  *) ;

public	:

	KBRecordsTest	() ;
	KBRecordsTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testRecords1	() ;
	void			testRecords2	() ;
	void			testRecords3	() ;
	void			testRecords4	() ;
	void			testRecords5	() ;
	void			testRecords6	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBRecordsTest::KBRecordsTest ()
{
}

KBRecordsTest::KBRecordsTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBRecordsTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("API.rkl", "test/api") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "api", "Records", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)0,
		m_form->m_form->getCurQRow()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)9,
		m_form->m_form->getNumRows()
	)	;
}

void	KBRecordsTest::tearDown ()
{
	rekallTestCloseForm () ;
	delete	m_dbInfo ;
}

void	KBRecordsTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBRecordsTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}


KBField
	*KBRecordsTest::getField
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBField")) ;

	KBField *field = (KBField *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(field->getName())
	)	;

	return	field	;
}

KBControl
	*KBRecordsTest::getControl
	(	KBField		*field
	)
{
	KBControl *control = KBTestAccessor::getControl (field) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	return	  control  ;
}

KBControl
	*KBRecordsTest::getControl
	(	KBField		*field,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (field, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKLineEdit
	*KBRecordsTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKLineEdit")) ;
	return	(RKLineEdit *)widget ;
}

void	KBRecordsTest::testRecords1 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (4)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)4,
		m_form->m_form->getCurQRow()
	)	;
}

void	KBRecordsTest::testRecords2 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (4)
	)	;
	CPPUNIT_ASSERT
	(	m_form->m_form->doAction (KB::Next)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)5,
		m_form->m_form->getCurQRow()
	)	;
}

void	KBRecordsTest::testRecords3 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (4)
	)	;
	CPPUNIT_ASSERT
	(	m_form->m_form->doAction (KB::Previous)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)3,
		m_form->m_form->getCurQRow()
	)	;
}

void	KBRecordsTest::testRecords4 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (4)
	)	;
	CPPUNIT_ASSERT
	(	m_form->m_form->doAction (KB::First)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)0,
		m_form->m_form->getCurQRow()
	)	;
}

void	KBRecordsTest::testRecords5 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (4)
	)	;
	CPPUNIT_ASSERT
	(	m_form->m_form->doAction (KB::Last)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(int)m_form->m_form->getNumRows() - 1,
		(int)m_form->m_form->getCurQRow()
	)	;
}

void	KBRecordsTest::testRecords6 ()
{
}

CppUnit::Test
	*KBRecordsTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Field") ;

	if (rekallTestSkip ("Records")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBRecordsTest> ("noDisplay", &KBRecordsTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBRecordsTest> ("withGUI",   &KBRecordsTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBRecordsTest>
			(	"records1",
				&KBRecordsTest::testRecords1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBRecordsTest>
			(	"records2",
				&KBRecordsTest::testRecords2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBRecordsTest>
			(	"records3",
				&KBRecordsTest::testRecords3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBRecordsTest>
			(	"records4",
				&KBRecordsTest::testRecords4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBRecordsTest>
			(	"records5",
				&KBRecordsTest::testRecords5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBRecordsTest>
			(	"records6",
				&KBRecordsTest::testRecords6
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *recordsTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBRecordsTest::suite (__settings) ;
}
