/***************************************************************************
    file	         : scan.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBScanTest : public CppUnit::TestCase
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;

	KBField		 	*m_field1	;
	KBField		 	*m_field2	;
	KBField		 	*m_field3	;
	KBField		 	*m_field4	;
	KBField		 	*m_field5	;
	KBField		 	*m_field6	;
	KBField		 	*m_field7	;

	KBField			*getField	(const char *) ;
	KBControl		*getControl	(KBField    *) ;
	KBControl		*getControl	(KBField    *, uint, uint) ;
	RKLineEdit		*getWidget	(KBControl  *) ;

public	:

	KBScanTest	() ;
	KBScanTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testScanInvalid1() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBScanTest::KBScanTest ()
{
}

KBScanTest::KBScanTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBScanTest::setUp ()
{
	KBError	error	;

	m_dbInfo = rekallDBInfo ("API.rkl", "test/api", "noname") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "api", "Validity", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_field1 = getField ("Field_t_1") ;
	m_field2 = getField ("Field_t_2") ;
	m_field3 = getField ("Field_t_3") ;
	m_field4 = getField ("Field_t_4") ;
	m_field5 = getField ("Field_t_5") ;
	m_field6 = getField ("Field_t_6") ;
	m_field7 = getField ("Field_t_7") ;

	CPPUNIT_ASSERT (m_field1 != 0) ;
	CPPUNIT_ASSERT (m_field2 != 0) ;
	CPPUNIT_ASSERT (m_field3 != 0) ;
	CPPUNIT_ASSERT (m_field4 != 0) ;
	CPPUNIT_ASSERT (m_field5 != 0) ;
	CPPUNIT_ASSERT (m_field6 != 0) ;
	CPPUNIT_ASSERT (m_field7 != 0) ;
}

void	KBScanTest::tearDown ()
{
	rekallTestCloseForm () ;
	delete	m_dbInfo ;
}

void	KBScanTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBScanTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBField
	*KBScanTest::getField
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBField")) ;

	KBField *field = (KBField *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(field->getName())
	)	;

	return	field	;
}

KBControl
	*KBScanTest::getControl
	(	KBField		*field
	)
{
	KBControl *control = KBTestAccessor::getControl (field) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	return	  control  ;
}

KBControl
	*KBScanTest::getControl
	(	KBField		*field,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (field, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKLineEdit
	*KBScanTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKLineEdit")) ;
	return	(RKLineEdit *)widget ;
}

void	KBScanTest::testScanInvalid1 ()
{
	QPtrList<KBItem> errors ;
	m_form->m_form->invalidControls (0, errors, true) ;

	CPPUNIT_ASSERT	(errors.findRef(m_field1) <  0) ;
	CPPUNIT_ASSERT	(errors.findRef(m_field2) <  0) ;
	CPPUNIT_ASSERT	(errors.findRef(m_field3) >= 0) ;
	CPPUNIT_ASSERT	(errors.findRef(m_field4) >= 0) ;
	CPPUNIT_ASSERT	(errors.findRef(m_field5) <  0) ;
	CPPUNIT_ASSERT	(errors.findRef(m_field6) <  0) ;
	CPPUNIT_ASSERT	(errors.findRef(m_field7) >= 0) ;
}

CppUnit::Test
	*KBScanTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Field") ;

	if (rekallTestSkip ("Scan")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBScanTest> ("noDisplay", &KBScanTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBScanTest> ("withGUI",   &KBScanTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBScanTest>
			(	"scanInvalid1",
				&KBScanTest::testScanInvalid1
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *scanTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBScanTest::suite (__settings) ;
}
