/***************************************************************************
    file	         : subform.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBSubFormTest : public CppUnit::TestCase
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;
	KBFormBlock		*m_subform	;

	KBField			*getField	(const char *) ;
	KBControl		*getControl	(KBField    *) ;
	KBControl		*getControl	(KBField    *, uint, uint) ;
	RKLineEdit		*getWidget	(KBControl  *) ;

public	:

	KBSubFormTest	() ;
	KBSubFormTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testSubForm1	() ;
	void			testSubForm2	() ;
	void			testSubForm3	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBSubFormTest::KBSubFormTest ()
{
}

KBSubFormTest::KBSubFormTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBSubFormTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("API.rkl", "test/api") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "api", "SubForm", error) ;

	rekallGUIPause ("SubForm") ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)0,
		m_form->m_form->getCurQRow()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)9,
		m_form->m_form->getNumRows()
	)	;

	KBNode	*subform = m_form->m_form->getNamedNode("subform") ;
	CPPUNIT_ASSERT_MESSAGE
	(	"Subform not found",
		m_form != 0
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	"Subform is not a form block",
		subform->isA("KBFormBlock")
	)	;
	m_subform = (KBFormBlock *)subform ;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)0,
		m_subform->getCurQRow()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)2,
		m_subform->getNumRows()
	)	;

}

void	KBSubFormTest::tearDown ()
{
	rekallTestCloseForm () ;
	delete	m_dbInfo ;
}

void	KBSubFormTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBSubFormTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}


KBField
	*KBSubFormTest::getField
	(	const char	*name
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBField")) ;

	KBField *field = (KBField *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(field->getName())
	)	;

	return	field	;
}

KBControl
	*KBSubFormTest::getControl
	(	KBField		*field
	)
{
	KBControl *control = KBTestAccessor::getControl (field) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	return	  control  ;
}

KBControl
	*KBSubFormTest::getControl
	(	KBField		*field,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (field, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlField")) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

RKLineEdit
	*KBSubFormTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("RKLineEdit")) ;
	return	(RKLineEdit *)widget ;
}

void	KBSubFormTest::testSubForm1 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (1)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)1,
		m_form->m_form->getCurQRow()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)0,
		m_subform->getCurQRow()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)3,
		m_subform->getNumRows()
	)	;
}

void	KBSubFormTest::testSubForm2 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (4)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)4,
		m_form->m_form->getCurQRow()
	)	;
	CPPUNIT_ASSERT
	(	m_subform->gotoQRow (1)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)1,
		m_subform->getCurQRow()
	)	;
}

void	KBSubFormTest::testSubForm3 ()
{
	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (4)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)4,
		m_form->m_form->getCurQRow()
	)	;
	CPPUNIT_ASSERT
	(	m_subform->gotoQRow (1)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)1,
		m_subform->getCurQRow()
	)	;

	CPPUNIT_ASSERT
	(	m_form->m_form->gotoQRow (3)
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)3,
		m_form->m_form->getCurQRow()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(uint)0,
		m_subform->getCurQRow()
	)	;
}

CppUnit::Test
	*KBSubFormTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Field") ;

	if (rekallTestSkip ("SubForm")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBSubFormTest> ("noDisplay", &KBSubFormTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBSubFormTest> ("withGUI",   &KBSubFormTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBSubFormTest>
			(	"subform1",
				&KBSubFormTest::testSubForm1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBSubFormTest>
			(	"subform2",
				&KBSubFormTest::testSubForm2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBSubFormTest>
			(	"subform3",
				&KBSubFormTest::testSubForm3
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *subformTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBSubFormTest::suite (__settings) ;
}
