/***************************************************************************
    file	         : tabber.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_tabber.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBTabberTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

public	:

	KBTabberTest	() ;
	KBTabberTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testTabber1	() ;
	void			testTabber2	() ;
	void			testTabber3	() ;
	void			testTabber4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBTabberTest::KBTabberTest ()
{
}

KBTabberTest::KBTabberTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBTabberTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenForm (0, "api", "Tabber", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBTabberTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBTabberTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBTabberTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBTabberTest::testTabber1 ()
{
	KBNode	*tabber	= m_form->m_form->getNamedNode("Tabber") ;

	CPPUNIT_ASSERT (tabber != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Tabber")),
		_STDSTR(tabber->getName())
	)	;
}

void	KBTabberTest::testTabber2 ()
{
	KBNode	*page1 = m_form->m_form->getNamedNode("Tabber/Page_1") ;
	KBNode	*page2 = m_form->m_form->getNamedNode("Tabber/Page_2") ;

	CPPUNIT_ASSERT (page1 != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_1")),
		_STDSTR(page1->getName())
	)	;

	CPPUNIT_ASSERT (page2 != 0) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_2")),
		_STDSTR(page2->getName())
	)	;
}

void	KBTabberTest::testTabber3 ()
{
	KBNode	*node	= m_form->m_form->getNamedNode("Tabber") ;
	CPPUNIT_ASSERT (node  != 0) ;

	KBTabber *tabber= node->isTabber() ;
	CPPUNIT_ASSERT (tabber != 0) ;

	KBNode  *page   = tabber->currentPage() ;

	CPPUNIT_ASSERT (page != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_1")),
		_STDSTR(page->getName())
	)	;

	KBNode	*button	= page->getNamedNode ("Button_1") ;
	CPPUNIT_ASSERT (button != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Button_1")),
		_STDSTR(button->getName())
	)	;
}

void	KBTabberTest::testTabber4 ()
{
	KBNode	*node	= m_form->m_form->getNamedNode("Tabber") ;
	CPPUNIT_ASSERT (node  != 0) ;

	KBTabber *tabber	= node->isTabber() ;
	CPPUNIT_ASSERT (tabber != 0) ;

	KBNode	*node2 = m_form->m_form->getNamedNode("Tabber/Page_2") ;
	CPPUNIT_ASSERT (node2 != 0) ;

	KBTabberPage *page2 = node2->isTabberPage() ;
	CPPUNIT_ASSERT (page2 != 0) ;

	page2->setCurrent() ;

	KBNode  *page   = tabber->currentPage() ;

	CPPUNIT_ASSERT (page != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Page_2")),
		_STDSTR(page->getName())
	)	;

	KBNode	*button	= page->getNamedNode ("Button_2") ;
	CPPUNIT_ASSERT (button != 0) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Button_2")),
		_STDSTR(button->getName())
	)	;
}


CppUnit::Test
	*KBTabberTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Container") ;

	if (rekallTestSkip ("Tabber")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTabberTest> ("noDisplay", &KBTabberTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTabberTest> ("withGUI",   &KBTabberTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBTabberTest>
			(	"tabber1",
				&KBTabberTest::testTabber1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTabberTest>
			(	"tabber2",
				&KBTabberTest::testTabber2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTabberTest>
			(	"tabber3",
				&KBTabberTest::testTabber3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTabberTest>
			(	"tabber4",
				&KBTabberTest::testTabber4
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *tabberTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBTabberTest::suite (__settings) ;
}
