/***************************************************************************
    file	         : validity.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_field.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"
#include	"api.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBValidityTest : public KBAPITest
{
public	:

	KBValidityTest	() ;
	KBValidityTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testValidity1	() ;
	void			testValidity2	() ;
	void			testValidity3	() ;
	void			testValidity4	() ;
	void			testValidity5	() ;
	void			testValidity6	() ;
	void			testValidity7	() ;
	void			testValidity8	() ;
	void			testValidity9	() ;
	void			testValidity10	() ;
	void			testValidity11	() ;
	void			testValidity12	() ;
	void			testValidity13	() ;
	void			testValidity14	() ;
	void			testValidity15	() ;
	void			testValidity16	() ;
	void			testValidity90	() ;
	void			testValidity91	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBValidityTest::KBValidityTest ()
	:
	KBAPITest ()
{
}

KBValidityTest::KBValidityTest
	(	std::string		name
	)
	:
	KBAPITest (name)
{
}

void	KBValidityTest::setUp ()
{
	openForm ("Validity") ;

	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
	KBField    *field2   = getField   ("Field_t_2", "Field_t_2") ;
	KBField    *field3   = getField   ("Field_t_3", "Field_t_3") ;
	KBField    *field4   = getField   ("Field_t_4", "Field_t_4") ;
	KBField    *field5   = getField   ("Field_t_5", "Field_t_5") ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Yes")),
		_STDSTR(field1->getAttrVal("nullok"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("No")),
		_STDSTR(field2->getAttrVal("nullok"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Yes")),
		_STDSTR(field3->getAttrVal("nullok"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("No")),
		_STDSTR(field4->getAttrVal("nullok"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("No")),
		_STDSTR(field5->getAttrVal("nullok"))
	)	;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("^[a-z]*$")),
		_STDSTR(field3->getAttrVal("evalid"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("^[a-z]*$")),
		_STDSTR(field4->getAttrVal("evalid"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("^[a-z]*$")),
		_STDSTR(field5->getAttrVal("evalid"))
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("Yes")),
		_STDSTR(field5->getAttrVal("igncase"))
	)	;
}

void	KBValidityTest::tearDown ()
{
	rekallTestCloseForm() ;
}

void	KBValidityTest::testValidity1 ()
{
	KBField    *field1   = getField    ("Field_t_1", "Field_t_1") ;
	KBControl  *control1 = getControl  (field1  ) ;
	RKLineEdit *widget1  = getRKLineEdit (control1) ;

	field1->setValue (0, KBValue("")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("")),
		_STDSTR(widget1->text())
	)	;
	CPPUNIT_ASSERT
	(	field1->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity2 ()
{
	KBField    *field2   = getField    ("Field_t_2", "Field_t_2") ;
	KBControl  *control2 = getControl  (field2  ) ;
	RKLineEdit *widget2  = getRKLineEdit (control2) ;

	field2->setValue (0, KBValue("")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("")),
		_STDSTR(widget2->text())
	)	;
	CPPUNIT_ASSERT
	(	!field2->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity3 ()
{
	KBField    *field2   = getField    ("Field_t_2", "Field_t_2") ;
	KBControl  *control2 = getControl  (field2  ) ;
	RKLineEdit *widget2  = getRKLineEdit (control2) ;

	field2->setValue (0, KBValue("some_text")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("some_text")),
		_STDSTR(widget2->text())
	)	;
	CPPUNIT_ASSERT
	(	field2->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity4 ()
{
	KBField    *field3   = getField    ("Field_t_3", "Field_t_3") ;
	KBControl  *control3 = getControl  (field3  ) ;
	RKLineEdit *widget3  = getRKLineEdit (control3) ;

	field3->setValue (0, KBValue("")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("")),
		_STDSTR(widget3->text())
	)	;
	CPPUNIT_ASSERT
	(	field3->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity5 ()
{
	KBField    *field3   = getField    ("Field_t_3", "Field_t_3") ;
	KBControl  *control3 = getControl  (field3  ) ;
	RKLineEdit *widget3  = getRKLineEdit (control3) ;

	field3->setValue (0, KBValue("sometext")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("sometext")),
		_STDSTR(widget3->text())
	)	;
	CPPUNIT_ASSERT
	(	field3->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity6 ()
{
	KBField    *field3   = getField    ("Field_t_3", "Field_t_3") ;
	KBControl  *control3 = getControl  (field3  ) ;
	RKLineEdit *widget3  = getRKLineEdit (control3) ;

	field3->setValue (0, KBValue("SOMETEXT")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("SOMETEXT")),
		_STDSTR(widget3->text())
	)	;
	CPPUNIT_ASSERT
	(	!field3->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity7 ()
{
	KBField    *field4   = getField    ("Field_t_4", "Field_t_4") ;
	KBControl  *control4 = getControl  (field4  ) ;
	RKLineEdit *widget4  = getRKLineEdit (control4) ;

	field4->setValue (0, KBValue("")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("")),
		_STDSTR(widget4->text())
	)	;
	CPPUNIT_ASSERT
	(	!field4->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity8 ()
{
	KBField    *field4   = getField    ("Field_t_4", "Field_t_4") ;
	KBControl  *control4 = getControl  (field4  ) ;
	RKLineEdit *widget4  = getRKLineEdit (control4) ;

	field4->setValue (0, KBValue("sometext")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("sometext")),
		_STDSTR(widget4->text())
	)	;
	CPPUNIT_ASSERT
	(	field4->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity9 ()
{
	KBField    *field4   = getField    ("Field_t_4", "Field_t_4") ;
	KBControl  *control4 = getControl  (field4  ) ;
	RKLineEdit *widget4  = getRKLineEdit (control4) ;

	field4->setValue (0, KBValue("SOMETEXT")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("SOMETEXT")),
		_STDSTR(widget4->text())
	)	;
	CPPUNIT_ASSERT
	(	!field4->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity10 ()
{
	KBField    *field5   = getField    ("Field_t_5", "Field_t_5") ;
	KBControl  *control5 = getControl  (field5  ) ;
	RKLineEdit *widget5  = getRKLineEdit (control5) ;

	field5->setValue (0, KBValue("")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("")),
		_STDSTR(widget5->text())
	)	;
	CPPUNIT_ASSERT
	(	!field5->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity11 ()
{
	KBField    *field5   = getField    ("Field_t_5", "Field_t_5") ;
	KBControl  *control5 = getControl  (field5  ) ;
	RKLineEdit *widget5  = getRKLineEdit (control5) ;

	field5->setValue (0, KBValue("sometext")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("sometext")),
		_STDSTR(widget5->text())
	)	;
	CPPUNIT_ASSERT
	(	field5->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity12 ()
{
	KBField    *field5   = getField    ("Field_t_5", "Field_t_5") ;
	KBControl  *control5 = getControl  (field5  ) ;
	RKLineEdit *widget5  = getRKLineEdit (control5) ;

	field5->setValue (0, KBValue("SOMETEXT")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("SOMETEXT")),
		_STDSTR(widget5->text())
	)	;
	CPPUNIT_ASSERT
	(	field5->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity13 ()
{
	KBField    *field6   = getField    ("Field_t_6", "Field_t_6") ;
	KBControl  *control6 = getControl  (field6  ) ;
	RKLineEdit *widget6  = getRKLineEdit (control6) ;

	field6->setValue (0, KBValue()) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("")),
		_STDSTR(widget6->text())
	)	;
	CPPUNIT_ASSERT
	(	!field6->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity14 ()
{
	KBField    *field7   = getField    ("Field_t_7", "Field_t_7") ;
	KBControl  *control7 = getControl  (field7  ) ;
	RKLineEdit *widget7  = getRKLineEdit (control7) ;

	field7->setValue (0, KBValue()) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("")),
		_STDSTR(widget7->text())
	)	;
	CPPUNIT_ASSERT
	(	!field7->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity15 ()
{
	KBField    *field6   = getField    ("Field_t_6", "Field_t_6") ;
	KBControl  *control6 = getControl  (field6  ) ;
	RKLineEdit *widget6  = getRKLineEdit (control6) ;

	field6->setValue (0, KBValue("SOMETEXT")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("SOMETEXT")),
		_STDSTR(widget6->text())
	)	;
	CPPUNIT_ASSERT
	(	field6->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity16 ()
{
	KBField    *field7   = getField    ("Field_t_7", "Field_t_7") ;
	KBControl  *control7 = getControl  (field7  ) ;
	RKLineEdit *widget7  = getRKLineEdit (control7) ;

	field7->setValue (0, KBValue("SOMETEXT")) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("SOMETEXT")),
		_STDSTR(widget7->text())
	)	;
	CPPUNIT_ASSERT
	(	field7->isValid (0, false)
	)	;
}

void	KBValidityTest::testValidity90 ()
{
	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
	KBField    *field2   = getField   ("Field_t_2", "Field_t_2") ;
	KBField    *field3   = getField   ("Field_t_3", "Field_t_3") ;
	KBField    *field4   = getField   ("Field_t_4", "Field_t_4") ;
	KBField    *field5   = getField   ("Field_t_5", "Field_t_5") ;
	KBField    *field6   = getField   ("Field_t_6", "Field_t_6") ;
	KBValue	   null	     ;

	field1->setValue (0, KBValue("")) ;
	field2->setValue (0, KBValue("")) ;
	field3->setValue (0, KBValue("SOMETEXT")) ;
	field4->setValue (0, KBValue("SOMETEXT")) ;
	field5->setValue (0, KBValue("SOMETEXT")) ;
	field6->setValue (0, null) ;

	rekallGUIPause	 ("testValidity90") ;

	QPtrList<KBItem> invalid ;
	m_form->m_form->invalidControls (0, invalid, true) ;

	CPPUNIT_ASSERT(invalid.find (field1) <  0) ;
	CPPUNIT_ASSERT(invalid.find (field2) >= 0) ;
	CPPUNIT_ASSERT(invalid.find (field3) >= 0) ;
	CPPUNIT_ASSERT(invalid.find (field4) >= 0) ;
	CPPUNIT_ASSERT(invalid.find (field5) <  0) ;
	CPPUNIT_ASSERT(invalid.find (field6) <  0) ;
}

void	KBValidityTest::testValidity91 ()
{
	KBField    *field1   = getField   ("Field_t_1", "Field_t_1") ;
	KBField    *field2   = getField   ("Field_t_2", "Field_t_2") ;
	KBField    *field3   = getField   ("Field_t_3", "Field_t_3") ;
	KBField    *field4   = getField   ("Field_t_4", "Field_t_4") ;
	KBField    *field5   = getField   ("Field_t_5", "Field_t_5") ;
	KBField    *field6   = getField   ("Field_t_6", "Field_t_6") ;
	KBValue	   null	     ;

	field1->setValue (0, KBValue("")) ;
	field2->setValue (0, KBValue("")) ;
	field3->setValue (0, KBValue("SOMETEXT")) ;
	field4->setValue (0, KBValue("SOMETEXT")) ;
	field5->setValue (0, KBValue("SOMETEXT")) ;
	field6->setValue (0, null) ;

	rekallGUIPause	 ("testValidity90") ;

	QPtrList<KBItem> invalid ;
	m_form->m_form->invalidControls (0, invalid, false) ;

	CPPUNIT_ASSERT(invalid.find (field1) <  0) ;
	CPPUNIT_ASSERT(invalid.find (field2) >= 0) ;
	CPPUNIT_ASSERT(invalid.find (field3) >= 0) ;
	CPPUNIT_ASSERT(invalid.find (field4) >= 0) ;
	CPPUNIT_ASSERT(invalid.find (field5) <  0) ;
	CPPUNIT_ASSERT(invalid.find (field6) <  0) ;
}

CppUnit::Test
	*KBValidityTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Field") ;

	if (rekallTestSkip ("Validity")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBValidityTest> ("noDisplay", &KBValidityTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBValidityTest> ("withGUI",   &KBValidityTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity1",
				&KBValidityTest::testValidity1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity2",
				&KBValidityTest::testValidity2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity3",
				&KBValidityTest::testValidity3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity4",
				&KBValidityTest::testValidity4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity5",
				&KBValidityTest::testValidity5
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity6",
				&KBValidityTest::testValidity6
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity7",
				&KBValidityTest::testValidity7
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity8",
				&KBValidityTest::testValidity8
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity9",
				&KBValidityTest::testValidity9
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity10",
				&KBValidityTest::testValidity10
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity11",
				&KBValidityTest::testValidity11
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity12",
				&KBValidityTest::testValidity12
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity13",
				&KBValidityTest::testValidity13
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity14",
				&KBValidityTest::testValidity14
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity15",
				&KBValidityTest::testValidity15
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity16",
				&KBValidityTest::testValidity16
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity90",
				&KBValidityTest::testValidity90
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBValidityTest>
			(	"validity91",
				&KBValidityTest::testValidity91
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *validityTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBValidityTest::suite (__settings) ;
}
