/***************************************************************************
    file	         : base64.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qdom.h>
#include	<qmap.h>

#include	"kb_base64.h"
#include	"kb_databuffer.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBBase64Test : public CppUnit::TestCase
{
public	:

	KBBase64Test	() ;
	KBBase64Test	(std::string name) ;


	void			testNeeded1	() ;
	void			testNeeded2	() ;
	void			testEncode1	() ;
	void			testEncode2	() ;
	void			testDecode1	() ;
	void			testDecode2	() ;
	void			testBothWays1	() ;
	void			testBothWays2	() ;
	void			testBothWays3	() ;

	static CppUnit::Test	*suite		() ;
}	;


KBBase64Test::KBBase64Test ()
{
}

KBBase64Test::KBBase64Test
	(	std::string		name
	)
	:
	CppUnit::TestCase 	(name)
{
}

void	KBBase64Test::testNeeded1 ()
{
	static const unsigned char data[] = { 'A', 'B', 'C', 12  } ;
	CPPUNIT_ASSERT ( kbB64Needed (data, sizeof(data))) ;
}

void	KBBase64Test::testNeeded2 ()
{
	static const unsigned char data[] = { 'A', 'B', 'C', 'D' } ;
	CPPUNIT_ASSERT (!kbB64Needed (data, sizeof(data))) ;
}

void	KBBase64Test::testEncode1 ()
{
	static const unsigned char data[] = "Aladdin:open sesame" ;
	static const unsigned char resv[] = "QWxhZGRpbjpvcGVuIHNlc2FtZQA=" ;

	KBDataBuffer buff ;
	kbB64Encode (data, sizeof(data), buff) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("[%1] became [%2]").arg((char *)resv).arg((char *)buff.data())),
		memcmp (buff.data(), resv, sizeof(resv)),
		0
	)	;
}

void	KBBase64Test::testEncode2 ()
{
	static const unsigned char data[] = "Aladdin:open sesame12" ;
	static const unsigned char resv[] = "QWxhZGRpbjpvcGVuIHNlc2FtZTEyAA==" ;

	KBDataBuffer buff ;
	kbB64Encode (data, sizeof(data), buff) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("[%1] became [%2]").arg((char *)resv).arg((char *)buff.data())),
		memcmp (buff.data(), resv, sizeof(resv)),
		0
	)	;
}

void	KBBase64Test::testDecode1 ()
{
	static const unsigned char data[] = "QWxhZGRpbjpvcGVuIHNlc2FtZTEy" ;
	static const unsigned char resv[] = "Aladdin:open sesame12" ;

	KBDataBuffer buff ;
	kbB64Decode (data, sizeof(data), buff) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("[%1] became [%2]").arg((char *)resv).arg((char *)buff.data())),
		memcmp (buff.data(), resv, sizeof(resv)),
		0
	)	;
}

void	KBBase64Test::testDecode2 ()
{
	static const unsigned char data[] = "QWxhZGRpbjpvcGVuIHNlc2FtZTEyAA==" ;
	static const unsigned char resv[] = "Aladdin:open sesame12" ;

	KBDataBuffer buff ;
	kbB64Decode (data, sizeof(data), buff) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("[%1] became [%2]").arg((char *)resv).arg((char *)buff.data())),
		memcmp (buff.data(), resv, sizeof(resv)),
		0
	)	;
}

void	KBBase64Test::testBothWays1 ()
{
	static const unsigned char data[] = "Aladdin:open sesame" ;

	KBDataBuffer buff1 ;
	KBDataBuffer buff2 ;

	kbB64Encode (data, sizeof(data), buff1) ;
	kbB64Decode ((const unsigned char *)buff1.data(), buff1.length(), buff2) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("[%1] became [%2]").arg((char *)data).arg((char *)buff2.data())),
		memcmp (buff2.data(), data, sizeof(data)),
		0
	)	;
}

void	KBBase64Test::testBothWays2 ()
{
	static const unsigned char data[] = "Aladdin:\x01open sesame" ;

	KBDataBuffer buff1 ;
	KBDataBuffer buff2 ;

	kbB64Encode (data, sizeof(data), buff1) ;
	kbB64Decode ((const unsigned char *)buff1.data(), buff1.length(), buff2) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("[%1] became [%2]").arg((char *)data).arg((char *)buff2.data())),
		memcmp (buff2.data(), data, sizeof(data)),
		0
	)	;
}

void	KBBase64Test::testBothWays3 ()
{
	static const unsigned char data[] = "Aladdin:\x01open \x8fsesame" ;

	KBDataBuffer buff1 ;
	KBDataBuffer buff2 ;

	kbB64Encode (data, sizeof(data), buff1) ;
	kbB64Decode ((const unsigned char *)buff1.data(), buff1.length(), buff2) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("[%1] became [%2]").arg((char *)data).arg((char *)buff2.data())),
		memcmp (buff2.data(), data, sizeof(data)),
		0
	)	;
}

CppUnit::Test
	*KBBase64Test::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Base64") ;

	if (rekallTestSkip("Base64")) return s ;

	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("needed_1",   &KBBase64Test::testNeeded1  )) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("needed_2",   &KBBase64Test::testNeeded2  )) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("encode_1",   &KBBase64Test::testEncode1  )) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("encode_2",   &KBBase64Test::testEncode2  )) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("decode_1",   &KBBase64Test::testDecode1  )) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("decode_2",   &KBBase64Test::testDecode2  )) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("bothways_1", &KBBase64Test::testBothWays1)) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("bothways_2", &KBBase64Test::testBothWays2)) ;
	s->addTest (new CppUnit::TestCaller<KBBase64Test> ("bothways_3", &KBBase64Test::testBothWays3)) ;
	return s ;
}

CppUnit::Test EXPORT_API *base64TestSuite
	(	const QMap<QString,QString> *
	)
{
	return	KBBase64Test::suite () ;
}

