/***************************************************************************
    file	         : baseSelect.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qdom.h>
#include	<qmap.h>

#include	"kb_basequery.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBBaseSelectTest : public CppUnit::TestCase
{
public	:

	KBBaseSelectTest	() ;
	KBBaseSelectTest	(std::string name) ;

	void			testBaseSelect1	() ;
	void			testBaseSelect2	() ;
	void			testBaseSelect3	() ;
	void			testBaseSelect4	() ;

	static CppUnit::Test	*suite		() ;
}	;


KBBaseSelectTest::KBBaseSelectTest ()
{
}

KBBaseSelectTest::KBBaseSelectTest
	(	std::string		name
	)
	:
	CppUnit::TestCase 	(name)
{
}

void	KBBaseSelectTest::testBaseSelect1 ()
{
	KBBaseSelect select ;
	select.parse  ("select a from b limit 10", 0) ;
	CPPUNIT_ASSERT_EQUAL (10, select.limit ()) ;
	CPPUNIT_ASSERT_EQUAL (-1, select.offset()) ;
}

void	KBBaseSelectTest::testBaseSelect2 ()
{
	KBBaseSelect select ;
	select.parse ("select a from b offset 10", 0) ;
	CPPUNIT_ASSERT_EQUAL (-1, select.limit ()) ;
	CPPUNIT_ASSERT_EQUAL (10, select.offset()) ;
}

void	KBBaseSelectTest::testBaseSelect3 ()
{
	KBBaseSelect select ;
	select.parse ("select a from b limit 5 offset 6", 0) ;
	CPPUNIT_ASSERT_EQUAL (5, select.limit ()) ;
	CPPUNIT_ASSERT_EQUAL (6, select.offset()) ;
}

void	KBBaseSelectTest::testBaseSelect4 ()
{
	KBBaseSelect select ;
	select.parse ("select a from b limit 3,7", 0) ;
	CPPUNIT_ASSERT_EQUAL (7, select.limit ()) ;
	CPPUNIT_ASSERT_EQUAL (3, select.offset()) ;
}

CppUnit::Test
	*KBBaseSelectTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("BaseSelect") ;

	if (rekallTestSkip("BaseSelect")) return s ;

	s->addTest (new CppUnit::TestCaller<KBBaseSelectTest> ("baseSelect_1",   &KBBaseSelectTest::testBaseSelect1 )) ;
	s->addTest (new CppUnit::TestCaller<KBBaseSelectTest> ("baseSelect_2",   &KBBaseSelectTest::testBaseSelect2 )) ;
	s->addTest (new CppUnit::TestCaller<KBBaseSelectTest> ("baseSelect_3",   &KBBaseSelectTest::testBaseSelect3 )) ;
	s->addTest (new CppUnit::TestCaller<KBBaseSelectTest> ("baseSelect_4",   &KBBaseSelectTest::testBaseSelect4 )) ;
	return s ;
}

CppUnit::Test EXPORT_API *baseSelectTestSuite
	(	const QMap<QString,QString> *
	)
{
	return	KBBaseSelectTest::suite () ;
}

