/***************************************************************************
    file	         : dbdociter.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmap.h>

#include	"kb_locator.h"
#include	"kb_dbdociter.h"
#include	"kb_dbinfo.h"
#include	"kb_serverinfo.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


class	KBDBDocIterTest : public CppUnit::TestCase
{
	KBDBInfo	*m_dbInfo	;
	KBServerInfo	*m_svInfo_f	;
	KBServerInfo	*m_svInfo_t	;

public	:

	KBDBDocIterTest	() ;
	KBDBDocIterTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testDBDocIter1	() ;
	void			testDBDocIter2	() ;
	void			testDBDocIter3	() ;
	void			testDBDocIter4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

static	const QMap<QString,QString> *settings	;


KBDBDocIterTest::KBDBDocIterTest ()
{
}

KBDBDocIterTest::KBDBDocIterTest
	(	std::string	name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBDBDocIterTest::setUp ()
{
	const char *tp	= getenv("TESTPATH") ;
	QString	path	;

	if (tp == 0)
		path	= locateFile("appdata", "test/common/Form1.rkl.frm") ;
	else	path	= QString("%1/%2").arg(tp).arg("test/common/Form1.rkl.frm") ;

	CPPUNIT_ASSERT(!path.isEmpty()) ;

	m_dbInfo	= new KBDBInfo	(path) ;
	m_dbInfo->init (false) ;

	CPPUNIT_ASSERT(!m_dbInfo->getDBPath().isEmpty()) ;

	m_svInfo_f	= new KBServerInfo
				(	m_dbInfo,
					"!Files",
					"test",
					QString::null,
					QString::null,
					"user",
					"pass",
					QString::null,
					QString::null
				)	;
	m_svInfo_t	= new KBServerInfo
				(	m_dbInfo,
					"Test",
					"test",
					QString::null,
					QString::null,
					"user",
					"pass",
					QString::null,
					QString::null
				)	;
	m_dbInfo->insert (m_svInfo_f) ;
	m_dbInfo->insert (m_svInfo_t) ;
}

void	KBDBDocIterTest::tearDown ()
{
	delete	m_dbInfo	;
}

void	KBDBDocIterTest::testDBDocIter1 ()
{
	KBError		error	;
	KBDBDocIter	dbIter	;

	if (!dbIter.init (m_dbInfo, "!Files", "form", "frm", error))
		CPPUNIT_FAIL
		(	_STDSTR
			(	QString("%1, %2")
					.arg(error.getMessage())
					.arg(error.getDetails())
		)	)	;

	QString		name	;
	QString		stamp	;
	QStringList	results	;

	while (dbIter.getNextDoc (name, stamp))
		results.append	(name) ;

	CPPUNIT_ASSERT_EQUAL(3, (int)results.count()) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("Form1")), _STDSTR(results[0])) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("Form2")), _STDSTR(results[1])) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("Form3")), _STDSTR(results[2])) ;
}

void	KBDBDocIterTest::testDBDocIter2 ()
{
	KBError		error	;
	KBDBDocIter	dbIter	;

	if (!dbIter.init (m_dbInfo, "!Files", "form", "frm", error))
		CPPUNIT_FAIL
		(	_STDSTR
			(	QString("%1, %2")
					.arg(error.getMessage())
					.arg(error.getDetails())
		)	)	;

	QString		name	;
	QString		stamp	;
	QString		extn	;
	QStringList	results	;

	while (dbIter.getNextDoc (name, stamp, &extn))
		results.append	(extn) ;

	CPPUNIT_ASSERT_EQUAL(3, (int)results.count()) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("rkl.frm")), _STDSTR(results[0])) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("rkl.frm")), _STDSTR(results[1])) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("rkl.frm")), _STDSTR(results[2])) ;
}

void	KBDBDocIterTest::testDBDocIter3 ()
{
	KBError		error	;
	KBDBDocIter	dbIter	;

	if (!dbIter.init (m_dbInfo, "Test", "form", "frm", error))
		CPPUNIT_FAIL
		(	_STDSTR
			(	QString("%1, %2")
					.arg(error.getMessage())
					.arg(error.getDetails())
		)	)	;

	QString		name	;
	QString		stamp	;
	QStringList	results	;

	while (dbIter.getNextDoc (name, stamp))
		results.append	(name) ;

	CPPUNIT_ASSERT_EQUAL(2, (int)results.count()) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("One")), _STDSTR(results[0])) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString("Two")), _STDSTR(results[1])) ;
}

void	KBDBDocIterTest::testDBDocIter4 ()
{
	KBError		error	;
	KBDBDocIter	dbIter	;

	if (!dbIter.init (m_dbInfo, "Test", "form", "frm", error))
		CPPUNIT_FAIL
		(	_STDSTR
			(	QString("%1, %2")
					.arg(error.getMessage())
					.arg(error.getDetails())
		)	)	;

	QString		name	;
	QString		stamp	;
	QString		extn	;
	QStringList	results	;

	while (dbIter.getNextDoc (name, stamp, &extn))
		results.append	(extn) ;

	CPPUNIT_ASSERT_EQUAL(2, (int)results.count()) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString::null), _STDSTR(results[0])) ;
	CPPUNIT_ASSERT_EQUAL(_STDSTR(QString::null), _STDSTR(results[1])) ;
}

CppUnit::Test
	*KBDBDocIterTest::suite
	(	const QMap<QString,QString> *__settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("DBDocIter") ;

	if (rekallTestSkip("DBDocIter")) return s ;

	settings = __settings ;
	s->addTest (new CppUnit::TestCaller<KBDBDocIterTest> ("DBDocIter1",   &KBDBDocIterTest::testDBDocIter1)) ;
	s->addTest (new CppUnit::TestCaller<KBDBDocIterTest> ("DBDocIter2",   &KBDBDocIterTest::testDBDocIter2)) ;
	s->addTest (new CppUnit::TestCaller<KBDBDocIterTest> ("DBDocIter3",   &KBDBDocIterTest::testDBDocIter3)) ;
	s->addTest (new CppUnit::TestCaller<KBDBDocIterTest> ("DBDocIter4",   &KBDBDocIterTest::testDBDocIter4)) ;
	return s ;
}

CppUnit::Test EXPORT_API *dbDocIterTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBDBDocIterTest::suite (__settings) ;
}

