/***************************************************************************
    file	         : location.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmap.h>

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_file.h"
#include	"kb_locator.h"
#include	"kb_dblink.h"
#include	"kb_location.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


class	KBLocationTest : public CppUnit::TestCase
{
public	:

	KBLocationTest	() ;
	KBLocationTest	(std::string name) ;


	void			testLocationSimple1	() ;
	void			testLocationSimple2	() ;
	void			testLocationSimple3	() ;
	void			testLocationSimple4	() ;
	void			testLocationSimple5	() ;
	void			testLocationSimple6	() ;
	void			testLocationSimple7	() ;
	void			testLocationSimple8	() ;
	void			testLocationSimple9	() ;
	void			testLocationSimple10	() ;
	void			testLocationSimple11	() ;
	void			testLocationInfo1	() ;
	void			testLocationInfo2	() ;
	void			testLocationData1	() ;
	void			testLocationData2	() ;
	void			testLocationData3	() ;
	void			testLocationInsert1	() ;
	void			testLocationInsert2	() ;
	void			testLocationInsert3	() ;
	void			testLocationUpdate1	() ;
	void			testLocationUpdate2	() ;
	void			testLocationDelete1	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

static	const QMap<QString,QString> *settings	;


KBLocationTest::KBLocationTest ()
{
}

KBLocationTest::KBLocationTest
	(	std::string	name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBLocationTest::testLocationSimple1 ()
{
	KBLocation locn
		(	0,
			"v_type",
			"v_server",
			"v_name",
			"v_ext"
		)	;

	CPPUNIT_ASSERT	(locn.type      () == "v_type"  ) ;
	CPPUNIT_ASSERT	(locn.server    () == "v_server") ;
	CPPUNIT_ASSERT	(locn.name      () == "v_name"  ) ;
	CPPUNIT_ASSERT	(locn.extension () == "v_ext"   ) ;
	CPPUNIT_ASSERT	(locn.dataServer().isNull()	) ;
}

void	KBLocationTest::testLocationSimple2 ()
{
	KBLocation locn
		(	0,
			"v_type",
			"v_server",
			"v_name",
			"v_ext"
		)	;

	locn.setServer	("s_server")	;
	locn.setName	("s_name"  )	;

	CPPUNIT_ASSERT	(locn.type     () == "v_type"  ) ;
	CPPUNIT_ASSERT	(locn.server   () == "s_server") ;
	CPPUNIT_ASSERT	(locn.name     () == "s_name"  ) ;
	CPPUNIT_ASSERT	(locn.extension() == "v_ext"   ) ;
}

void	KBLocationTest::testLocationSimple3 ()
{
	KBLocation locn
		(	0,
			"v_type",
			KBLocation::m_pFile,
			"v_name",
			"v_ext"
		)	;

	CPPUNIT_ASSERT	( locn.isFile  ()) ;
	CPPUNIT_ASSERT	(!locn.isInline()) ;
	CPPUNIT_ASSERT	(!locn.isStock ()) ;
	CPPUNIT_ASSERT	(!locn.isInDB  ()) ;
}

void	KBLocationTest::testLocationSimple4 ()
{
	KBLocation locn
		(	0,
			"v_type",
			KBLocation::m_pInline,
			"v_name",
			"v_ext"
		)	;

	CPPUNIT_ASSERT	(!locn.isFile  ()) ;
	CPPUNIT_ASSERT	( locn.isInline()) ;
	CPPUNIT_ASSERT	(!locn.isStock ()) ;
	CPPUNIT_ASSERT	(!locn.isInDB  ()) ;
}

void	KBLocationTest::testLocationSimple5 ()
{
	KBLocation locn
		(	0,
			"v_type",
			KBLocation::m_pStock,
			"v_name",
			"v_ext"
		)	;

	CPPUNIT_ASSERT	(!locn.isFile  ()) ;
	CPPUNIT_ASSERT	(!locn.isInline()) ;
	CPPUNIT_ASSERT	( locn.isStock ()) ;
	CPPUNIT_ASSERT	(!locn.isInDB  ()) ;
}

void	KBLocationTest::testLocationSimple6 ()
{
	KBLocation locn
		(	0,
			"v_type",
			"d_server",
			"v_name",
			"v_ext"
		)	;

	CPPUNIT_ASSERT	(!locn.isFile  ()) ;
	CPPUNIT_ASSERT	(!locn.isInline()) ;
	CPPUNIT_ASSERT	(!locn.isStock ()) ;
	CPPUNIT_ASSERT	( locn.isInDB  ()) ;
}

void	KBLocationTest::testLocationSimple7 ()
{
	KBLocation locn
		(	0,
			"v_type",
			KBLocation::m_pInline,
			"v_name",
			"inline_text"
		)	;
	QByteArray data ;
	KBError	   error;

	CPPUNIT_ASSERT	(locn.contents(data, error)) ;
	CPPUNIT_ASSERT	(QString::fromAscii(data.data(), data.size()) == "inline_text") ;
	CPPUNIT_ASSERT	(locn.contents(error) == "inline_text") ;
}

void	KBLocationTest::testLocationSimple8 ()
{
	bool	   rc		;
	KBError	   error	;

	CPPUNIT_ASSERT (!(*settings)["tmp"].isNull()) ;

	KBLocation locnWrite
		   (	0,
			"form",
			KBLocation::m_pLocal,
			(*settings)["tmp"] + "/kbLocationFile",
			"frm"
		   )	;

	rc = locnWrite.save (QString::null, QString::null, "Some Text", error) ;
	if (!rc)
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBLocation locnRead
		   (	0,
			"form",
			KBLocation::m_pLocal,
			(*settings)["tmp"] + "/kbLocationFile",
			"frm"
		   )	;

	QString	   text	= locnRead.contents (error) ;
	if (text.isNull())
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	CPPUNIT_ASSERT (text == "Some Text") ;
}

void	KBLocationTest::testLocationSimple9 ()
{
	KBError	   error ;
	KBLocation locnRead
		   (	0,
			"component",
			KBLocation::m_pStock,
			"py/Buttons/Add",
			"cmp"
		   )	;
	QByteArray comp ;
	bool	   rc	= locnRead.contents (comp, error) ;
	if (!rc)
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	QString	   name = locateFile ("appdata", "stock/component/py/Buttons/Add.cmp") ;
	CPPUNIT_ASSERT (!name.isEmpty()) ;

	KBFile     file (name) ;
	CPPUNIT_ASSERT	(file.open(IO_ReadOnly)) ;

	QByteArray data = file.readAll() ;
	if (data != comp)
	{	QCString cs (comp.data(), comp.size()) ;
		QCString ds (data.data(), data.size()) ;
		fprintf
		(	stderr,
			"[\n%s\n]\n[\n%s]\n",
			cs.data(),
			ds.data()
		)	;
	}
	CPPUNIT_ASSERT (data == comp) ;
}

void	KBLocationTest::testLocationSimple10 ()
{
	KBLocation locn
		(	0,
			"v_type",
			"v_server",
			"v_name",
			"v_ext"
		)	;

	locn.setServer	   ("s_server")	;
	locn.setName	   ("s_name"  )	;
	locn.setDataServer ("d_server") ;

	CPPUNIT_ASSERT	(locn.type      () == "v_type"  ) ;
	CPPUNIT_ASSERT	(locn.server    () == "s_server") ;
	CPPUNIT_ASSERT	(locn.dataServer() == "d_server") ;
	CPPUNIT_ASSERT	(locn.name      () == "s_name"  ) ;
	CPPUNIT_ASSERT	(locn.extension () == "v_ext"   ) ;
}

void	KBLocationTest::testLocationSimple11 ()
{
	KBLocation locn
		(	0,
			"v_type",
			"v_server",
			"v_name",
			"v_ext"
		)	;

	locn.setServer	   ("s_server")	;
	locn.setName	   ("s_name"  )	;
	locn.setDataServer ("d_server") ;

	KBLocation locn2(locn) ;
	CPPUNIT_ASSERT	(locn2.type      () == "v_type"  ) ;
	CPPUNIT_ASSERT	(locn2.server    () == "s_server") ;
	CPPUNIT_ASSERT	(locn2.dataServer() == "d_server") ;
	CPPUNIT_ASSERT	(locn2.name      () == "s_name"  ) ;
	CPPUNIT_ASSERT	(locn2.extension () == "v_ext"   ) ;
}

void	KBLocationTest::testLocationInfo1 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName",
			"ext"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildInfoQuery (dbLink) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select Id, Description, Definition, SaveDate, Type, Name, Extension from RekallObjects where Type = ? and Name = ?")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationInfo2 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildInfoQuery (dbLink) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select Id, Description, Definition, SaveDate, Type, Name, Extension from RekallObjects where Type = ? and Name = ?")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationData1 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName",
			"ext"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildDataQuery (dbLink, "Definition", false) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select Definition from RekallObjects where Name = ? and Type = ?")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationData2 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName",
			"ext"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildDataQuery (dbLink, "SaveDate", false) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select SaveDate from RekallObjects where Name = ? and Type = ?")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationData3 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName",
			"ext"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildDataQuery (dbLink, "Definition", true) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("select Definition from RekallObjects where Name = ? and Type = ? and Extension = ?")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationInsert1 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName",
			"ext"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildInsertQuery (dbLink, false) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("insert into RekallObjects (Id, Description, Definition, SaveDate, Type, Name, Extension) values (?, null, ?, ?, ?, ?, ?)")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationInsert2 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName",
			"ext"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildInsertQuery (dbLink, true) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("insert into RekallObjects (Description, Definition, SaveDate, Type, Name, Extension) values (null, ?, ?, ?, ?, ?)")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationInsert3 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildInsertQuery (dbLink, true) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("insert into RekallObjects (Description, Definition, SaveDate, Type, Name) values (null, ?, ?, ?, ?)")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationUpdate1 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName",
			"ext"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildUpdateQuery (dbLink) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("update RekallObjects set Definition = ?, SaveDate = ? where Type = ? and Name = ? and Extension = ?")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationUpdate2 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName"
		   )		;
	KBDBLink   dbLink	;

	QString	   insert = locnForm.buildUpdateQuery (dbLink) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("update RekallObjects set Definition = ?, SaveDate = ? where Type = ? and Name = ?")),
		_STDSTR(insert)
	)	;
}

void	KBLocationTest::testLocationDelete1 ()
{
	KBLocation locnForm
		   (	0,
			"type",
			"anyServer",
			"anyName"
		   )		;
	KBDBLink   dbLink	;

	QString	   delrec = locnForm.buildDeleteQuery (dbLink) ;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString("delete from RekallObjects where Name = ? and Type = ?")),
		_STDSTR(delrec)
	)	;
}

CppUnit::Test
	*KBLocationTest::suite
	(	const QMap<QString,QString> *__settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Location") ;

	if (rekallTestSkip("Location")) return s ;

	settings = __settings ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple1",   &KBLocationTest::testLocationSimple1 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple2",   &KBLocationTest::testLocationSimple2 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple3",   &KBLocationTest::testLocationSimple3 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple4",   &KBLocationTest::testLocationSimple4 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple5",   &KBLocationTest::testLocationSimple5 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple6",   &KBLocationTest::testLocationSimple6 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple7",   &KBLocationTest::testLocationSimple7 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple8",   &KBLocationTest::testLocationSimple8 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple9",   &KBLocationTest::testLocationSimple9 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple10",  &KBLocationTest::testLocationSimple10)) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Simple11",  &KBLocationTest::testLocationSimple11)) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Data1",     &KBLocationTest::testLocationData1   )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Data2",     &KBLocationTest::testLocationData2   )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Data3",     &KBLocationTest::testLocationData3   )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Info1",     &KBLocationTest::testLocationInfo1   )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Info2",     &KBLocationTest::testLocationInfo2   )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Insert1",   &KBLocationTest::testLocationInsert1 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Insert2",   &KBLocationTest::testLocationInsert2 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Insert3",   &KBLocationTest::testLocationInsert3 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Update1",   &KBLocationTest::testLocationUpdate1 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Update2",   &KBLocationTest::testLocationUpdate2 )) ;
	s->addTest (new CppUnit::TestCaller<KBLocationTest> ("Delete1",   &KBLocationTest::testLocationDelete1 )) ;
	return s ;
}

CppUnit::Test EXPORT_API *locationTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBLocationTest::suite (__settings) ;
}

