/***************************************************************************
    file	         : tester.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/TestResult.h>
#include	<cppunit/TestRunner.h>
#include	<cppunit/extensions/TestFactoryRegistry.h>
#include	<cppunit/ui/text/TextTestRunner.h>
#include	<cppunit/ui/qt/QtTestRunner.h>
#include	<cppunit/BriefTestProgressListener.h>
#include	<cppunit/XmlOutputter.h>
#include	<cppunit/XmlOutputterHook.h>
#include	<cppunit/tools/XmlDocument.h>
#include	<cppunit/tools/XmlElement.h>
#include	<cppunit/tools/StringTools.h>

#include	<fstream>
 
#include	<qapplication.h>
#include	<qfileinfo.h>
#include	<qmap.h>
#include	<qmessagebox.h>

#if		__KB_KDE
#include	<kapp.h>
#include	<klocale.h>
#include	<kglobal.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>
#include	<kinstance.h>
#endif

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


#include	"rk_application.h"
#include	"kb_classes.h"
#include	"kb_error.h"
#include	"kb_serverinfo.h"
#include	"kb_dbinfo.h"
#include	"kb_dom.h"
#include	"kb_callback.h"
#include	"kb_appptr.h"

#include	<time.h>


class	XmlOutputterHook : public CppUnit::XmlOutputterHook
{
	std::string	m_projectName	;
	std::string	m_author	;

public	:

	XmlOutputterHook
	(	const std::string	projectName,
		const std::string	author
	)
	{
		m_projectName	= projectName	;
		m_author	= author	;
   	}
 
	virtual ~XmlOutputterHook()
	{
	}
 
	void	beginDocument
		(	CppUnit::XmlDocument	*document
		)
	{
		if (document)
		{
			std::string	    szDate  = CppUnit::StringTools::toString((int)time(0)) ;
			CppUnit::XmlElement *metaEl = new CppUnit::XmlElement("SuiteInfo", "")	   ;

			metaEl->addElement  (new CppUnit::XmlElement("Author",  m_author     )) ;
			metaEl->addElement  (new CppUnit::XmlElement("Project", m_projectName)) ;
			metaEl->addElement  (new CppUnit::XmlElement("Date",    szDate 	     )) ;
    
			document->rootElement().addElement(metaEl) ;
		}
	}
}	;

static	bool			__verbose	;
static	const char 		*__cmd		= "rkt_unknown" ;
static	const char		*__rootd	;
static	QMap<QString,QString>	*__skip		;
static	QMap<QString,QString>	*__only		;
static	QMap<QString,QString>	*__pause	;

static	bool			nocrash		;


EXPORT_API
	bool	rekallTestVerbose ()
{
	return	__verbose ;
}

EXPORT_API
	const char *rekallTestCommand ()
{
	return	__cmd	;
}

EXPORT_API
	void	rekallGUIPause
	(	const QString	&text
	)
{
	if (__pause != 0)
		if (text.isNull() || (*__pause)[text].toInt())
		{
			qApp->processEvents() ;
			QMessageBox::information
			(	qApp->activeWindow(),
				QString("Pause: %1").arg(text),
				"Continue ...."
			)	;
		}
}

EXPORT_API
	bool	rekallTestSkip
	(	const QString	&text
	)
{
//	fprintf	(stderr, "rekallTestSkip [%s]\n", text.latin1()) ;

	if (__skip != 0)
		if ( (*__skip)[text].toInt())
			return	true	;

	if (__only != 0)
		if (!(*__only)[text].toInt())
			return	true	;

	return	false	;
}

EXPORT_API
	QString	rekallTestError
	(	const KBError	&error
	)
{
	return	error.getMessage() + ": " + error.getDetails() ;
}

static	RKApplication
	*rekallTestSetupApp
	(	int		argc,
		const char	**argv
	)
{
#if	__KB_KDE
	char	*dummy_v[32]	;
	int	dummy_c		;

	memset	(dummy_v, 0, sizeof(dummy_v))	;
	dummy_v[0] = (char *)__KB_APPNAME	;
	dummy_c	   = 1				;

	if (nocrash)
	{
		dummy_v[1] = "--nocrashhandler"	;
		dummy_c	   = 2			;
	}

	KAboutData *about =
		new KAboutData
		(	__KB_APPNAME,
			"RekallCPPUnitTest",
			"0.0.0",
			"Rekall: CPPUnit Test",
			KAboutData::License_Unknown,
			"(c) 2000,2001,2002,2003,2004,2005 mike@quaking.demon.co.uk",
			0,
			"http://www.rekall.a-i-s.co.uk",
			"mike@quaking.demon.co.uk"
		) ;
	about->addAuthor  ("Mike Richardson", 0, "mike@quaking.demon.co.uk") ; 

	KCmdLineArgs::init (dummy_c, dummy_v, about) ;

	return	new RKApplication () ;
#endif
#if	__KB_TKC
	return	new RKApplication (argc, (char **)argv) ;
#endif
}

EXPORT_API
	KBDBInfo
		*rekallDBInfo 
		(	const char	*dbFile,
			const char	*dbDir,
			const char	*dbName
		)
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile (dbFile, dbDir, getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"rekallServerInfo: %s: cannot open: %s\n",
			dbFile,
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     (dbName) ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);
	return	dbInfo	;
}

/*  rekallTest	: Run rekall tests under CPPUNIT			*/
/*  cmd		: const char *	: Command name used for error messages	*/
/*  argc	: int		: Argument count			*/
/*  argv	: const char **	: Argument vector			*/
/*  setup	:		: Test setup callback			*/
/*  (returns)	: int		: Test run return code			*/

EXPORT_API int	rekallTest
	(	const char 	*cmd,
		int		a_argc,
		const char	**a_argv,
		CppUnit::Test	*(*setup) (const QMap<QString,QString> *)
	)
{
	QMap<QString,QString>	settings	;
	bool			gui	= false	;
	bool			rapp	= false	;
	bool			showp	= true	;
	bool			names	= false	;
	const char		*xml	= 0	;
	const char		*debug	= 0	;

	static	const char	*usage	=
		"%s: "
		"[--silent] "
		"[--verbose] "
		"[--names] "
		"[--rapp] "
		"[--gui] "
		"[--nocrashhandler] "
		"[--xml=file] "
		"[--pause=opt1,opt2,...] "
		"[--dbhost=host] "
		"[--rootdir=path] "
		"[--tmp=path] "
		"[--xbase=path] "
		"[--skip=opt1,opt2,...] "
		"[--only=opt1,opt2,...] "
		"[--debug=name] "
		"\n"	;

	__cmd	 = cmd	;

	int		argc	= a_argc - 1 ;
	const char	**argv	= a_argv + 1 ;

	while ((argc > 0) && (argv[0][0] == '-'))
	{
		if (qstrcmp (argv[0], "--help-kde") == 0)
		{
			argc	-= 1	;
			argv	+= 1	;
			continue	;
		}

		if (qstrcmp (argv[0], "--help-qt") == 0)
		{
			argc	-= 1	;
			argv	+= 1	;
			continue	;
		}

		if (qstrcmp (argv[0], "--nocrashhandler") == 0)
		{
			nocrash	 = true	;
			argc	-= 1	;
			argv	+= 1	;
			continue	;
		}

		if (qstrcmp (argv[0], "--silent") == 0)
		{
			showp	 = false;
			argc	-= 1	;
			argv	+= 1	;
			continue    	;
		}

		if (qstrcmp (argv[0], "--names" ) == 0)
		{
			names	 = true	;
			argc	-= 1	;
			argv	+= 1	;
			continue    	;
		}

		if (qstrcmp (argv[0], "--verbose") == 0)
		{
			__verbose = true;
			argc	-= 1	;
			argv	+= 1	;
			continue    	;
		}

		if (qstrcmp (argv[0], "--rapp") == 0)
		{
			rapp	 = true	;
			argc	-= 1	;
			argv	+= 1	;
			continue    	;
		}

		if (qstrcmp (argv[0], "--gui" ) == 0)
		{
			gui	 = true	;
			argc	-= 1	;
			argv	+= 1	;
			continue    	;
		}

		if (qstrncmp (argv[0], "--xml=",     6) == 0)
		{
			xml	= &argv[0][6] ;
			argc	-= 1	;
			argv	+= 1	;
			continue    	;
		}

		if (qstrncmp (argv[0], "--pause=",   8) == 0)
		{
			if (__pause == 0)
				__pause	 = new QMap<QString,QString>	;
			QStringList pause = QStringList::split(',', &argv[0][8]) ;
			for (uint idx = 0 ; idx < pause.count() ; idx += 1)
				(*__pause)[pause[idx]] = "1" ;
			argc	-= 1 ;
			argv	+= 1 ;
			continue     ;
		}

		if (qstrncmp (argv[0], "--dbhost=",  9) == 0)
		{
			settings["dbhost"] = &argv[0][9] ;
			argc	-= 1 ;
			argv	+= 1 ;
			continue     ;
		}

		if (qstrncmp (argv[0], "--rootdir=", 10) == 0)
		{
			__rootd	 = &argv[0][10] ;
			argc	-= 1 ;
			argv	+= 1 ;
			continue     ;
		}

		if (qstrncmp (argv[0], "--tmp=",    6) == 0)
		{
			settings["tmp"   ] = &argv[0][6] ;
			argc	-= 1 ;
			argv	+= 1 ;
			continue     ;
		}

		if (qstrncmp (argv[0], "--xbase=",  8) == 0)
		{
			settings["xbase" ] = &argv[0][8] ;
			argc	-= 1 ;
			argv	+= 1 ;
			continue     ;
		}

		if (qstrncmp (argv[0], "--skip=",    7) == 0)
		{
			if (__skip == 0)
				__skip	 = new QMap<QString,QString>	;

			QStringList skip = QStringList::split(',', &argv[0][7]) ;
			for (uint idx = 0 ; idx < skip.count() ; idx += 1)
			{
				(*__skip)[skip[idx]] = "1" ;
			}
			argc	-= 1 ;
			argv	+= 1 ;
			continue     ;
		}

		if (qstrncmp (argv[0], "--only=",    7) == 0)
		{
			if (__only == 0)
				__only	 = new QMap<QString,QString>	;

			QStringList only = QStringList::split(',', &argv[0][7]) ;
			for (uint idx = 0 ; idx < only.count() ; idx += 1)
			{
				(*__only)[only[idx]] = "1" ;
			}
			argc	-= 1 ;
			argv	+= 1 ;
			continue     ;
		}

		if (qstrncmp (argv[0], "--debug=",   8) == 0)
		{
			debug	= &argv[0][8] ;
			argc	-= 1	;
			argv	+= 1	;
			continue    	;
		}

		break	;
	}

	if (argc != 0)
	{
		fprintf	(stderr, usage, __cmd) ;
		exit	(1) ;
	}

	settings["gui"] = gui ? "1" : "0" ;


	if (debug != 0)
	{
		FILE *df = fopen (debug, "w") ;
		if (df == 0)
		{
			fprintf	(stderr, "tester: %s: cannot open\n", debug) ;
			exit	(1) ;
		}
		kbDPrintfSetStream (df) ;
	}

	/* Start application if required. If no application and this is	*/
	/* KDE then we have to set the root directory and create an	*/
	/* instance; on the MAC we must always set the root directory.	*/
	if (rapp)
		rekallTestSetupApp (a_argc,  a_argv) ;		

	extern	void    setRootDir (const QString &) ;
	if (__rootd == 0)
		setRootDir    (TESTROOTDIR) ;
	else	setRootDir    (__rootd	  ) ;

#if	__KB_KDE
	if (!rapp)
		new KInstance (__KB_APPNAME) ;
#endif

	KBAppPtr::setCallback (new KBCallback) ;

	/* If we want a GUI and do not have the rekall application	*/
	/* then start a base QT application.				*/
	if (gui && !rapp)
		new QApplication (a_argc, (char **)a_argv) ;

	if (gui)
	{
#if	__KB_CPPUNITQT
		CppUnit::QtTestRunner runner ;
		runner.addTest (setup (&settings)) ;
		runner.run() ;
#else
		fprintf	(stderr, "%s: no QT runner\n", __cmd) ;
#endif
		return	0 ;		
	}


	std::ofstream		*outputFile = 0 ;
	CppUnit::TextTestRunner runner 	   ;
	runner.addTest (setup (&settings)) ;

	if (names)
		runner.eventManager().addListener (new CppUnit::BriefTestProgressListener) ;

	if (xml)
	{
		outputFile = new std::ofstream (xml) ;

		CppUnit::XmlOutputter	*outputter = new CppUnit::XmlOutputter(&runner.result(), *outputFile) ;
		XmlOutputterHook	*hook	   = new XmlOutputterHook ("rekall", "mikeTA") ;

		outputter->addHook	(hook)		;
		runner.setOutputter	(outputter)	;

	}

	bool	rc = runner.run("", false, true, showp) ;

	if (outputFile != 0)
		outputFile->close() ;

	return	rc ? 0 : 1;
}

