/***************************************************************************
    file	         : value.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qstring.h>
#include	<qmap.h>

#include	"kb_type.h"
#include	"kb_value.h"

#include	"tester.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBValueTest : public CppUnit::TestCase
{
public	:

	KBValueTest	() ;
	KBValueTest	(std::string name) ;


	void			testSimple	() ;
	void			testFormat	() ;
	void			testDate	() ;
	void			testDateTime	() ;
	void			testDeformat	() ;

	static CppUnit::Test	*suite		() ;
}	;

KBValueTest::KBValueTest ()
{
}

KBValueTest::KBValueTest
	(	std::string	name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBValueTest::testSimple ()
{
	static	struct
		{	unsigned int	m_index		;
			const char	*m_text		;
			KBType		*m_type		;
			const char	*m_result	;
		}
		tests[]	=
	{
	{	1,	"Mike",				&_kbString,			"Mike"			},
	{	2,	"46",				&_kbFixed,			"46"			},
	{	3,	"46.7",				&_kbFloat,			"46.7"			}
	}	;

	for (unsigned int idx = 0 ; idx < sizeof(tests)/sizeof(tests[0]) ; idx += 1)
	{
		KBValue	v (tests[idx].m_text, tests[idx].m_type) ;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString	("%1: %2 with %3")
				.arg(tests[idx].m_index )
				.arg(tests[idx].m_text  )
				.arg(tests[idx].m_type->getDescrip())),
			_STDSTR(QString(tests[idx].m_result)),
			_STDSTR(v.getText(QString::null))
		)	;
	}
}


void	KBValueTest::testDateTime ()
{
	static	struct
		{	unsigned int	m_index		;
			int		m_order		;
			const char	*m_text		;
			const char	*m_format	;
			const char	*m_result	;
		}
		dateTimeTests[]	=
	{
	{	1,	0,	"23-Feb-03",				"DateTime:",			"23-Feb-03"		},
	{	2,	0,	"23-Feb-03",				"DateTime:%d-%b-%y",		"23-Feb-03"		},
	{	3,	0,	"23-Feb-03",				"DateTime:%d-%b-%Y",		"23-Feb-2003"		},
	{	4,	0,	"23-Feb-03 -1200",			"DateTime:%d-%b-%Y",		"23-Feb-2003"		},
	{	5,	0,	"23-02-2003",				"DateTime:%d-%b-%Y",		"23-Feb-2003"		},
	{	6,	0,	"23/02/2003",				"DateTime:%d-%b-%Y",		"23-Feb-2003"		},
	{	7,	0,	"23.02.2003",				"DateTime:%d-%b-%Y",		"23-Feb-2003"		},
	{	8,	0,	"2003-02-03",				"DateTime:%d-%b-%Y",		"03-Feb-2003"		},
	{	9,	0,	"2003/02/03",				"DateTime:%d-%b-%Y",		"03-Feb-2003"		},
	{	10,	0,	"2003.02.03",				"DateTime:%d-%b-%Y",		"03-Feb-2003"		},
	{	11,	0,	"23 Feb 2004",				"DateTime:%d-%b-%Y",		"23-Feb-2004"		},
	{	12,	0,	"23 Feb 2004",				"DateTime:%d-%b-%Y",		"23-Feb-2004"		},
	{	13,	0,	"23 Feb 2004, 10:46am",			"DateTime:%d-%b-%Y %H:%M",	"23-Feb-2004 10:46"	},
	{	14,	0,	"23 Feb 2004, 10:46am",			"DateTime:%d-%b-%Y %H:%M",	"23-Feb-2004 10:46"	},
	{	15,	0,	"Thurs, 26 Feb 2004 05:16:56",		"DateTime:%d-%b-%Y %H:%M:%S",	"26-Feb-2004 05:16:56"	},
	{	16,	0,	"Thurs, 26 Feb 2004 05:16:56 +0200",	"DateTime:%d-%b-%Y %H:%M:%S",	"26-Feb-2004 05:16:56"	},
	{	17,	0,	"Fri, 22 Jun 2001 11:22:44 +0000 (ut)",	"DateTime:%d-%b-%Y %H:%M:%S",	"22-Jun-2001 11:22:44"	},
	{	18,	0,	"Fri, 22 Jun 2001 11:22:44 +0000 (UT)",	"DateTime:%d-%b-%Y %H:%M:%S",	"22-Jun-2001 11:22:44"	},
	{	19,	0,	"20040223",				"DateTime:%d-%b-%Y %H:%M:%S",	"23-Feb-2004 00:00:00"	},
	{	20,	0,	"20040223104633",			"DateTime:%d-%b-%Y %H:%M:%S",	"23-Feb-2004 10:46:33"	},
	{	21,	0,	"01/02/2003",				"DateTime:%d-%b-%Y",		"01-Feb-2003"		},
	{	22,	0,	"02/01/2003",				"DateTime:%d-%b-%Y",		"02-Jan-2003"		},
	{	23,	1,	"01/02/2003",				"DateTime:%d-%b-%Y",		"02-Jan-2003"		},
	{	24,	1,	"02/01/2003",				"DateTime:%d-%b-%Y",		"01-Feb-2003"		},
	{	25,	0,	"28 Jul 2005, 8:00am",			"DateTime:%d-%b-%Y %H:%M",	"28-Jul-2005 08:00"	},
	{	26,	0,	"Thu, 28 Jul 2005, 4:34am",		"DateTime:%d-%b-%Y %H:%M",	"28-Jul-2005 04:34"	},
	{	27,	0,	"Thu, 28 Jul 2005 06:27:11 -07:00(PDT)","DateTime:%d-%b-%Y %H:%M",	"28-Jul-2005 06:27"	},
	{	28,	0,	"Wed Mar 1 2006",			"DateTime:%d/%m/%y",		"01/03/06"		},
	{	29,	0,	"Wed Mar 1 2006",			"!Date:%d/%m/%y",		"01/03/06"		},
	{	30,	0,	"Wed Mar 1 2006",			"Date:%d/%m/%y",		"Date?Wed Mar 1 2006"	}
	}	;

	for (unsigned int idx = 0 ; idx < sizeof(dateTimeTests)/sizeof(dateTimeTests[0]) ; idx += 1)
	{
		KBDateTime::setDateOrder ((KBDateTime::DateOrder)dateTimeTests[idx].m_order) ;
		KBValue	v (dateTimeTests[idx].m_text, &_kbDateTime) ;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString	("%1: %2 with %3")
				.arg(dateTimeTests[idx].m_index )
				.arg(dateTimeTests[idx].m_text  )
				.arg(dateTimeTests[idx].m_format)),
			_STDSTR(QString(dateTimeTests[idx].m_result)),
			_STDSTR(v.getText(dateTimeTests[idx].m_format))
		)	;
	}
}

void	KBValueTest::testDate ()
{
	static	struct
		{	unsigned int	m_index		;
			int		m_order		;
			const char	*m_text		;
			const char	*m_format	;
			const char	*m_result	;
		}
		dateTests[]	=
	{
	{	1,	0,	"23-Feb-03",				"Date:",			"23-Feb-03"		},
	{	2,	0,	"23-Feb-03",				"Date:%d-%b-%y",		"23-Feb-03"		},
	{	3,	0,	"23-Feb-03",				"Date:%d-%b-%Y",		"23-Feb-2003"		},
	{	4,	0,	"23-Feb-03 -1200",			"Date:%d-%b-%Y",		"23-Feb-2003"		},
	{	5,	0,	"23-02-2003",				"Date:%d-%b-%Y",		"23-Feb-2003"		},
	{	6,	0,	"23/02/2003",				"Date:%d-%b-%Y",		"23-Feb-2003"		},
	{	7,	0,	"23.02.2003",				"Date:%d-%b-%Y",		"23-Feb-2003"		},
	{	8,	0,	"2003-02-03",				"Date:%d-%b-%Y",		"03-Feb-2003"		},
	{	9,	0,	"2003/02/03",				"Date:%d-%b-%Y",		"03-Feb-2003"		},
	{	10,	0,	"2003.02.03",				"Date:%d-%b-%Y",		"03-Feb-2003"		},
	{	11,	0,	"23 Feb 2004",				"Date:%d-%b-%Y",		"23-Feb-2004"		},
	{	12,	0,	"23 Feb 2004",				"Date:%d-%b-%Y",		"23-Feb-2004"		},
	{	13,	0,	"23 Feb 2004, 10:46am",			"Date:%d-%b-%Y",		"23-Feb-2004"		},
	{	14,	0,	"23 Feb 2004, 10:46am",			"Date:%d-%b-%Y",		"23-Feb-2004"		},
	{	15,	0,	"Thurs, 26 Feb 2004 05:16:56",		"Date:%d-%b-%Y",		"26-Feb-2004"		},
	{	16,	0,	"Thurs, 26 Feb 2004 05:16:56 +0200",	"Date:%d-%b-%Y",		"26-Feb-2004"		},
	{	17,	0,	"Fri, 22 Jun 2001 11:22:44 +0000 (ut)",	"Date:%d-%b-%Y",		"22-Jun-2001"		},
	{	18,	0,	"Fri, 22 Jun 2001 11:22:44 +0000 (UT)",	"Date:%d-%b-%Y",		"22-Jun-2001"		},
	{	19,	0,	"20040223",				"Date:%d-%b-%Y",		"23-Feb-2004"		},
	{	20,	0,	"20040223104633",			"Date:%d-%b-%Y",		"23-Feb-2004"		},
	{	21,	0,	"01/02/2003",				"Date:%d-%b-%Y",		"01-Feb-2003"		},
	{	22,	0,	"02/01/2003",				"Date:%d-%b-%Y",		"02-Jan-2003"		},
	{	23,	1,	"01/02/2003",				"Date:%d-%b-%Y",		"02-Jan-2003"		},
	{	24,	1,	"02/01/2003",				"Date:%d-%b-%Y",		"01-Feb-2003"		},
	{	25,	0,	"28 Jul 2005, 8:00am",			"Date:%d-%b-%Y",		"28-Jul-2005"		},
	{	26,	0,	"Thu, 28 Jul 2005, 4:34am",		"Date:%d-%b-%Y",		"28-Jul-2005"		},
	{	27,	0,	"Thu, 28 Jul 2005 06:27:11 -07:00(PDT)","Date:%d-%b-%Y",		"28-Jul-2005"		},
	{	28,	0,	"Wed Mar 1 2006",			"Date:%d/%m/%y",		"01/03/06"		},
	{	29,	0,	"Wed Mar 1 2006",			"!DateTime:%d/%m/%y",		"01/03/06"		},
	{	30,	0,	"Wed Mar 1 2006",			"DateTime:%d/%m/%y",		"DateTime?Wed Mar 1 2006"
																}
	}	;

	for (unsigned int idx = 0 ; idx < sizeof(dateTests)/sizeof(dateTests[0]) ; idx += 1)
	{
		KBDateTime::setDateOrder ((KBDateTime::DateOrder)dateTests[idx].m_order) ;
		KBValue	v (dateTests[idx].m_text, &_kbDate) ;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString	("%1: %2 with %3")
				.arg(dateTests[idx].m_index )
				.arg(dateTests[idx].m_text  )
				.arg(dateTests[idx].m_format)),
			_STDSTR(QString(dateTests[idx].m_result)),
			_STDSTR(v.getText(dateTests[idx].m_format))
		)	;
	}
}

void	KBValueTest::testFormat ()
{
	static	struct
		{	unsigned int	m_index		;
			int		m_order		;
			const char	*m_text		;
			const char	*m_format	;
			const char	*m_result	;
		}
		tests[]	=
	{
	{	1,	0,	"Mike Richardson",			"",				"Mike Richardson"	},
	{	2,	0,	"Mike Richardson",			"String:",			"Mike Richardson"	},
	{	3,	0,	"Mike Richardson    ",			"String:",			"Mike Richardson    "	},
	{	4,	0,	"Mike Richardson    ",			"String:##Trim",		"Mike Richardson"	},
	{	5,	0,	"    Mike Richardson    ",		"String:##Trim",		"    Mike Richardson"	}
	}	;

	for (unsigned int idx = 0 ; idx < sizeof(tests)/sizeof(tests[0]) ; idx += 1)
	{
		KBValue	v (tests[idx].m_text) ;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString	("%1: %2 with %3")
				.arg(tests[idx].m_index )
				.arg(tests[idx].m_text  )
				.arg(tests[idx].m_format)),
			_STDSTR(QString(tests[idx].m_result)),
			_STDSTR(v.getText(tests[idx].m_format))
		)	;
	}
}

void	KBValueTest::testDeformat ()
{
	static	struct
		{	unsigned int	m_index		;
			const char	*m_text		;
			const char	*m_format	;
			KBType		*m_type		;
			const char	*m_result	;
		}
		tests[]	=
	{
	{	0,	"123456",		"Number",		&_kbFixed,			"123456"		},
	{	1,	"123,456",		"Number",		&_kbFixed,			"123456"		},
	{	2,	"123 456",		"Number",		&_kbFixed,			"123"			},
	{	3,	"123/456",		"Number",		&_kbFixed,			"123"			},
	{	1,	"Mike is 46",		"Number",		&_kbFixed,			"46"			},
	{	5,	"Mike is 46.05",	"Number",		&_kbFloat,			"46.05"			},
	{	6,	"Mike. is 46.05",	"Number",		&_kbFloat,			"46.05"			},
	{	7,	"123,456.99",		"Number",		&_kbFloat,			"123456.99"		},
	{	8,	"2005-02-01",		"%Y-%m-%d",		&_kbDate,			"2005-02-01"		},
	{	9,	"2005-01-02",		"%Y-%d-%m",		&_kbDate,			"2005-02-01"		},
	{	10,	"02/01/05",		"%d-%m-%y",		&_kbDate,			"2005-01-02"		},
	{	11,	"01/02/05",		"%m-%d-%y",		&_kbDate,			"2005-01-02"		},
	{	12,	"01-Feb-2009",		"%Y-%m-%d",		&_kbDate,			"2009-02-01"		},
	{	13,	"01-Feb-03",		"%Y-%m-%d",		&_kbDate,			"2003-02-01"		},
	{	14,	"12:13:14 2005-02-01",	" %H:%M:%S %Y-%m-%d",	&_kbDateTime,			"2005-02-01 12:13:14"	},
	{	15,	"2005-02-01 12:13:14",	"%Y-%m-%d %H:%M:%S",	&_kbDateTime,			"2005-02-01 12:13:14"	},
	{	16,	"2005-01-02 01:02:03",	"%Y-%d-%m %H:%M:%S",	&_kbDateTime,			"2005-02-01 01:02:03"	},
	{	17,	"01/02/05 01:02:03",	"%d-%m-%y %H:%M:%S",	&_kbDateTime,			"2005-02-01 01:02:03"	},
	{	18,	"02/01/05 01:02:03",	"%m-%d-%y %H:%M:%S",	&_kbDateTime,			"2005-02-01 01:02:03"	},
	{	19,	"+123,456.99",		"Number",		&_kbFloat,			"+123456.99"		},
	{	20,	"-123,456.99",		"Number",		&_kbFloat,			"-123456.99"		},
	{	21,	"123456.99e10",		"Number",		&_kbFloat,			"123456.99e10"		},
	{	22,	"123456.99E10",		"Number",		&_kbFloat,			"123456.99E10"		},
	{	23,	"123456.99e+10",	"Number",		&_kbFloat,			"123456.99e+10"		},
	{	24,	"123456.99e-10",	"Number",		&_kbFloat,			"123456.99e-10"		},
	}	;

	for (unsigned int idx = 0 ; idx < sizeof(tests)/sizeof(tests[0]) ; idx += 1)
	{
		KBValue	v (tests[idx].m_text, tests[idx].m_type, tests[idx].m_format) ;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString	("%1: %2 with %3/%4")
				.arg(tests[idx].m_index )
				.arg(tests[idx].m_text  )
				.arg(tests[idx].m_type->getDescrip())
				.arg(tests[idx].m_format)),
			_STDSTR(QString(tests[idx].m_result)),
			_STDSTR(v.getRawText())
		)	;
	}
}

CppUnit::Test
	*KBValueTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Value") ;

	if (rekallTestSkip("Value")) return s ;

	s->addTest (new CppUnit::TestCaller<KBValueTest> ("Simple",   &KBValueTest::testSimple  )) ;
	s->addTest (new CppUnit::TestCaller<KBValueTest> ("Format",   &KBValueTest::testFormat  )) ;
	s->addTest (new CppUnit::TestCaller<KBValueTest> ("DateTime", &KBValueTest::testDateTime)) ;
	s->addTest (new CppUnit::TestCaller<KBValueTest> ("Date",     &KBValueTest::testDate    )) ;
	s->addTest (new CppUnit::TestCaller<KBValueTest> ("Deformat", &KBValueTest::testDeformat)) ;
	return s ;
}

CppUnit::Test EXPORT_API *valueTestSuite
	(	const QMap<QString,QString> *
	)
{
	return	KBValueTest::suite () ;
}
