/***************************************************************************
    file	         : copier.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>

#include	<qstring.h>
#include	<qmap.h>



#if		__KB_KDE
#include	<kapp.h>
#include	<klocale.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>
#endif

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_dbinfo.h"
#include	"kb_serverinfo.h"
#include	"kb_location.h"
#include	"kb_copybase.h"
#include	"kb_copyfile.h"
#include	"kb_copytable.h"
#include	"kb_copyxml.h"
#include	"kb_copysql.h"
#include	"kb_dom.h"
#include	"kb_locator.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

static	void	addXMLFields
	(	KBCopyXML		*copy,
		const QDomElement	&elem
	)
{
	DOMITER_BEGIN(elem,"field",child)
	{
		copy->addField (child.attribute("name"), child.attribute("asattr").toInt()) ;
	}
	DOMITER_END
}

class	KBCopierTest : public CppUnit::TestCase
{
	int			execCopy	(KBCopyBase *, KBCopyBase *, const QString &) ;
	QString			saveToFile	(const QString     &)	;
	void			evaluate	(const QDomElement &)	;
	QString			getResultText	(KBCopyBase *, const QString &, QString &) ;
	QString			trimWhiteSpace	(QString) ;
	QString			nextFileName	() ;
	QDomElement		m_elem		;

public	:

	KBCopierTest	() ;
	KBCopierTest	(std::string name, const QDomElement &) ;

	void			run		() ;

	static CppUnit::Test	*suite		(const char *) ;
}	;

static	const QMap<QString,QString> *settings	;



KBCopierTest::KBCopierTest ()
{
}

KBCopierTest::KBCopierTest
	(	std::string		name,
		const QDomElement	&elem
	)
	:
	CppUnit::TestCase (name),
	m_elem		  (elem)
{
}

void	KBCopierTest::run ()
{
	CPPUNIT_ASSERT	(!(*settings)["tmp"   ].isNull()) ;
	CPPUNIT_ASSERT	(!(*settings)["dbhost"].isNull()) ;
	evaluate	(m_elem) ;
}

QString	KBCopierTest::trimWhiteSpace
	(	QString		text
	)
{
	int	idx ;
	for (idx = 0 ; idx < (int)text.length() ; idx += 1)
		if (!text[idx].isSpace())
					break	;
	text	= text.mid(idx) ;
	for (idx = text.length() - 1 ; idx >= 0 ; idx -= 1)
		if (text[idx] != ' ')
			break	;
	text	= text.mid(0, idx + 1) ;

//	fprintf
//	(	stderr,
//		"[[[\n%s]]]\n",
//		text.latin1()
//	)	;

	return	text	;
}

QString	KBCopierTest::nextFileName()
{
	static	 int	seq	= 0	;
	QString	 name	;
	name.sprintf	("%s/copier.%06d", (*settings)["tmp"].latin1(), seq++) ;
	return	 name	;
}

QString	KBCopierTest::saveToFile
	(	const QString	&rtext
	)
{
	QString	 name	= nextFileName();
	QFile	 file	(name)		;
	file.remove	()		;
	file.open	(IO_WriteOnly)	;

	QString	 stext	= trimWhiteSpace(rtext) ;
	QCString ctext	= stext.utf8()	;
	file.writeBlock	(ctext.data(), ctext.length()) ;

	return	name	;
}

QString	KBCopierTest::getResultText
	(	KBCopyBase	*dest,
		const QString	&id,
		QString		&whence
	)
{
	if (qstrcmp (dest->tag(), "file") == 0)
	{
		KBCopyFile *copy = (KBCopyFile *)dest ;
		QFile	   file	   (copy->getFile ()) ;
		CPPUNIT_ASSERT_MESSAGE
		(
			_STDSTR	(QString("File open failed: %1: %2").arg(id).arg(copy->getFile())),
			file.open (IO_ReadOnly)
		)	;

		whence	= QString("in file %1").arg(copy->getFile()) ;

		QByteArray data = file.readAll() ;
		return	QString::fromUtf8 (data.data(), data.size()) ;
	}

	if (qstrcmp (dest->tag(), "xml") == 0)
	{
		KBCopyXML  *copy = (KBCopyXML *)dest ;
		QFile	   file	   (copy->getFile ()) ;
		CPPUNIT_ASSERT_MESSAGE
		(
			_STDSTR	(QString("File open failed: %1: %2").arg(id).arg(copy->getFile())),
			file.open (IO_ReadOnly)
		)	;

		whence	= QString("in file %1").arg(copy->getFile()) ;

		QByteArray data = file.readAll() ;
		return	QString::fromUtf8 (data.data(), data.size()) ;
	}

	if (qstrcmp (dest->tag(), "table") == 0)
	{
		KBCopyTable *copy = (KBCopyTable *)dest ;
		return	QString::null	;
	}

	CPPUNIT_FAIL
	(
		_STDSTR	(QString("Get results failed: %1: %2").arg(id).arg(dest->tag()))
	)	;
	return	QString::null	;
}

int	KBCopierTest::execCopy
	(	KBCopyBase	*srce,
		KBCopyBase	*dest,
		const QString	&id
	)
{
	KBError			error	;
	const QDict<QString>	pDict	;

	bool	sp		= srce->prepare (pDict, dest) ;
	CPPUNIT_ASSERT_MESSAGE
	(
		_STDSTR	(QString("Source setup failed: %1: %2").arg(id).arg(srce->lastError().getMessage())),
		sp
	)	;

	bool	dp		= dest->prepare (pDict, srce) ;
	CPPUNIT_ASSERT_MESSAGE
	(
		_STDSTR	(QString("Destination setup failed: %1: %2").arg(id).arg(dest->lastError().getMessage())),
		dp
	)	;

	int	srceCols = srce->getNumCols() ;
	int	destCols = dest->getNumCols() ;

	int	numCols	 = srceCols > 0 ? srceCols : destCols > 0 ? destCols : 500 ;

	bool	di	 = dest->putRow(0, 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(
		_STDSTR	(QString("Initial putrow failed: %1: %2").arg(id).arg(dest->lastError().getMessage())),
		di
	)	;

	KBValue	*values = new KBValue [numCols + 1] ;
	int	nRows   = srce->execute (dest, values, numCols, 0) ;
	delete [] values ;

	if (nRows < 0)
		CPPUNIT_FAIL
		(
			_STDSTR	(QString("Execute failed: %1: %2").arg(id).arg(srce->lastError().getMessage()))
		)	;


	QString	rep	;
	bool	df	= dest->finish	(rep) ;
	CPPUNIT_ASSERT_MESSAGE
	(
		_STDSTR	(QString("Finish failed: %1: %2").arg(id).arg(dest->lastError().getMessage())),
		df
	)	;

	return	nRows	;
}

void	KBCopierTest::evaluate
	(	const QDomElement	&elem
	)
{
	KBCopyBase	*srce	= 0	;
	KBCopyBase	*dest	= 0	;
	KBServerInfo    *svInfo	= 0	;
	KBServer        *server = 0	;
	KBError         error   	;
	QString		result		;
	QString		id		= elem.attribute("id") ;

	KBDBInfo	dbInfo	("Copier") ;

	DOMITER_ELEM_BEGIN(elem,child)
	{
		KBCopyBase *base = 0	;

		if	(child.nodeName() == "serverinfo")
		{
			if (!(*settings)["dbhost"].isEmpty())
				child.setAttribute ("HostName", (*settings)["dbhost"]) ;

			svInfo	= new KBServerInfo (0, child) ;
			server	= svInfo->getServer (error)   ;
			CPPUNIT_ASSERT_MESSAGE
			(
				_STDSTR	(QString("GetServer failed: %1").arg(id)),
				server != 0
			)	;
			dbInfo.insert (svInfo)	;
		}
		else if	(child.nodeName() == "file")
		{
			KBLocation	locn	;
			KBCopyFile	*copy = new KBCopyFile (srce == 0, locn) ;

			copy->setWhich	(child.attribute("which" ).toInt()) ;
			copy->setDelim	(child.attribute("delim" )) ;
			copy->setQualif	(child.attribute("qualif")) ;
			copy->setErrOpt	(child.attribute("erropt").toInt()) ;
			copy->setHeader	(child.attribute("header").toInt(), 0) ;
			copy->setErrOpt (child.attribute("erropt", "3").toInt()) ;

			if (srce == 0)
			{
				DOMITER_BEGIN(child,"data",data)
				{
					QString	file	= saveToFile    (trimWhiteSpace(data.text())) ;
					copy->setFile	(file) ;
				}
				DOMITER_END
			}
			else	copy->setFile (nextFileName()) ;

			DOMITER_BEGIN(child,"field",field)
			{
				copy->addField
				(	field.attribute("name"  ),
					field.attribute("offset").toInt(),
					field.attribute("width" ).toInt(),
					field.attribute("strip" ).toInt()
				)	;
			}
			DOMITER_END

			base	= copy	;
		}
		else if	(child.nodeName() == "xml")
		{
			KBLocation	locn	;
			KBCopyXML	*copy = new KBCopyXML (srce == 0, locn) ;

			QString	file	= saveToFile   (child.text()) ;

			copy->setFile	(file	    ) ;
			copy->setMainTag(child.attribute("maintag")) ;
			copy->setRowTag (child.attribute("rowtag" )) ;
			addXMLFields    (copy, child) ;
			copy->setErrOpt	(0)	;
			base	= copy	;
		}
		else if	(child.nodeName() == "table")
		{
			KBLocation	locn
					(	&dbInfo,
						"table",
						svInfo->serverName(),
						child.attribute("name")
					)	;
			KBCopyTable	*copy	= new KBCopyTable (srce == 0, locn) ;
			copy->setServer (svInfo->serverName  ()) ;
			copy->setTable	(child.attribute("name")) ;
			copy->setOption
			(	child.attribute("option"  ).toInt(),
				child.attribute("optfield")
			)	;

			DOMITER_BEGIN(child,"field",field)
			{
				copy->addField(field.attribute("name")) ;
			}
			DOMITER_END

			base	= copy	;
		}
		else if	(child.nodeName() == "sql")
		{
			KBLocation	locn
					(	&dbInfo,
						"table",
						svInfo->serverName(),
						child.attribute("name")
					)	;
			KBCopySQL	*copy	= new KBCopySQL (srce == 0, locn) ;
			copy->setServer (svInfo->serverName ()) ;
			copy->setSQL	(child.attribute("sql")) ;
			base	= copy	;

		}
		else if (child.nodeName() == "result")
		{
			result	= trimWhiteSpace (child.text()) ;
			continue;
		}

		if	(srce == 0) srce = base ;
		else if (dest == 0) dest = base ;
		else		    delete base	;
	}
	DOMITER_END

	if ((srce == 0) || (dest == 0))
		CPPUNIT_FAIL
		(
			_STDSTR	(QString("Underspecified copier failed: %1").arg(id))
		)	;


	int	nRows	= execCopy (srce, dest, id) ;
	CPPUNIT_ASSERT_MESSAGE
	(
		_STDSTR	(QString("Rowcount mismatch: %1: got %2, expected %3")
				.arg(id   )
				.arg(nRows)
				.arg(elem.attribute("numrows").toInt())
			),
		nRows == elem.attribute("numrows").toInt()
	)	;

	QString	whence	;
	QString	got	= getResultText (dest, id, whence) ;
	if (!result.isNull())
	{
		CPPUNIT_ASSERT_MESSAGE
		(
			_STDSTR	(QString("Results mismatch: %1: %2").arg(id).arg(whence)),
			got   == result
		)	;
	}

	delete	srce	;
	delete	dest	;
	if (svInfo != 0) delete svInfo	;
}


static	void	addCopierTests
	(	CppUnit::TestSuite	*suite,
		const QString		&spec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile ("copier_" + spec + ".xml", "test/copier", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"copier: %s: cannot open: copier_%s.xml\n",
			spec.latin1(),
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	DOMITER_BEGIN(doc.documentElement(),"test",elem)
	{
		suite->addTest
		(	new CppUnit::TestCaller<KBCopierTest>
			(	_STDSTR(elem.attribute("id")),
				&KBCopierTest::run,
				new KBCopierTest (_STDSTR(elem.attribute("id")), elem)
		)	)	;
	}
	DOMITER_END
}

CppUnit::Test
	*KBCopierTest::suite
	(	
		const char	*spec
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite(spec) ;
	addCopierTests (s, spec) ;
	return s ;
}

CppUnit::Test EXPORT_API *testSetup
	(	const QMap<QString,QString> *__settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Copier") ;

	settings = __settings ;
	s->addTest (KBCopierTest::suite ("file_file" )) ;
	s->addTest (KBCopierTest::suite ("file_table")) ;
	s->addTest (KBCopierTest::suite ("file_xml"  )) ;
	s->addTest (KBCopierTest::suite ("sql_file"  )) ;

	return	s ;
}

#include	"kb_libloader.h"
#include	"factory.h"

TESTFACTORY	(KBTestCopierFactory,Copier,rkt_copier)
