/***************************************************************************
    file	         : copypaste.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qstring.h>
#include	<qmap.h>
#include	<qlabel.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_label.h"
#include	"kb_dom.h"
#include	"kb_ascii.h"
#include	"kb_formcopier.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBCopyPasteTest : public CppUnit::TestCase
{
	TestOpenForm		*m_form		;

	const char		*m_label	;
	const char		*m_legend	;
	QRect			m_from		;
	QRect			m_at		;

	KBLabel			*getLabel	(const char *, const char *) ;
	KBLabel			*getLabel	(KBNode    *) ;
	KBControl		*getControl	(KBLabel   *) ;
	QLabel			*getWidget	(KBControl *) ;

public	:

	KBCopyPasteTest	() ;
	KBCopyPasteTest	(std::string name, const char *, const char *, QRect, QRect) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testCopyPaste	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBCopyPasteTest::KBCopyPasteTest ()
{
}

KBCopyPasteTest::KBCopyPasteTest
	(	std::string		name,
		const char		*label,
		const char		*legend,
		QRect			from,
		QRect			at
	)
	:
	CppUnit::TestCase (name),
	m_label		  (label),
	m_legend	  (legend),
	m_from		  (from),
	m_at		  (at)
{
}

void	KBCopyPasteTest::setUp ()
{
	KBError	error	;
	m_form	= rekallTestOpenFormMode (0, "design", "Labels", KB::ShowAsDesign, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;
}

void	KBCopyPasteTest::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBCopyPasteTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBCopyPasteTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBLabel	*KBCopyPasteTest::getLabel
	(	const char	*name,
		const char	*text
	)
{
	KBNode	 *target = m_form->m_form->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBLabel")) ;

	KBLabel *label = (KBLabel *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(label->getName())
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(text)),
		_STDSTR(label->getText())
	)	;

	return	label	;
}

KBLabel	*KBCopyPasteTest::getLabel
	(	KBNode		*node
	)
{
	CPPUNIT_ASSERT (node != 0) ;
	CPPUNIT_ASSERT (node->isA("KBLabel")) ;

	return	(KBLabel *)node	;
}

KBControl
	*KBCopyPasteTest::getControl
	(	KBLabel	*label
	)
{
	KBControl *control = KBTestAccessor::getControl (label) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA("KBCtrlLabel")) ;
	return	  control  ;
}

QLabel	*KBCopyPasteTest::getWidget
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT (widget->isA("QLabel")) ;
	return	(QLabel *)widget ;
}

void	KBCopyPasteTest::testCopyPaste ()
{
	KBLabel   *label   = getLabel  (m_label, m_legend) ;
	KBControl *control = getControl(label  ) ;
	QWidget	  *widget  = getWidget (control) ;

	rekallGUIPause (m_label) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR
		(	QString	("Unexpected geometry: got %1, expected %2")
				.arg(_TEXT(control->ctrlGeometry ()))
				.arg(_TEXT(m_from))
		),
		control->ctrlGeometry() == m_from
	)	;

	KBFormCopier::self()->clearCopy() ;
	KBFormCopier::self()->addToCopy(label->replicate(0), label->objType()) ;

	QPtrList<KBNode> inserted = KBTestAccessor::insertHere
			 (	m_form->m_form,
				KBFormCopier::self()->getCopied(KB::ObjForm),
				m_form->m_form->getContainer(),
				QRect(190,210,1,1)
			 )	;

	rekallGUIPause (m_label) ;

	CPPUNIT_ASSERT (inserted.count() == 1) ;
	KBControl *control2 = getControl(getLabel(inserted.at(0))) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR
		(	QString	("Unexpected geometry: got %1, expected %2")
				.arg(_TEXT(control2->ctrlGeometry ()))
				.arg(_TEXT(m_at))
		),
		control2->ctrlGeometry() == m_at
	)	;
}

struct	CopyPasteTest
{
	const char	*m_tag		;
	const char	*m_label	;
	const char	*m_legend	;
	QRect		m_from		;
	QRect		m_at		;
}	;

static	CopyPasteTest	copyPasteTests[]=
{
	{	"Label_0",
		"Label_0",
		"fixed_fixed",
		QRect( 20, 20,150, 20),
		QRect(190,210,150, 20)
	},
	{	"Label_1",
		"Label_1",
		"fixed_stretch",
		QRect( 20, 70,150,310),
		QRect(190,210,150,170)
	},
	{	"Label_2",
		"Label_2",
		"fixed_float",
		QRect( 20,280,150, 20),
		QRect(190,190,150, 20)
	},
	{	"Label_3",
		"Label_3",
		"stretch_fixed",
		QRect(190, 20,190, 20),
		QRect(190,210,190, 20)
	},
	{	"Label_4",
		"Label_4",
		"stretch_stretch",
		QRect(190, 70,190,310),
		QRect(190,210,190,170)
	},
	{	"Label_5",
		"Label_5",
		"stretch_float",
		QRect(190,280,190, 20),
		QRect(190,190,190, 20)
	},
	{	"Label_6",
		"Label_6",
		"float_fixed",
		QRect(170, 20,150, 20),
		QRect(320,210,150, 20)
	},
	{	"Label_7",
		"Label_7",
		"float_stretch",
		QRect(170, 70,150,310),
		QRect(320,210,150,170)
	},
	{	"Label_8",
		"Label_8",
		"float_float",
		QRect(170,280,150, 20),
		QRect(320,190,150, 20)
	}
}	;

CppUnit::Test
	*KBCopyPasteTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Label") ;

	if (rekallTestSkip ("CopyPaste")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;


	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBCopyPasteTest> ("noDisplay", &KBCopyPasteTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBCopyPasteTest> ("withGUI",   &KBCopyPasteTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
		for (uint idx = 0 ; idx < sizeof(copyPasteTests)/sizeof(CopyPasteTest) ; idx += 1)
		{
			CopyPasteTest *cpt = &copyPasteTests[idx] ;

			s->addTest
			(	new CppUnit::TestCaller<KBCopyPasteTest>
				(	cpt->m_tag,
					&KBCopyPasteTest::testCopyPaste,
					new KBCopyPasteTest
					(	cpt->m_tag,
						cpt->m_label,
						cpt->m_legend,
						cpt->m_from,
						cpt->m_at
					)
			)	)	;
		}

	return s ;
}

CppUnit::Test EXPORT_API *copyPasteTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBCopyPasteTest::suite (__settings) ;
}
