/***************************************************************************
    file	         : coltypes.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<errno.h>

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qapplication.h>
#include	<qfileinfo.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_dbinfo.h"
#include	"kb_serverinfo.h"
#include	"kb_location.h"
#include	"kb_location_p.h"
#include	"kb_dom.h"

#include	"tester.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

static	QString	dbhost	;
static	QString	tmp	;
static	QString	xbase	;

class	KBColTypesTest : public CppUnit::TestCase
{
	QDomElement		m_spec		;


public	:

	KBColTypesTest	() ;
	KBColTypesTest	(std::string name, const QDomElement &) ;

	void			test		() ;
	static CppUnit::Test	*suite		() ;
}	;

KBColTypesTest::KBColTypesTest ()
{
}

KBColTypesTest::KBColTypesTest
	(	std::string		name,
		const QDomElement	&spec
	)
	:
	CppUnit::TestCase (name),
	m_spec		  (spec)
{
}

void	KBColTypesTest::test ()
{
	KBDBInfo	dbInfo	("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (&dbInfo, m_spec) ;
	dbInfo.insert	(svInfo);

	KBLocation	locn
			(	&dbInfo,
				"form",
				m_spec.attribute("ServerName"),
				"aname"
			)	;

        KBDBLink dbLink ;
	if (!dbLink.connect (&dbInfo, m_spec.attribute("ServerName"), true))
	{
		CPPUNIT_FAIL(_STDSTR(rekallTestError(dbLink.lastError()))) ;
                return 	;
        }

	KBError	error	;
	const RKColumnTypes *rkt = locn.columnTypes (dbLink, error) ;
	if (rkt == 0)
	{
		CPPUNIT_FAIL(_STDSTR(rekallTestError(error))) ;
                return 	;
        }

	QDomElement	types	;
	DOMITER_BEGIN(m_spec,"types",elem)
	{
		types	= elem	;
	}
	DOMITER_END

	CPPUNIT_ASSERT_EQUAL
	(	(int)rkt->m_types[RKT_ID]->getIType(),
		types.attribute("id").toInt()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(int)rkt->m_types[RKT_DESCRIPTION]->getIType(),
		types.attribute("description").toInt()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(int)rkt->m_types[RKT_DEFINITION]->getIType(),
		types.attribute("definition").toInt()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(int)rkt->m_types[RKT_SAVEDATE]->getIType(),
		types.attribute("savedate").toInt()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(int)rkt->m_types[RKT_TYPE]->getIType(),
		types.attribute("type").toInt()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(int)rkt->m_types[RKT_NAME]->getIType(),
		types.attribute("name").toInt()
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	(int)rkt->m_types[RKT_EXTENSION]->getIType(),
		types.attribute("extension").toInt()
	)	;
}

CppUnit::Test
	*KBColTypesTest::suite ()
{
	KBDomDocument	doc	;
	if (!doc.loadFile ("coltypes.xml", "test/driver", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"coltypes: %s: cannot open: %s\n",
			"coltypes.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	CppUnit::TestSuite *suite = new CppUnit::TestSuite ("ColTypes") ;

	DOMITER_BEGIN(doc.documentElement(), "serverinfo", elem)
	{
		if (rekallTestSkip (QString("ColTypes.%1").arg(elem.attribute("DBType"))))
			continue ;

		if (!dbhost.isEmpty())
			elem.setAttribute ("HostName", dbhost) ;

		if (!tmp   .isEmpty())
			if (elem.attribute("DBName") == "$TMP"  )
				elem.setAttribute ("DBName", tmp  ) ;
		if (!xbase .isEmpty())
			if (elem.attribute("DBName") == "$XBASE")
				elem.setAttribute ("DBName", xbase) ;

		suite->addTest
		(	new CppUnit::TestCaller<KBColTypesTest>
			(	_STDSTR(elem.attribute("DBType")),
				&KBColTypesTest::test,
				new KBColTypesTest (_STDSTR(elem.attribute("DBType")), elem)
		)	)	;
	}
	DOMITER_END

	return	suite	;
}

CppUnit::Test EXPORT_API *driverColTypesSetup
	(	const QMap<QString,QString> *settings
	)
{
	dbhost = (*settings)["dbhost"] ;
	tmp    = (*settings)["tmp"   ] ;
	xbase  = (*settings)["xbase" ] ;

	return	KBColTypesTest::suite () ;
}
