/***************************************************************************
    file	         : dummy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<errno.h>

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qapplication.h>
#include	<qfileinfo.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_dbinfo.h"
#include	"kb_serverinfo.h"
#include	"kb_location.h"
#include	"kb_location_p.h"
#include	"kb_dom.h"

#include	"tester.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


class	KBDummyTest : public CppUnit::TestCase
{
	KBServerInfo		*m_svInfo	;
	KBServer		*m_server	;

public	:

	KBDummyTest	() ;
	KBDummyTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testSelect	() ;
	void			testExpr1	() ;
	void			testExpr2	() ;
	void			testWhere	() ;

	static CppUnit::Test	*suite		() ;
}	;

static	const char *values_t[] =
{
	"One",
	"Two",
	"Three",
	"Four",
	"Five",
	"Six",
	"Seven",
	"Eight",
	"Nine"
}	;

static	const char *values_n[] =
{
	"1",
	"2",
	"3",
	"4",
	"5",
	"6",
	"7",
	"8",
	"9"
}	;

static	const char *values_n_2[] =
{
	"2",
	"4",
	"6",
	"8",
	"10",
	"12",
	"14",
	"16",
	"18"
}	;

KBDummyTest::KBDummyTest ()
{
	m_svInfo	= 0 ;
	m_server	= 0 ;
}

KBDummyTest::KBDummyTest
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
	m_svInfo	= 0 ;
	m_server	= 0 ;
}

void	KBDummyTest::setUp ()
{
	KBError	 error	;
	m_svInfo = new KBServerInfo
			(	0,
				"dummy",
				"test",
				QString::null,
				QString::null,
				"username",
				"password",
				QString::null,
				QString::null
			)	;
	m_server = m_svInfo->getServer (error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("server: %1").arg(error.getMessage())),
		m_server != 0 
	)	;
}

void	KBDummyTest::tearDown ()
{
	if (m_svInfo != 0) delete m_svInfo ;
}

void	KBDummyTest::testSelect ()
{
	KBSQLSelect *qrySelect = m_server->qrySelect
				 (	true,
					"select id, t, n from TableA",
					false
				 )	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("qrySelect: %1").arg(m_server->lastError().getMessage())),
		qrySelect != 0 
	)	;

	if (!qrySelect->execute (0, 0))
		CPPUNIT_FAIL (_STDSTR(QString("qrySelect: %1").arg(qrySelect->lastError().getMessage()))) ;

	for (uint idx = 0 ; idx < 8 ; idx += 1)
	{
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=0").arg(idx)),
			_STDSTR(QString::number(idx + 1)),
			_STDSTR(qrySelect->getField(idx, 0).getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=1").arg(idx)),
			_STDSTR(QString(values_t[idx])),
			_STDSTR(qrySelect->getField(idx, 1).getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=2").arg(idx)),
			_STDSTR(QString(values_n[idx])),
			_STDSTR(qrySelect->getField(idx, 2).getRawText())
		)	;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("qrySelect: expected null: r=%1 c=99: [%2]").arg(idx).arg(qrySelect->getField(idx, 99).getRawText())),
			qrySelect->getField(idx, 99).isNull()
		)	;
	}

	delete	qrySelect ;
}

void	KBDummyTest::testExpr1 ()
{
	KBSQLSelect *qrySelect = m_server->qrySelect
				 (	true,
					"select n, n * 2 from TableA",
					false
				 )	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("qrySelect: %1").arg(m_server->lastError().getMessage())),
		qrySelect != 0 
	)	;

	if (!qrySelect->execute (0, 0))
		CPPUNIT_FAIL (_STDSTR(QString("qrySelect: %1").arg(qrySelect->lastError().getMessage()))) ;

	for (uint idx = 0 ; idx < 8 ; idx += 1)
	{
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=0").arg(idx)),
			_STDSTR(QString::number(idx + 1)),
			_STDSTR(qrySelect->getField(idx, 0).getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=1").arg(idx)),
			_STDSTR(QString(values_n_2[idx])),
			_STDSTR(qrySelect->getField(idx, 1).getRawText())
		)	;
	}

	delete	qrySelect ;
}

void	KBDummyTest::testExpr2 ()
{
	KBSQLSelect *qrySelect = m_server->qrySelect
				 (	true,
					"select TableA.n, TableA.n * 2 from TableA",
					false
				 )	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("qrySelect: %1").arg(m_server->lastError().getMessage())),
		qrySelect != 0 
	)	;

	if (!qrySelect->execute (0, 0))
		CPPUNIT_FAIL (_STDSTR(QString("qrySelect: %1").arg(qrySelect->lastError().getMessage()))) ;

	for (uint idx = 0 ; idx < 8 ; idx += 1)
	{
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=0").arg(idx)),
			_STDSTR(QString::number(idx + 1)),
			_STDSTR(qrySelect->getField(idx, 0).getRawText())
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=1").arg(idx)),
			_STDSTR(QString(values_n_2[idx])),
			_STDSTR(qrySelect->getField(idx, 1).getRawText())
		)	;
	}

	delete	qrySelect ;
}

void	KBDummyTest::testWhere ()
{
	KBSQLSelect *qrySelect = m_server->qrySelect
				 (	true,
					"select id from TableA where n % 2 = 1",
					false
				 )	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("qrySelect: %1").arg(m_server->lastError().getMessage())),
		qrySelect != 0 
	)	;

	if (!qrySelect->execute (0, 0))
		CPPUNIT_FAIL (_STDSTR(QString("qrySelect: %1").arg(qrySelect->lastError().getMessage()))) ;

	for (uint idx = 0 ; idx < 5 ; idx += 1)
	{
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("qrySelect: unexpected value: r=%1 c=0").arg(idx)),
			_STDSTR(QString::number(idx * 2 + 1)),
			_STDSTR(qrySelect->getField(idx, 0).getRawText())
		)	;
	}

	delete	qrySelect ;
}

CppUnit::Test
	*KBDummyTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Dummy") ;

	s->addTest
	(	new CppUnit::TestCaller<KBDummyTest>
		(	"testSelect",
			&KBDummyTest::testSelect
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBDummyTest>
		(	"testExpr1",
			&KBDummyTest::testExpr1
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBDummyTest>
		(	"testExpr2",
			&KBDummyTest::testExpr2
	)	)	;
	s->addTest
	(	new CppUnit::TestCaller<KBDummyTest>
		(	"testWhere",
			&KBDummyTest::testWhere
	)	)	;

	return	s ;
}

CppUnit::Test EXPORT_API *driverDummySetup
	(	const QMap<QString,QString> *settings
	)
{
	return	KBDummyTest::suite () ;
}
