/***************************************************************************
    file	         : focus_event.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#define		private	public

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_ascii.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif


#if		__KB_PLAT_WIN32
#define		PAUSE	_sleep (1)
#else
#include	<unistd.h>
#define		PAUSE	usleep(100000)
#endif



class	KBFocusEventTest : public KBAPIIF
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;

	KBField			*field_id	;
	KBField			*field_t	;
	KBField			*field_n	;

	QString			nullToEmpty	(const QString &) ;

public	:

	KBFocusEventTest	() ;
	KBFocusEventTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testFocusEvent1	() ;
	void			testFocusEvent2	() ;
	void			testFocusEvent3	() ;
	void			testFocusEvent4	() ;
	void			testFocusEvent5	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBFocusEventTest::KBFocusEventTest ()
{
}

KBFocusEventTest::KBFocusEventTest
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

void	KBFocusEventTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("Focus.rkl", "test/focus") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "focus", "FocusAPI", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_docroot	= m_form->m_form	;

	field_id	= getField ("id", "id") ;
	field_t		= getField ("t",  "t" ) ;
	field_n		= getField ("n",  "n" ) ;

	KBControl	*c_n_0	= getControl	(field_n, 0, 0) ;
	RKLineEdit	*e_n_0	= getRKLineEdit	(c_n_0) ;

	e_n_0->setActiveWindow() ;
	for (uint idx = 0 ; idx < 10 ; idx += 1)
		if (!e_n_0->isActiveWindow())
		{	qApp->syncX() ;
			PAUSE ;
			qApp->processEvents() ;
		}

	CPPUNIT_ASSERT
	(	e_n_0->isActiveWindow()
	)	;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (e_n_0, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_n
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;

}

void	KBFocusEventTest::tearDown ()
{
	rekallTestCloseForm ()	;
	delete	m_dbInfo	;
}

void	KBFocusEventTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBFocusEventTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

QString	KBFocusEventTest::nullToEmpty
	(	const QString	&text
	)
{
	return	text.isNull() ? QString("") : text ;
}

void	KBFocusEventTest::testFocusEvent1 ()
{
	KBControl	*c_id_0	= getControl	(field_id, 0, 0) ;
	RKLineEdit	*e_id_0	= getRKLineEdit	(c_id_0) ;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (e_id_0, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBFocusEventTest::testFocusEvent2 ()
{
	KBControl	*c_id_0	= getControl	(field_id, 0, 0) ;
	KBControl	*c_n_0	= getControl	(field_n,  0, 0) ;

	RKLineEdit	*e_id_0	= getRKLineEdit	(c_id_0) ;
	RKLineEdit	*e_n_0	= getRKLineEdit	(c_n_0 ) ;

	e_n_0->setText ("DUFF") ;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (e_id_0, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_n
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBFocusEventTest::testFocusEvent3 ()
{
	KBControl	*c_id_0	= getControl	(field_id, 0, 0) ;
	KBControl	*c_n_0	= getControl	(field_n,  0, 0) ;

	RKLineEdit	*e_id_0	= getRKLineEdit	(c_id_0) ;
	RKLineEdit	*e_n_0	= getRKLineEdit	(c_n_0 ) ;

	e_n_0->setText ("12345") ;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (e_id_0, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBFocusEventTest::testFocusEvent4 ()
{
	KBControl	*c_id_0	= getControl	(field_id, 0, 0) ;
	RKLineEdit	*e_id_0	= getRKLineEdit	(c_id_0) ;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (e_id_0, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 0
	)	;
}

void	KBFocusEventTest::testFocusEvent5 ()
{
	KBControl	*c_id_1	= getControl	(field_id, 1, 1) ;
	RKLineEdit	*e_id_1	= getRKLineEdit	(c_id_1) ;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (e_id_1, &me) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus in unexpected item: %1").arg(KBTestAccessor::currentItem(m_form->m_form)->getName())),
		KBTestAccessor::currentItem(m_form->m_form) == field_id
	)	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Focus at unexpected row: %1").arg(KBTestAccessor::currentQRow(m_form->m_form))),
		KBTestAccessor::currentQRow(m_form->m_form) == 1
	)	;
}

CppUnit::Test
	*KBFocusEventTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("FocusEvent") ;

	if (rekallTestSkip ("FocusEvent")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBFocusEventTest> ("noDisplay", &KBFocusEventTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBFocusEventTest> ("withGUI",   &KBFocusEventTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBFocusEventTest>
			(	"focusEvent1",
				&KBFocusEventTest::testFocusEvent1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFocusEventTest>
			(	"focusEvent2",
				&KBFocusEventTest::testFocusEvent2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFocusEventTest>
			(	"focusEvent3",
				&KBFocusEventTest::testFocusEvent3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFocusEventTest>
			(	"focusEvent4",
				&KBFocusEventTest::testFocusEvent4
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBFocusEventTest>
			(	"focusEvent5",
				&KBFocusEventTest::testFocusEvent5
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *focusEventTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBFocusEventTest::suite (__settings) ;
}
