/***************************************************************************
    file	         : api_if.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qlabel.h>

#include	"rk_application.h"
#include	"rk_lineedit.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_label.h"
#include	"kb_button.h"
#include	"kb_field.h"
#include	"kb_choice.h"
#include	"kb_link.h"
#include	"kb_memo.h"
#include	"kb_textedit.h"
#include	"kb_richtext.h"
#include	"kb_qtrichtext.h"
#include	"kb_listbox.h"
#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"
#include	"api_if.h"



KBAPIIF::KBAPIIF ()
{
}

KBAPIIF::KBAPIIF
	(	std::string		name
	)
	:
	CppUnit::TestCase (name)
{
}

void	KBAPIIF::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBAPIIF::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

KBObject
	*KBAPIIF::getObject
	(	const char	*path,
		const char	*name,
		const char	*what
	)
{
	KBNode	 *target = m_docroot->getNamedNode(path) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("locating %1 as %2").arg(path).arg(what)),
		target != 0
	) 	;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("locating %1 as %2: class is %3").arg(path).arg(what).arg(target->className())),
		target->inherits("KBObject")
	)	;

	if (what != 0)
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("locating %1 as %2: class is %3").arg(path).arg(what).arg(target->className())),
			target->isA	(what)
		)	;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("locating %1 as %2").arg(path).arg(what)),
		_STDSTR(QString(name)),
		_STDSTR(target->getName())
	)	;

	return	(KBObject *)target	;
}

KBField
	*KBAPIIF::getField
	(	const char	*path,
		const char	*name
	)
{
	return	(KBField *)getObject (path, name, "KBField") ;
}

KBLabel
	*KBAPIIF::getLabel
	(	const char	*path,
		const char	*name
	)
{
	return	(KBLabel *)getObject (path, name, "KBLabel") ;
}

KBButton
	*KBAPIIF::getButton
	(	const char	*path,
		const char	*name
	)
{
	return	(KBButton *)getObject (path, name, "KBButton") ;
}

KBChoice
	*KBAPIIF::getChoice
	(	const char	*path,
		const char	*name
	)
{
	return	(KBChoice *)getObject (path, name, "KBChoice") ;
}

KBLink	*KBAPIIF::getLink
	(	const char	*path,
		const char	*name
	)
{
	return	(KBLink *)getObject (path, name, "KBLink") ;
}

KBMemo
	*KBAPIIF::getMemo
	(	const char	*path,
		const char	*name
	)
{
	return	(KBMemo *)getObject (path, name, "KBMemo") ;
}

KBRichText
	*KBAPIIF::getRichText
	(	const char	*path,
		const char	*name
	)
{
	return	(KBRichText *)getObject (path, name, "KBRichText") ;
}

KBListBox
	*KBAPIIF::getListBox
	(	const char	*path,
		const char	*name
	)
{
	return	(KBListBox *)getObject (path, name, "KBListBox") ;
}

KBSummary
	*KBAPIIF::getSummary
	(	const char	*path,
		const char	*name
	)
{
	return	(KBSummary *)getObject (path, name, "KBSummary") ;
}

KBFormBlock
	*KBAPIIF::getFormBlock
	(	const char	*path,
		const char	*name
	)
{
	KBNode	 *target = m_docroot->getNamedNode(name) ;

	CPPUNIT_ASSERT (target != 0) ;
	CPPUNIT_ASSERT (target->isA("KBFormBlock")) ;

	KBFormBlock *fblk = (KBFormBlock *)target ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(QString(name)),
		_STDSTR(fblk->getName())
	)	;

	return	fblk ;
}

KBControl
	*KBAPIIF::getControl
	(	KBObject	*object,
		const char	*what
	)
{
	KBControl *control = KBTestAccessor::getControl (object) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA(what)) ;
	return	  control  ;
}

KBControl
	*KBAPIIF::getControl
	(	KBItem		*item,
		const char	*what,
		uint		qrow,
		uint		drow
	)
{
	KBControl *control = KBTestAccessor::getControl (item, qrow) ;
	CPPUNIT_ASSERT (control != 0) ;
	CPPUNIT_ASSERT (control->isA(what)) ;
	CPPUNIT_ASSERT (control->getDRow() == drow ) ;
	return	  control  ;
}

KBControl
	*KBAPIIF::getControl
	(	KBField		*field
	)
{
	return	getControl (field, "KBCtrlField") ;
}

KBControl
	*KBAPIIF::getControl
	(	KBLabel		*label
	)
{
	return	getControl (label, "KBCtrlLabel") ;
}

KBControl
	*KBAPIIF::getControl
	(	KBField		*field,
		uint		qrow,
		uint		drow
	)
{
	return	getControl (field, "KBCtrlField", qrow, drow) ;
}

KBControl
	*KBAPIIF::getControl
	(	KBChoice	*choice
	)
{
	return	getControl (choice, "KBCtrlChoice") ;
}

KBControl
	*KBAPIIF::getControl
	(	KBChoice	*choice,
		uint		qrow,
		uint		drow
	)
{
	return	getControl (choice, "KBCtrlChoice", qrow, drow) ;
}

KBControl
	*KBAPIIF::getControl
	(	KBListBox	*listbox
	)
{
	return	getControl (listbox, "KBCtrlListBox") ;
}

KBControl
	*KBAPIIF::getControl
	(	KBMemo		*memo
	)
{
	return	getControl (memo, "KBCtrlMemo") ;
}

KBControl
	*KBAPIIF::getControl
	(	KBRichText	*richtext
	)
{
	return	getControl (richtext, "KBCtrlRichText") ;
}

KBControl
	*KBAPIIF::getControl
	(	KBListBox	*listbox,
		uint		qrow,
		uint		drow
	)
{
	return	getControl (listbox, "KBCtrlListBox", qrow, drow) ;
}

RKLineEdit
	*KBAPIIF::getRKLineEdit
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKLineEdit, got %1").arg(widget->className())),
		widget->isA("RKLineEdit")
	)	;
	return	(RKLineEdit *)widget ;
}

RKComboBox
	*KBAPIIF::getRKComboBox
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKComboBox, got %1").arg(widget->className())),
		widget->isA("RKComboBox")
	) ;
	return	(RKComboBox *)widget ;
}

RKListBox
	*KBAPIIF::getRKListBox
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKListBox, got %1").arg(widget->className())),
		widget->isA("RKListBox")
	)	;
	return	(RKListBox *)widget ;
}

RKTextEdit
	*KBAPIIF::getRKTextEdit
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;

	if (widget->isA("KBTextEdit"))
		widget = ((KBTextEdit *)widget)->textEdit() ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKTextEdit, got %1").arg(widget->className())),
		widget->inherits("RKTextEdit")
	)	;
	return	(RKTextEdit *)widget ;
}

RKTextBrowser
	*KBAPIIF::getRKTextBrowser
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;

	if (widget->isA("KBCtrlRichTextContainer"))
		widget = ((KBCtrlRichTextContainer *)widget)->wrapper() ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting RKTextEdit, got %1").arg(widget->className())),
		widget->inherits("RKTextBrowser")
	)	;
	return	(RKTextBrowser *)widget ;
}

QLabel	*KBAPIIF::getQLabel
	(	KBControl	*control
	)
{
	QWidget	*widget	  = control->getWidget() ;
	CPPUNIT_ASSERT (widget != 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Expecting QLabel, got %1").arg(widget->className())),
		widget->isA("QLabel")
	)	;
	return	(QLabel *)widget ;
}
