/***************************************************************************
    file	         : tester_kbase.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"


#include	"kb_locator.h"
#include	"kb_libloader.h"
#include	"kb_desktop.h"
#include	"kb_script.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_report.h"
#include	"kb_writer.h"
#include	"kb_sizer.h"

#include	<qmainwindow.h>
#include	<qwidgetstack.h>
#include	<qscrollview.h>

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

#if	__KB_RUNTIME
#define	PREFIX		"rekallrt_"
#else
#define	PREFIX		"rekall_"
#endif

static	TestOpenForm	*__testOpenForm		;
static	TestOpenReport	*__testOpenReport	;


/*  rekallTestLoadScripting						*/
/*		: Load scripting languages				*/
/*  (returns)	: void		: 					*/

void	EXPORT_API	rekallTestLoadScripting ()
{
	static	bool	scriptLoaded	= false ;

	if (scriptLoaded) return ;

	QString		 dtDir	= locateDir
				  (	"appdata",
					"services/rekall_dummy.desktop"
				  )	;
	QPtrList<KBDesktop> dtDefs	;

	KBDesktop::scan (dtDir + "/services", PREFIX,   dtDefs) ;
	KBLibLoader	*loader	= KBLibLoader::self () ;

	for (unsigned int idx = 0 ; idx < dtDefs.count() ; idx += 1)
	{
		KBDesktop *desktop = dtDefs.at(idx) ;

		if (desktop->property ("ServiceTypes") != "Rekall/Script")
			continue	;

		QString	libname	 = desktop->property("X-KDE-Library") ;
		if (libname.isEmpty()) continue ;

		KBLibrary *lib 	 = loader->getGlobalLibrary (libname) ;
		if (lib == 0)
		{
			fprintf
			(	stderr,
				"%s: %s: %s\n",
				rekallTestCommand(),
				libname.latin1(),
				loader->lastErrorMessage().latin1()
			)	;
			continue;
		}

		KBFactory *factory = (KBFactory *)lib->factory () ;
		if (factory ==  0)
		{	
			fprintf
			(	stderr,
				"%s: cannot create %s\n",
				rekallTestCommand(),
				libname.latin1()
			)	;
			continue  ;
		}

		QString	lang	= desktop->property("X-KDE-RekallPart-Language") ;
		QString	options	= desktop->property("X-KDE-RekallPart-Options" ) ;

//		fprintf
//		(	stderr,
//			"%s: got language [%s] f=%p\n",
//			rekallTestCommand(),
//			lang.latin1(),
//			(void  *)factory
//		)	;

		KBScriptIF::registerScript  (lang, factory, options) ;
	}

	scriptLoaded = true ;
}


static	KBForm	*loadForm
	(	KBDBInfo	*dbInfo,
		const QString	&which,
		const QString	&name,
		KBLocation	&location,
		KBError		&pError
	)
{
	const char *testpath = getenv("TESTPATH") ;

        QString path    = name	;

	if (testpath == 0)
		path	= locateFile("appdata", QString("test/%1/%2.rkl.frm").arg(which).arg(name)) ;
	else	path	= QString("%1/test/%2/%3.rkl.frm").arg(testpath).arg(which).arg(name) ;


	QByteArray data	;
	location = KBLocation
		   (	dbInfo,
			"form",
			KBLocation::m_pLocal,
			path
		   )	;
	if (!location.contents (data, pError))
		return	0 ;

	extern	KBForm	*KBOpenFormText (const KBLocation &, QByteArray &, KBError &) ;
	KBForm	*form 	;
	if ((form = KBOpenFormText (location, data, pError)) == 0)
		return	0 ;

	return	form	;
}

static	QWidget	*showForm
	(	KBForm		*form,
		KB::ShowAs	mode,
		KB::ShowRC	&showRC
	)
{
	QMainWindow	*window = new QMainWindow () ;

	QSize		size	;
	QDict<QString>	dict	;
	KBValue		value	;

	switch (mode)
	{
#if	! __KB_RUNTIME
		case KB::ShowAsDesign :
			showRC	= form->showDesign (window, size) ;
			break	;
#endif
		default	:
			showRC	= form->showData   (window, dict, value, size) ;
			break	;
	}

	QWidget	*top	= form->getDisplay()->getTopWidget() ;
	window->setCentralWidget (top)	;

	window->show     ()		;
	window->resize   (size.width(), size.height()) ;
	top   ->show	 () ;
	top   ->resize	 (size.width(), size.height()) ;

	return	window	;
}


EXPORT_API
	TestOpenForm	*rekallTestOpenFormMode
		(	KBDBInfo	*dbInfo,
			const QString	&which,
			const QString	&name,
			KB::ShowAs	mode,
			KBError		&pError
		)
{
	if (__testOpenForm != 0)
	{
		fprintf
		(	stderr,
			"rekallTestOpenFormMode: form still open\n"
		)	;
		exit	(1)	;
//		if (!reload && (__testOpenForm->m_name == name))
//			return	__testOpenForm	 ;
//
//		delete	__testOpenForm->m_widget ;
//		delete	__testOpenForm		 ;
//		__testOpenForm = 0		 ;
	}

	KBForm		*form	;
	QWidget		*widget	;
	KBLocation	location;
	KB::ShowRC	showRC	;

	if ((form   = loadForm (dbInfo, which, name, location, pError)) == 0)
		return	0	;

	if ((widget = showForm (form, mode, showRC)) == 0)
	{
		delete	form	;
		pError	= KBError
			  (	KBError::Error,
				"Failed to create form window",
				QString::null,
				__ERRLOCN
			  )	;
		return	0 ;
	}

	__testOpenForm	= new TestOpenForm ;
	__testOpenForm->m_name	   = name	;
	__testOpenForm->m_form	   = form	;
	__testOpenForm->m_widget   = widget 	;
	__testOpenForm->m_location = location	;
	__testOpenForm->m_showRC   = showRC	;

	return	__testOpenForm ;
}

EXPORT_API
	TestOpenForm	*rekallTestOpenForm
		(	KBDBInfo	*dbInfo,
			const QString	&which,
			const QString	&name,
			KBError		&pError
		)
{
	return	rekallTestOpenFormMode
		(	dbInfo,
			which,
			name,
			KB::ShowAsData,
			pError
		)	;
}

static	KBReport *loadReport
	(	 KBDBInfo	*dbInfo,
		 const QString	&which,
		 const QString	&name,
		 KBLocation	&location,
		 KBError	&pError
	)
{
	const char *testpath = getenv("TESTPATH") ;

        QString path    = name	;

	if (testpath == 0)
		path	= locateFile("appdata", QString("test/%1/%2.rkl.rep").arg(which).arg(name)) ;
	else	path	= QString("%1/test/%2/%3.rkl.rep").arg(testpath).arg(which).arg(name) ;


	QByteArray data	;
	location = KBLocation
		   (	dbInfo,
			"report",
			KBLocation::m_pLocal,
			path
		   )	;
	if (!location.contents (data, pError))
		return	0 ;

	extern	KBReport *KBOpenReportText (const KBLocation &, QByteArray &, KBError &) ;
	KBReport	 *report  ;
	if ((report = KBOpenReportText (location, data, pError)) == 0)
		return	0 ;

	KBTestAccessor::setAttibute (report, "printer", "__SCREEN__") ;
//	report->setAttrVal ("printer", "__SCREEN__") ;

	return	report	;
}

static	QWidget	*showReport
	(	KBReport	*report,
		KB::ShowAs	mode,
		KBLocation	&location,
		const QString	&which,
		KBWriter	*&writer,
		KB::ShowRC	&showRC
	)
{
	QMainWindow	*window = new QMainWindow  () ;
	QWidgetStack	*stack	= new QWidgetStack (window) ;
	window->setCentralWidget (stack) ;

	QWidget		*design	= new QWidget	   (stack ) ;
	QScrollView	*scroll	= new QScrollView  (stack ) ;

	QSize		size	;
	QDict<QString>	dict	;
	KBValue		value	;
	QString		printer	= report->getAttrVal("printer") ;

	if (!printer.isEmpty() && (printer != "__SCREEN__"))
	{
		const char *testpath = getenv("TESTPATH") ;
		if (testpath == 0)
			printer	= locateFile("appdata", QString("test/%1/%2.rkl.prn").arg(which).arg(printer)) ;
		else	printer	= QString("%1/test/%2/%3.rkl.prn").arg(testpath).arg(which).arg(printer) ;
		KBTestAccessor::setAttibute (report, "printer", printer) ;
	}

	switch (mode)
	{
#if	! __KB_RUNTIME
		case KB::ShowAsDesign :
			writer	= 0 ;
			showRC	= report->showDesign (design, size) ;
			stack->raiseWidget (design) ;
			break	;
#endif
		default	:
			writer	= new KBWriter       (scroll->viewport(), location) ;
			showRC	= report->showData   (design, writer, dict, value, size, false) ;
			stack ->raiseWidget (scroll) ;
		        scroll->addChild    (writer) ;
		        writer->show         () ;
			break	;
	}

        scroll  ->resizeContents (size.width(), size.height()) ;
        writer  ->resize         (size.width(), size.height()) ;

//	QWidget	*top	= report->getDisplay()->getTopWidget() ;
//	window->setCentralWidget (top)	;

	window ->show     ()	;
//	window->resize   (size.width(), size.height()) ;
//	top   ->show	 () ;
//	top   ->resize	 (size.width(), size.height()) ;

	return	window	;
}


EXPORT_API
	TestOpenReport	*rekallTestOpenReportMode
		(	KBDBInfo	*dbInfo,
			const QString	&which,
			const QString	&name,
			KB::ShowAs	mode,
			KBError		&pError
		)
{
	if (__testOpenReport != 0)
	{
		fprintf
		(	stderr,
			"rekallTestOpenReportMode: report still open\n"
		)	;
		exit	(1)	;

//		if (!reload && (__testOpenReport->m_name == name))
//			return	__testOpenReport ;
//
//		delete	__testOpenReport->m_widget ;
//		delete	__testOpenReport	 ;
//		__testOpenReport = 0		 ;
	}

	KBReport   *report  ;
	QWidget	   *widget  ;
	KBLocation location ;
	KBWriter   *writer  ;
	KB::ShowRC showRC   ;

	if ((report   = loadReport (dbInfo, which, name, location, pError)) == 0)
		return	0	;

	if ((widget = showReport (report, mode, location, which, writer, showRC)) == 0)
	{
		delete	report	;
		pError	= KBError
			  (	KBError::Error,
				"Failed to create report window",
				QString::null,
				__ERRLOCN
			  )	;
		return	0 ;
	}

	__testOpenReport		= new TestOpenReport ;
	__testOpenReport->m_name	= name	   ;
	__testOpenReport->m_report	= report   ;
	__testOpenReport->m_widget 	= widget   ;
	__testOpenReport->m_location	= location ;
	__testOpenReport->m_writer	= writer   ;
	__testOpenReport->m_showRC	= showRC   ;

	return	__testOpenReport ;
}

EXPORT_API
	TestOpenReport	*rekallTestOpenReport
		(	KBDBInfo	*dbInfo,
			const QString	&which,
			const QString	&name,
			KBError		&pError
		)
{
	return	rekallTestOpenReportMode
		(	dbInfo,
			which,
			name,
			KB::ShowAsData,
			pError
		)	;
}

TestOpenForm::TestOpenForm ()
{
	m_form		= 0 ;
	m_widget	= 0 ;
}

TestOpenForm::~TestOpenForm ()
{
	delete	m_form   ;
	delete	m_widget ;
}

TestOpenReport::TestOpenReport ()
{
	m_report	= 0 ;
	m_widget	= 0 ;
	m_writer	= 0 ;
}

TestOpenReport::~TestOpenReport ()
{
	delete	m_report ;
	delete	m_widget ;
}

EXPORT_API
	void	rekallTestCloseForm ()
{
	if (__testOpenForm != 0)
	{	delete	__testOpenForm ;
		__testOpenForm = 0     ;
	}
}

EXPORT_API
	void	rekallTestCloseReport ()
{
	if (__testOpenReport != 0)
	{	delete	__testOpenReport ;
		__testOpenReport = 0	 ;
	}
}
