/***************************************************************************
    file	         : macro1.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_ascii.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBMacro1Test : public KBAPIIF
{
	TestOpenForm		*m_form		;

	QString			nullToEmpty	(const QString &) ;

public	:

	KBMacro1Test	() ;
	KBMacro1Test	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testMacro11	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBMacro1Test::KBMacro1Test ()
{
}

KBMacro1Test::KBMacro1Test
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

void	KBMacro1Test::setUp ()
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("Macro.rkl", "test/macro", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"macro1: %s: cannot open: %s\n",
			"Macro.rkl",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_form	= rekallTestOpenForm (dbInfo, "macro", "Macro1", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage() + ": " + error.getDetails()),
		m_form != 0
	)	;

	m_docroot	= m_form->m_form ;
}

void	KBMacro1Test::tearDown ()
{
	rekallTestCloseForm () ;
}

void	KBMacro1Test::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBMacro1Test::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

QString	KBMacro1Test::nullToEmpty
	(	const QString	&text
	)
{
	return	text.isNull() ? QString("") : text ;
}

void	KBMacro1Test::testMacro11 ()
{
	KBField		*srce	= getField  ("Srce_0", "Srce_0") ;
	KBButton	*copy	= getButton ("Copy_0", "Copy_0") ;
	KBField		*dest	= getField  ("Dest_0", "Dest_0") ;

	srce->setValue	(0, KBValue("A.VALUE")) ;

	rekallGUIPause	("macro11") ;

	bool		evRc	;
	KBEvent	*event	= KBTestAccessor::findEvent(copy, "onclick") ;
	CPPUNIT_ASSERT (event != 0) ;

	KBTestAccessor::eventHook (copy, *event, 0, (KBValue *)0, evRc) ;
	rekallGUIPause	("macro11") ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Value error after macro execution")),
		_STDSTR(QString("A.VALUE")),
		_STDSTR(srce->getValue(0).getRawText())
	)	;
}

CppUnit::Test
	*KBMacro1Test::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Macro1") ;

	if (rekallTestSkip ("Macro1")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBMacro1Test> ("noDisplay", &KBMacro1Test::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBMacro1Test> ("withGUI",   &KBMacro1Test::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBMacro1Test>
			(	"macro11",
				&KBMacro1Test::testMacro11
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *macro1TestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBMacro1Test::suite (__settings) ;
}
