/***************************************************************************
    file	         : attr_legend.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#define		private	public

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>


#include	"kb_ascii.h"
#include	"kb_nodereg.h"
#include	"kb_node.h"
#include	"kb_propdict.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBMiscAttrLegendTest : public CppUnit::TestCase
{
	NodeSpec		*m_spec		;
	KBNode			*m_node		;

public	:

	KBMiscAttrLegendTest	() ;
	KBMiscAttrLegendTest	(std::string name, NodeSpec *spec) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testAttrLegend	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBMiscAttrLegendTest::KBMiscAttrLegendTest ()
{
}

KBMiscAttrLegendTest::KBMiscAttrLegendTest
	(	std::string		name,
		NodeSpec		*spec
	)
	:
	CppUnit::TestCase	(name),
	m_spec			(spec)
{
}

void	KBMiscAttrLegendTest::setUp ()
{
	QDict<QString> alist ;
	m_node	= (*m_spec->m_nodeFunc) (0, alist, 0) ;
	CPPUNIT_ASSERT (m_node != 0) ;
}

void	KBMiscAttrLegendTest::tearDown ()
{
	delete	m_node	;
}

void	KBMiscAttrLegendTest::noDisplay ()
{
}

void	KBMiscAttrLegendTest::withGUI ()
{
}

void	KBMiscAttrLegendTest::testAttrLegend ()
{
	for
	(	QPtrListIterator<KBAttr> iter (m_node->getAttribs()) ;
		iter.current() != 0 ;
		++iter
	)
	{
//		fprintf
//		(	stderr,
//			"\n %s.%s->[%s]\n",
//			m_spec->m_nodeName,
//			iter.current()->getName  ().latin1(),
//			iter.current()->getLegend().latin1()
//		)	;
		QString	tag	= QString("%1.%2").arg(m_spec->m_nodeName).arg(iter.current()->getName()) ;

		if (tag == "KBRowMark.dblclick"	) continue ;
		if (tag == "KBGrid.nosort"	) continue ;
		if (tag == "KBQrySQL.ptype"	) continue ;
		if (tag == "KBQryTable.ptype"	) continue ;
		if (tag == "KBQrySQL.pexpr"	) continue ;
		if (tag == "KBQryTable.pexpr"	) continue ;

		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("%1: no legend").arg(tag)),
			iter.current()->getLegend() != tag
		)	;
	}
}

CppUnit::Test
	*KBMiscAttrLegendTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("EventSkel") ;

	if (rekallTestSkip ("EventSkel")) return s ;

	extern	void	registerAllNodes	() ;
	registerAllNodes	() ;


	extern	QDict<NodeSpec>	&getFormNodeDict () ;
	for
	(	QDictIterator<NodeSpec> iter(getFormNodeDict()) ;
		iter.current() != 0 ;
		++iter
	)
	{

		
		if (qstrcmp(iter.current()->m_nodeName, "KBCompLink") == 0)
			continue ;

		char *tname  = new char[12 + qstrlen(iter.current()->m_nodeName) + 4] ;
		sprintf	(tname,  "%s_%s",  "attr_legend_", iter.current()->m_nodeName) ;

		s->addTest
		(	new CppUnit::TestCaller<KBMiscAttrLegendTest>
			(	tname,
				&KBMiscAttrLegendTest::testAttrLegend,
				new KBMiscAttrLegendTest (tname,  iter.current())
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *miscAttrLegendTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBMiscAttrLegendTest::suite (__settings) ;
}
