/***************************************************************************
    file	         : event_skel.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#define		private	public

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>


#include	"kb_ascii.h"
#include	"kb_nodereg.h"
#include	"kb_node.h"
#include	"kb_propdict.h"
#include	"kb_event.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBMiscEventSkelTest : public CppUnit::TestCase
{
	QString			m_element	;
	const char		*m_language	;

public	:

	KBMiscEventSkelTest	() ;
	KBMiscEventSkelTest	(std::string name, const char *, const char *) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testEventSkel	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBMiscEventSkelTest::KBMiscEventSkelTest ()
{
}

KBMiscEventSkelTest::KBMiscEventSkelTest
	(	std::string		name,
		const char		*element,
		const char		*language
	)
	:
	CppUnit::TestCase	(name),
	m_element		(element),
	m_language		(language)
{
}

void	KBMiscEventSkelTest::setUp ()
{
}

void	KBMiscEventSkelTest::tearDown ()
{
}

void	KBMiscEventSkelTest::noDisplay ()
{
}

void	KBMiscEventSkelTest::withGUI ()
{
}

void	KBMiscEventSkelTest::testEventSkel ()
{
	extern	QDict<NodeSpec>	&getFormNodeDict () ;

	NodeSpec *spec	= getFormNodeDict().find(m_element) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Element '%1' not found").arg(m_element)),
		spec != 0
	)	;

	QDict<QString>	aList ;
	KBNode	 *node	= (*spec->m_nodeFunc) (0, aList, 0) ;
	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Element '%1' not created").arg(m_element)),
		spec != 0
	)	;

	for
	(	QPtrListIterator<KBAttr> iter(node->getAttribs()) ;
		iter.current() != 0 ;
		++iter
	)
	{
		KBEvent *event = iter.current()->isEvent() ;
		if (event == 0)
			continue ;

		if ((m_element == "KBRowMark") && (event->getName() == "dblclick"))
			continue ;

//		fprintf
//		(	stderr,
//			"\n[%s.%s]\n",
//			m_element.latin1(),
//			event->getName().latin1()
//		)	;

		KBPropDictEntry *entry  = event->dictEntry (QString("%1.%2").arg(event->getName()).arg(m_language)) ;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR	(QString("Missing propdict '%1.%2' for '%3'")
					.arg(m_element)
					.arg(event->getName())
					.arg(m_language)
				),
			entry != 0
		)	;

		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR	(QString("Missing extras '%1.%2' for '%3'")
					.arg(m_element)
					.arg(event->getName())
					.arg(m_language)
				),
			entry->m_extra != 0
		)	;

		QString	*skeleton	= entry->m_extra->find("skeleton") ;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR	(QString("Missing skeleton '%1.%2' for '%3'")
					.arg(m_element)
					.arg(event->getName())
					.arg(m_language)
				),
			skeleton != 0
		)	;
	}

	delete	node	;
}

CppUnit::Test
	*KBMiscEventSkelTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("EventSkel") ;

	if (rekallTestSkip ("EventSkel")) return s ;

	extern	void	registerAllNodes	() ;
	registerAllNodes	() ;


	extern	QDict<NodeSpec>	&getFormNodeDict () ;
	for
	(	QDictIterator<NodeSpec> iter(getFormNodeDict()) ;
		iter.current() != 0 ;
		++iter
	)
	{
		if (qstrcmp(iter.current()->m_nodeName, "KBCompLink") == 0)
			continue ;

		char *tname_py  = new char[12 + qstrlen(iter.current()->m_nodeName) + 4] ;
		sprintf	(tname_py,  "%s_%s_py",  "event_skel", iter.current()->m_nodeName) ;

		s->addTest
		(	new CppUnit::TestCaller<KBMiscEventSkelTest>
			(	tname_py,
				&KBMiscEventSkelTest::testEventSkel,
				new KBMiscEventSkelTest (tname_py,  iter.current()->m_nodeName, "py" )
		)	)	;

		char *tname_kjs = new char[12 + qstrlen(iter.current()->m_nodeName) + 4] ;
		sprintf	(tname_kjs, "%s_%s_kjs", "event_skel", iter.current()->m_nodeName) ;

		s->addTest
		(	new CppUnit::TestCaller<KBMiscEventSkelTest>
			(	tname_kjs,
				&KBMiscEventSkelTest::testEventSkel,
				new KBMiscEventSkelTest (tname_kjs, iter.current()->m_nodeName, "kjs")
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *miscEventSkelTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBMiscEventSkelTest::suite (__settings) ;
}
