/***************************************************************************
    file	         : query1.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_dom.h"

#include	"tester.h"
#include	"tester_kbase.h"

#include	"kb_database.h"
#include	"kb_node.h"
#include	"kb_table.h"
#include	"kb_qryquery.h"
#include	"kb_select.h"


#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBQuery1Test : public CppUnit::TestCase
{
	QDomElement		m_spec		;
	KBQryQuery		*m_query	;

	QPtrList<KBTable>	m_qryList	;
	QPtrList<KBTable>	m_blkList	;

	void			buildSelect	(KBSelect &) ;
	KBTable			*findTable	(const QString &) ;
	KBTable			*findTable	(KBTable *, const QString &) ;

public	:

	KBQuery1Test	() ;
	KBQuery1Test	(std::string name, const QDomElement &) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			testQuery	() ;

	static	CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
	static	QString		nullToEmpty	(const QString &) ;
}	;

KBQuery1Test::KBQuery1Test ()
{
	m_query	= 0	;
}

KBQuery1Test::KBQuery1Test
	(	std::string		name,
		const QDomElement	&spec
	)
	:
	CppUnit::TestCase (name),
	m_spec		  (spec)
{
	m_query		= 0	;
}

void	KBQuery1Test::setUp ()
{
	QDict<QString>	qDict	;
	qDict.setAutoDelete (true) ;
	qDict.insert ("query",  new QString("query_name")) ;
	m_query	= new KBQryQuery (0, qDict, 0) ;

	DOMITER_BEGIN(m_spec,"KBTable",table)
	{
		QDict<QString>   tDict ;
		tDict.setAutoDelete (true) ;

		QDomNamedNodeMap tAttr = table.attributes() ;
		for (uint idx = 0 ; idx < tAttr.count() ; idx += 1)
		{	QDomAttr attr  = tAttr.item(idx).toAttr() ;
			tDict.insert (attr.name(), new QString(attr.value())) ;
		}

		new KBTable (m_query, tDict, 0) ;
	}
	DOMITER_END

	LITER
	(	KBNode,
		m_query->getChildren(),
		n,
		if (n->isTable() != 0) m_qryList.append(n->isTable()) ;
	)
}

void	KBQuery1Test::tearDown ()
{
	if (m_query != 0) delete m_query ;
}

void	KBQuery1Test::buildSelect
	(	KBSelect	&select
	)
{
	LITER
	(	KBTable,
		m_blkList,
		t,
			t->addToSelect (select, true) ;
	)

	select.appendExpr ("dummy") ;
}

KBTable	*KBQuery1Test::findTable
	(	KBTable		*table,
		const QString	&ident
	)
{
	if (table->getIdent() == ident)
		return	table	;

	TITER
	(	Table,
		table->getChildren(),
		t,
		KBTable *s = findTable (t, ident) ;
		if (s != 0) return s ;
	)

	return	0 ;
}

KBTable	*KBQuery1Test::findTable
	(	const QString	&ident
	)
{
	LITER
	(	KBTable,
		m_blkList,
		t,
		KBTable *s = findTable (t, ident) ;
		if (s != 0) return s ;
	)

	return	0 ;
}

void	KBQuery1Test::testQuery ()
{
	KBError	error	;
	bool	rc	= KBTable::blockUp
			  (	m_qryList,
				m_spec.attribute("toptable"),
				m_blkList,
				error
			  )	;
	if (!rc)
		CPPUNIT_FAIL (_STDSTR(rekallTestError(error))) ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("%1: Unexpected blocking count").arg(m_spec.attribute("id"))),
		(uint)m_spec.attribute("blkcount").toInt(),
		m_blkList.count()
	)	;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("%1: Top table not correctly found").arg(m_spec.attribute("id"))),
		_STDSTR(m_spec.attribute("toptable")),
		_STDSTR(m_blkList.at(0)->getIdent())
	)	;

	KBSelect select ;
	buildSelect (select) ;

	QString  query  = select.getQueryText (0) ;
	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("%1: Incorrect query").arg(m_spec.attribute("id"))),
		_STDSTR(m_spec.attribute("expect")),
		_STDSTR(query )
	)	;

	DOMITER_BEGIN(m_spec,"table",table)
	{
		KBTable	*t = findTable(table.attribute("ident")) ;
		CPPUNIT_ASSERT_MESSAGE
		(	_STDSTR(QString("%1: Table [%2] lost").arg(m_spec.attribute("id")).arg(table.attribute("ident"))),
			t != 0
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("%1: Join expression failed on [%2]").arg(m_spec.attribute("id")).arg(table.attribute("ident"))),
			_STDSTR(nullToEmpty(table.attribute("jexpr"))),
			_STDSTR(nullToEmpty(t->getJExpr()))
		)	;
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("%1: Join type failed on [%2]").arg(m_spec.attribute("id")).arg(table.attribute("ident"))),
			_STDSTR(nullToEmpty(table.attribute("jtype"))),
			_STDSTR(nullToEmpty(t->getJType()))
		)	;
	}
	DOMITER_END
}

QString	KBQuery1Test::nullToEmpty
	(	const QString	&string
	)
{
	return	string.isNull() ? QString("") : string ;
}

CppUnit::Test
	*KBQuery1Test::suite
	(	const QMap<QString,QString>	*settings
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile ("Query1.xml", "test/query", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"query1: cannot open: %s\n",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}

	extern	void	registerAllNodes () ;
	registerAllNodes	() ;

	CppUnit::TestSuite *s = new CppUnit::TestSuite("KBQueryTest") ;

	DOMITER_BEGIN(doc.documentElement(),"KBQuery",query)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBQuery1Test>
			(	_STDSTR(query.attribute("id")),
				&KBQuery1Test::testQuery,
				new KBQuery1Test (_STDSTR(query.attribute("id")), query)
		)	)	;
	}
	DOMITER_END

	return s ;
}

CppUnit::Test EXPORT_API *testSetup
	(	const QMap<QString,QString> *settings
	)
{

	return	KBQuery1Test::suite (settings) ;
}

#include	"kb_libloader.h"
#include	"factory.h"

TESTFACTORY	(KBTestQueryFactory,Query,rkt_query)
