/***************************************************************************
    file	         : basic.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_report.h"
#include	"kb_writer.h"
#include	"kb_sizer.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_memo.h"
#include	"kb_summary.h"
#include	"kb_ascii.h"

#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBReportsBasic : public KBAPIIF
{
	TestOpenReport		*m_report	;
	KBWriter		*m_writer	;

	void			openReport	(const char    *) ;
	QString			nullToEmpty	(const QString &) ;

protected :

	void			fieldHasText
				(	const char	*path,
					const char	*name,
					int		page,	
					int		row,
					const char	*text
				)	;

	void			summaryHasText
				(	const char	*path,
					const char	*name,
					int		page,	
					int		row,
					const char	*text
				)	;

	void			memoHasText
				(	const char	*path,
					const char	*name,
					int		page,	
					int		row,
					const char	*text
				)	;

	void			objectHasGeometry
				(	const char	*path,
					const char	*name,
					int		page,	
					int		row,
					const QRect	&rect
				)	;

public	:

	KBReportsBasic	() ;
	KBReportsBasic	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testBasic1	() ;
	void			testBasic2	() ;
	void			testBasic3	() ;
	void			testBasic4	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBReportsBasic::KBReportsBasic ()
{
}

KBReportsBasic::KBReportsBasic
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

void	KBReportsBasic::setUp ()
{
}

void	KBReportsBasic::tearDown ()
{
	rekallTestCloseReport () ;
}

void	KBReportsBasic::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBReportsBasic::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

QString	KBReportsBasic::nullToEmpty
	(	const QString	&text
	)
{
	return	text.isNull() ? QString("") : text ;
}

void	KBReportsBasic::openReport
	(	const char	*name
	)
{
	KBDomDocument	doc	;
	QDomElement	info	;
	if (!doc.loadFile ("API.rkl", "test/api", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"fields: %s: cannot open: %s\n",
			"API.xml",
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	DOMITER_BEGIN(doc.documentElement(),"serverinfo",elem)
	{
		info	= elem	;
	}
	DOMITER_END

	KBDBInfo	*dbInfo	= new KBDBInfo     ("noname") ;
	KBServerInfo	*svInfo	= new KBServerInfo (dbInfo, info) ;

	dbInfo->insert	(svInfo);

	KBError	error	;
	m_report	= rekallTestOpenReport (dbInfo, "reports", name, error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_report != 0
	)	;
	CPPUNIT_ASSERT_EQUAL
	(	KB::ShowRCData,
		m_report->m_showRC
	)	;

	m_docroot	= m_report->m_report ;
	m_writer	= m_report->m_report->getWriter() ;

	CPPUNIT_ASSERT	(m_writer != 0) ;
}

void	KBReportsBasic::fieldHasText
	(	const char	*path,
		const char	*name,
		int		page,
		int		row,
		const char	*text
	)
{
	KBField	 	*field	 = getField	      (path, name) ;
	KBWriterItem	*item	 = m_writer->findItem (page, field, row) ;

	CPPUNIT_ASSERT       (item != 0) ;
	CPPUNIT_ASSERT_EQUAL ((int)KBWriterItem::WriterText, (int)item->rtti()) ;

//	fprintf
//	(	stderr,
//		"KBReportsBasic::fieldHasText: path=[%s] name=[%s] page=%d row=%d text=[%s] got=[%s]\n",
//		path,
//		name,
//		page,
//		row,
//		text,
//		((KBWriterText *)item)->text().latin1()
//	)	;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Field value error: %1.%2").arg(path).arg(row)),
		_STDSTR(nullToEmpty(text)),
		_STDSTR(nullToEmpty(((KBWriterText *)item)->text()))
	)	;
}


void	KBReportsBasic::summaryHasText
	(	const char	*path,
		const char	*name,
		int		page,
		int		row,
		const char	*text
	)
{
	KBSummary 	*summary = getSummary	      (path, name) ;
	KBWriterItem	*item	 = m_writer->findItem (page, summary, row) ;

	CPPUNIT_ASSERT       (item != 0) ;
	CPPUNIT_ASSERT_EQUAL ((int)KBWriterItem::WriterText, (int)item->rtti()) ;

//	fprintf
//	(	stderr,
//		"KBReportsBasic::summaryHasText: path=[%s] name=[%s] page=%d row=%d text=[%s] got=[%s]\n",
//		path,
//		name,
//		page,
//		row,
//		text,
//		((KBWriterText *)item)->text().latin1()
//	)	;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Summary value error: %1.%2").arg(path).arg(row)),
		_STDSTR(nullToEmpty(text)),
		_STDSTR(nullToEmpty(((KBWriterText *)item)->text()))
	)	;
}


void	KBReportsBasic::memoHasText
	(	const char	*path,
		const char	*name,
		int		page,
		int		row,
		const char	*text
	)
{
	KBMemo	 	*memo	 = getMemo	      (path, name) ;
	KBWriterItem	*item	 = m_writer->findItem (page, memo, row) ;

	CPPUNIT_ASSERT       (item != 0) ;
	CPPUNIT_ASSERT_EQUAL ((int)KBWriterItem::WriterText, (int)item->rtti()) ;

//	fprintf
//	(	stderr,
//		"KBReportsBasic::memoHasText: path=[%s] name=[%s] page=%d row=%d text=[%s] got=[%s]\n",
//		path,
//		name,
//		page,
//		row,
//		text,
//		((KBWriterText *)item)->text().latin1()
//	)	;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Memo value error: %1.%2").arg(path).arg(row)),
		_STDSTR(nullToEmpty(text)),
		_STDSTR(nullToEmpty(((KBWriterText *)item)->text()))
	)	;
}

void	KBReportsBasic::objectHasGeometry
	(	const char	*path,
		const char	*name,
		int		page,
		int		row,
		const QRect	&rect
	)
{
	KBObject 	*object	 = getObject	      (path, name, 0) ;
	KBWriterItem	*item	 = m_writer->findItem (page, object, row) ;

	CPPUNIT_ASSERT       (item != 0) ;

//	fprintf
//	(	stderr,
//		"KBReportsBasic::objectHasGeometry: path=[%s] name=[%s] page=%d row=%d rect=[%s][%s]\n",
//		path,
//		name,
//		page,
//		row,
//		_TEXT(item->rect()),
//		_TEXT(rect)
//	)	;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Geometry error: %1.%2").arg(path).arg(row)),
		_STDSTR(QString(_TEXT(rect))),
		_STDSTR(QString(_TEXT(item->rect())))
	)	;
}


void	KBReportsBasic::testBasic1 ()
{
	openReport 	("TableA") ;
	rekallGUIPause	("basic1") ;
	fieldHasText	("id",   "id",   0, 0, "1"	   ) ;
	fieldHasText	("t",    "t",    0, 0, "One"	   ) ;
	fieldHasText	("n",    "n",    0, 0, "1"	   ) ;
	fieldHasText	("date", "date", 0, 0, "2006-01-01") ;
}

void	KBReportsBasic::testBasic2 ()
{
	openReport 	("TableA_Summary") ;
	rekallGUIPause	("basic2") ;
	fieldHasText	("id",   		"id",   	0, 0, "1"	   ) ;
	fieldHasText	("t",    		"t",    	0, 0, "One"	   ) ;
	fieldHasText	("n",    		"n",    	0, 0, "1"	   ) ;
	fieldHasText	("date", 		"date", 	0, 0, "2006-01-01" ) ;
	fieldHasText	("id",   		"id",   	0, 1, "2"	   ) ;
	fieldHasText	("t",    		"t",    	0, 1, "Two"	   ) ;
	fieldHasText	("n",    		"n",    	0, 1, "2"	   ) ;
	fieldHasText	("date", 		"date", 	0, 1, "2006-01-02" ) ;
	fieldHasText	("id",   		"id",   	0, 2, "3"	   ) ;
	fieldHasText	("t",    		"t",    	0, 2, "Three"	   ) ;
	fieldHasText	("n",    		"n",    	0, 2, "3"	   ) ;
	fieldHasText	("date", 		"date", 	0, 2, "2006-01-03" ) ;

	summaryHasText	("footer/id_sum",	"id_sum",	0, 3, "6"	   ) ;
	summaryHasText	("footer/id_min",	"id_min",	0, 3, "1"	   ) ;
	summaryHasText	("footer/id_max",	"id_max",	0, 3, "3"	   ) ;
	summaryHasText	("footer/t_min",	"t_min",	0, 3, "One"	   ) ;
	summaryHasText	("footer/t_max",	"t_max",	0, 3, "Two"	   ) ;
	summaryHasText	("footer/date_min",	"date_min",	0, 3, "2006-01-01" ) ;
	summaryHasText	("footer/date_max",	"date_max",	0, 3, "2006-01-03" ) ;
}

void	KBReportsBasic::testBasic3 ()
{
	openReport 	("TableA_Summary_Stretch") ;
	rekallGUIPause	("basic3") ;
	fieldHasText	("id",   		"id",   	0, 0, "1"	   ) ;
	fieldHasText	("t",    		"t",    	0, 0, "One"	   ) ;
	fieldHasText	("n",    		"n",    	0, 0, "1"	   ) ;
	fieldHasText	("date", 		"date", 	0, 0, "2006-01-01" ) ;
	fieldHasText	("id",   		"id",   	1, 1, "2"	   ) ;
	fieldHasText	("t",    		"t",    	1, 1, "Two"	   ) ;
	fieldHasText	("n",    		"n",    	1, 1, "2"	   ) ;
	fieldHasText	("date", 		"date", 	1, 1, "2006-01-02" ) ;
	fieldHasText	("id",   		"id",   	2, 2, "3"	   ) ;
	fieldHasText	("t",    		"t",    	2, 2, "Three"	   ) ;
	fieldHasText	("n",    		"n",    	2, 2, "3"	   ) ;
	fieldHasText	("date", 		"date", 	2, 2, "2006-01-03" ) ;

	summaryHasText	("footer/id_sum",	"id_sum",	0, 1, "1"	   ) ;
	summaryHasText	("footer/id_min",	"id_min",	0, 1, "1"	   ) ;
	summaryHasText	("footer/id_max",	"id_max",	0, 1, "1"	   ) ;
	summaryHasText	("footer/t_min",	"t_min",	0, 1, "One"	   ) ;
	summaryHasText	("footer/t_max",	"t_max",	0, 1, "One"	   ) ;
	summaryHasText	("footer/date_min",	"date_min",	0, 1, "2006-01-01" ) ;
	summaryHasText	("footer/date_max",	"date_max",	0, 1, "2006-01-01" ) ;

	summaryHasText	("footer/id_sum",	"id_sum",	1, 2, "3"	   ) ;
	summaryHasText	("footer/id_min",	"id_min",	1, 2, "1"	   ) ;
	summaryHasText	("footer/id_max",	"id_max",	1, 2, "2"	   ) ;
	summaryHasText	("footer/t_min",	"t_min",	1, 2, "One"	   ) ;
	summaryHasText	("footer/t_max",	"t_max",	1, 2, "Two"	   ) ;
	summaryHasText	("footer/date_min",	"date_min",	1, 2, "2006-01-01" ) ;
	summaryHasText	("footer/date_max",	"date_max",	1, 2, "2006-01-02" ) ;

	summaryHasText	("footer/id_sum",	"id_sum",	2, 3, "6"	   ) ;
	summaryHasText	("footer/id_min",	"id_min",	2, 3, "1"	   ) ;
	summaryHasText	("footer/id_max",	"id_max",	2, 3, "3"	   ) ;
	summaryHasText	("footer/t_min",	"t_min",	2, 3, "One"	   ) ;
	summaryHasText	("footer/t_max",	"t_max",	2, 3, "Two"	   ) ;
	summaryHasText	("footer/date_min",	"date_min",	2, 3, "2006-01-01" ) ;
	summaryHasText	("footer/date_max",	"date_max",	2, 3, "2006-01-03" ) ;
}

void	KBReportsBasic::testBasic4 ()
{
	openReport 	("Lines")  ;
	rekallGUIPause	("basic4") ;

//	fprintf
//	(	stderr,
//		"%s\n",
//		m_writer->describe().latin1()
//	)	;

	fieldHasText		("Id",	    "Id",   	0, 0, "1"	   ) ;
	fieldHasText		("Id",      "Id",   	0, 1, "2"	   ) ;
	fieldHasText		("Id",      "Id",   	0, 2, "3"	   ) ;
	memoHasText		("Memo_1",  "Memo_1",   0, 0, ""	   ) ;
	memoHasText		("Memo_1",  "Memo_1",   0, 1, ""	   ) ;
	memoHasText		("Memo_1",  "Memo_1",   0, 2, ""	   ) ;
	fieldHasText		("Field_1", "Field_1",  0, 0, ""	   ) ;
	fieldHasText		("Field_1", "Field_1",  0, 1, ""	   ) ;
	fieldHasText		("Field_1", "Field_1",  0, 2, ""	   ) ;

	objectHasGeometry	("Memo_1",  "Memo_1",   0, 0, QRect( 50, 50,140,20)) ;
	objectHasGeometry	("Memo_1",  "Memo_1",   0, 1, QRect( 50,100,140,20)) ;
	objectHasGeometry	("Memo_1",  "Memo_1",   0, 2, QRect( 50,150,140,20)) ;

	objectHasGeometry	("Field_1", "Field_1",  0, 0, QRect(200, 50,120,20)) ;
	objectHasGeometry	("Field_1", "Field_1",  0, 1, QRect(200,100,120,20)) ;
	objectHasGeometry	("Field_1", "Field_1",  0, 2, QRect(200,150,120,20)) ;
}

CppUnit::Test
	*KBReportsBasic::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Container") ;

	if (rekallTestSkip ("Reports")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBReportsBasic> ("noDisplay", &KBReportsBasic::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBReportsBasic> ("withGUI",   &KBReportsBasic::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBReportsBasic>
			(	"report_basic1",
				&KBReportsBasic::testBasic1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBReportsBasic>
			(	"report_basic2",
				&KBReportsBasic::testBasic2
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBReportsBasic>
			(	"report_basic3",
				&KBReportsBasic::testBasic3
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBReportsBasic>
			(	"report_basic4",
				&KBReportsBasic::testBasic4
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *basicTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBReportsBasic::suite (__settings) ;
}
