/***************************************************************************
    file	         : kjs.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"kb_kjsscript.h"
#include	"kb_idbase.h"
#include	"kb_objectproxy.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


class	KBKJSTest : public CppUnit::TestCase
{
	KJS::Interpreter	*m_interp	;
	KJS::ExecState		*m_exec		;

public	:

	KBKJSTest	() ;
	KBKJSTest	(std::string name) ;

	virtual	void		setUp			() ;
	virtual	void		tearDown		() ;

	void			testStringArg1	() ;
	void			testStringArg2	() ;
	void			testStringArg3	() ;
	void			testStringArg4	() ;
	void			testStringArg5	() ;
	void			testStringArg6	() ;
	void			testBooleanArg1	() ;
	void			testBooleanArg2	() ;
	void			testBooleanArg3	() ;
	void			testBooleanArg4	() ;
	void			testBooleanArg5	() ;
	void			testBooleanArg6	() ;
	void			testBooleanArg7	() ;
	void			testNumberArg1	() ;
	void			testNumberArg2	() ;
	void			testNumberArg3	() ;
	void			testNumberArg4	() ;

	void			testCheckArgs1	() ;
	void			testCheckArgs2	() ;
	void			testCheckArgs3	() ;

	static CppUnit::Test	*suite		() ;
}	;

KBKJSTest::KBKJSTest ()
{
	m_interp = 0 ;
}

KBKJSTest::KBKJSTest
	(	std::string		name
	)
	:
	CppUnit::TestCase 	(name)
{
	m_interp = 0 ;
}

void	KBKJSTest::setUp ()
{
	m_interp = new KJS::Interpreter   ;
	m_exec	 = m_interp->globalExec() ;
}

void	KBKJSTest::tearDown ()
{
	if (m_interp != 0) delete m_interp ;
}

void	KBKJSTest::testStringArg1 ()
{
	KJS::List	args	;
	args.append (KJS::String("String")) ;
	args.append (KJS::String("Text"  )) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(kjsStringArg (m_exec, args, 0, QString::null)),
		_STDSTR(QString("String"))
	)	;
}

void	KBKJSTest::testStringArg2 ()
{
	KJS::List	args	;
	args.append (KJS::String("String")) ;
	args.append (KJS::String("Text"  )) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(kjsStringArg (m_exec, args, 1, QString::null)),
		_STDSTR(QString("Text"))
	)	;
}

void	KBKJSTest::testStringArg3 ()
{
	KJS::List	args	;
	args.append (KJS::Number(42)) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(kjsStringArg (m_exec, args, 0, QString::null)),
		_STDSTR(QString("42"))
	)	;
}

void	KBKJSTest::testStringArg4 ()
{
	KJS::List	args	;
	args.append (KJS::Boolean(false)) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(kjsStringArg (m_exec, args, 0, QString::null)),
		_STDSTR(QString("0"))
	)	;
}

void	KBKJSTest::testStringArg5 ()
{
	KJS::List	args	;
	args.append (KJS::Boolean(true)) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(kjsStringArg (m_exec, args, 0, QString::null)),
		_STDSTR(QString("1"))
	)	;
}

void	KBKJSTest::testStringArg6 ()
{
	KJS::List	args	;
	args.append (KJS::Number(42.123)) ;

	CPPUNIT_ASSERT_EQUAL
	(	_STDSTR(kjsStringArg (m_exec, args, 0, QString::null)),
		_STDSTR(QString("42.123"))
	)	;
}

void	KBKJSTest::testBooleanArg1 ()
{
	KJS::List	args	;
	args.append (KJS::Boolean(false)) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsBooleanArg (m_exec, args, 0, true),
		false
	)	;
}

void	KBKJSTest::testBooleanArg2 ()
{
	KJS::List	args	;
	args.append (KJS::Boolean(true)) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsBooleanArg (m_exec, args, 0, false),
		true
	)	;
}

void	KBKJSTest::testBooleanArg3 ()
{
	KJS::List	args	;
	args.append (KJS::Number(0)) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsBooleanArg (m_exec, args, 0, true),
		false
	)	;
}

void	KBKJSTest::testBooleanArg4 ()
{
	KJS::List	args	;
	args.append (KJS::Number(1)) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsBooleanArg (m_exec, args, 0, false),
		true
	)	;
}

void	KBKJSTest::testBooleanArg5 ()
{
	KJS::List	args	;
	args.append (KJS::String("")) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsBooleanArg (m_exec, args, 0, false),
		false
	)	;
}

void	KBKJSTest::testBooleanArg6 ()
{
	KJS::List	args	;
	args.append (KJS::String("0")) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsBooleanArg (m_exec, args, 0, false),
		true
	)	;
}

void	KBKJSTest::testBooleanArg7 ()
{
	KJS::List	args	;
	args.append (KJS::String("1")) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsBooleanArg (m_exec, args, 0, true),
		true
	)	;
}

void	KBKJSTest::testNumberArg1 ()
{
	KJS::List	args	;
	args.append (KJS::Number(0)) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsNumberArg (m_exec, args, 0, 999),
		0
	)	;
}

void	KBKJSTest::testNumberArg2 ()
{
	KJS::List	args	;
	args.append (KJS::Number(42)) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsNumberArg (m_exec, args, 0, 999),
		42
	)	;
}

void	KBKJSTest::testNumberArg3 ()
{
	KJS::List	args	;
	args.append (KJS::String("0")) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsNumberArg (m_exec, args, 0, 999),
		0
	)	;
}

void	KBKJSTest::testNumberArg4 ()
{
	KJS::List	args	;
	args.append (KJS::String("42")) ;

	CPPUNIT_ASSERT_EQUAL
	(	kjsNumberArg (m_exec, args, 0, 999),
		42
	)	;
}

void	KBKJSTest::testCheckArgs1 ()
{
	KJS::List	args	;
	CPPUNIT_ASSERT ( KBObjectProxy::checkArgs (m_exec, args, 0)) ;
}

void	KBKJSTest::testCheckArgs2 ()
{
	KJS::List	args	;
	args.append (KJS::String("string")) ;
	CPPUNIT_ASSERT ( KBObjectProxy::checkArgs (m_exec, args, "s")) ;
}

void	KBKJSTest::testCheckArgs3 ()
{
	KJS::List	args	;
	args.append (KJS::String("string")) ;
	CPPUNIT_ASSERT (!KBObjectProxy::checkArgs (m_exec, args, "n")) ;
}


CppUnit::Test
	*KBKJSTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite ("KJS") ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("stringArg1",  &KBKJSTest::testStringArg1 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("stringArg2",  &KBKJSTest::testStringArg2 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("stringArg3",  &KBKJSTest::testStringArg3 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("stringArg4",  &KBKJSTest::testStringArg4 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("stringArg5",  &KBKJSTest::testStringArg5 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("stringArg6",  &KBKJSTest::testStringArg6 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("booleanArg1", &KBKJSTest::testBooleanArg1)) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("booleanArg2", &KBKJSTest::testBooleanArg2)) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("booleanArg3", &KBKJSTest::testBooleanArg3)) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("booleanArg4", &KBKJSTest::testBooleanArg4)) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("booleanArg5", &KBKJSTest::testBooleanArg5)) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("booleanArg6", &KBKJSTest::testBooleanArg6)) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("booleanArg7", &KBKJSTest::testBooleanArg7)) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("numberArg1",  &KBKJSTest::testNumberArg1 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("numberArg2",  &KBKJSTest::testNumberArg2 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("numberArg3",  &KBKJSTest::testNumberArg3 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("numberArg4",  &KBKJSTest::testNumberArg4 )) ;

	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("checkArgs1",  &KBKJSTest::testCheckArgs1 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("checkArgs2",  &KBKJSTest::testCheckArgs2 )) ;
	s->addTest (new CppUnit::TestCaller<KBKJSTest> ("checkArgs3",  &KBKJSTest::testCheckArgs3 )) ;
	return	s ;
}

CppUnit::Test EXPORT_API *kjsSetup
	(	const QMap<QString,QString> *
	)
{
	return	KBKJSTest::suite () ;
}

