/***************************************************************************
    file	         : py.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"kb_script.h"
#include	"kb_form.h"
#include	"kb_locator.h"
#include	"kb_test.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif


class	KBPYTest : public CppUnit::TestCase
{
	KBScriptIF	*m_iface	;

	void	testImport
	(	const char	*code,
		bool		compileOK,
		bool		execOK,
		int		expected
	)	;
	void	testError
	(	const char	*code,
		uint		elno,
		const char	*etext
	)	;

public	:

	KBPYTest	() ;
	KBPYTest	(std::string name) ;

	virtual	void		setUp			() ;
	virtual	void		tearDown		() ;

	void			testImport0	() ;
	void			testImport1	() ;
	void			testImport2	() ;
	void			testImport3	() ;
	void			testImport4	() ;
	void			testImport5	() ;
	void			testImport6	() ;

	void			testError0	() ;
	void			testError1	() ;
	static CppUnit::Test	*suite		() ;
}	;

KBPYTest::KBPYTest ()
{
	m_iface = 0 ;
}

KBPYTest::KBPYTest
	(	std::string		name
	)
	:
	CppUnit::TestCase 	(name)
{
	m_iface = 0 ;
}

void	KBPYTest::setUp ()
{
	KBError	   error    ;
	m_iface  = LinkKBScript ("py", error) ;
	if (m_iface == 0)
		CPPUNIT_FAIL (_STDSTR(error.getMessage())) ;

	KBTest::setTestMode (KBScriptIF::testOff) ;
}

void	KBPYTest::tearDown ()
{
	if (m_iface != 0) delete m_iface ;
}

void	KBPYTest::testImport
	(	const char	*script,
		bool		compileOK,
		bool		execOK,
		int		expected
	)
{
        const char	*testpath = getenv("TESTPATH") ;

	KBError		error	;
        QString 	path    ;

        if (testpath == 0)
                path    = locateFile("appdata", "test/script/dummy.rkl.frm") ;
        else    path    = QString("%1/test/script/dummy.rkl.frm").arg(testpath) ;

        KBLocation locn
                   (    0,
                        "form",
                        KBLocation::m_pLocal,
                        path
                   )    ;

	QDict<QString>	dict	 ;
	KBForm		*node	 = new KBForm (locn, dict) ;

	KBScriptCode	*code	= m_iface->compileFunc
				  (	node,
					script,
				  	QString::null,
					"eventFunc",
					QStringList(),
					0,
					error
	    			  )	;

	if ( compileOK && (code == 0))
	{	delete	node ;
		CPPUNIT_FAIL (_STDSTR(QString("compile failed: %2").arg(error.getMessage()))) ;
	}
	if (!compileOK && (code != 0))
	{	delete	node ;
		CPPUNIT_FAIL (_STDSTR(QString("compile succeeded"))) ;
	}

	if (code == 0)
	{	delete	node ;
		return	;
	}


	KBValue	        rv	 ;
	KBScript::ExeRC rc	 = code->execute (0, 0, rv) ;
	delete		node	 ;

	QString		eMsg	;
	uint 		eLine	;
	QString		eText	 ;
	m_iface->lastError (eMsg, eLine, eText) ;

	switch (rc)
	{
		case KBScript::ExeError :
                case KBScript::ExeFail  :
                case KBScript::ExeAbort :
                case KBScript::ExeTest  :
			if ( execOK)
				CPPUNIT_FAIL (_STDSTR(QString("Execution failed: %1: %2").arg((int)rc).arg(eMsg))) ;
			return	;

                case KBScript::ExeTrue  :
                case KBScript::ExeFalse :
			if (!execOK)
				CPPUNIT_FAIL (_STDSTR(QString("Execution succeeded: %1").arg((int)rc))) ;
			break	;

		default	:
 			CPPUNIT_FAIL (_STDSTR(QString("Execution error: %1").arg((int)rc))) ;
			break	;
	}

	if (rv.getRawText().toInt() != expected)
		CPPUNIT_ASSERT_EQUAL_MESSAGE
		(	_STDSTR(QString("Unexpected result")),
			expected,
			rv.getRawText().toInt()
		)	;
}


void	KBPYTest::testImport0 ()
{
	const char *script =
		"import t_localmod0\n"
		"def eventFunc (self) :\n"
		"    return t_localmod0.value\n"
		"\n"
		;

	testImport (script, false, false, 1) ;
}

void	KBPYTest::testImport1 ()
{
	const char *script =
		"def eventFunc (self) :\n"
		"    import t_localmod0\n"
		"    return t_localmod0.value\n"
		"\n"
		;

	testImport (script, true, false, 1) ;
}

void	KBPYTest::testImport2 ()
{
	const char *script =
		"def eventFunc (self) :\n"
		"    import t_localmod1\n"
		"    return t_localmod1.value\n"
		"\n"
		;

	testImport (script, true, true, 1) ;
}

void	KBPYTest::testImport3 ()
{
	const char *script =
		"def eventFunc (self) :\n"
		"    import t_localmod2\n"
		"    return t_localmod2.value\n"
		"\n"
		;

	testImport (script, true, true, 2) ;
}

void	KBPYTest::testImport4 ()
{
	const char *script =
		"def eventFunc (self) :\n"
		"    import t_localmod1\n"
		"    import t_localmod1\n"
		"    import t_localmod2\n"
		"    import t_localmod2\n"
		"    return t_localmod1.value + t_localmod2.value\n"
		"\n"
		;

	testImport (script, true, true, 3) ;
}

void	KBPYTest::testImport5 ()
{
	const char *script =
		"import t_localmod1\n"
		"import t_localmod2\n"
		"def eventFunc (self) :\n"
		"    return t_localmod1.value + t_localmod2.value\n"
		"\n"
		;

	testImport (script, true, true, 3) ;
}

void	KBPYTest::testImport6 ()
{
	const char *script =
		"def eventFunc (self) :\n"
		"    import t_localmod1\n"
		"    t_localmod1.value = 101\n"
		"    import t_localmod1\n"
		"    import t_localmod2\n"
		"    t_localmod2.value = 102\n"
		"    import t_localmod2\n"
		"    return t_localmod1.value + t_localmod2.value\n"
		"\n"
		;

	testImport (script, true, true, 203) ;
}

void	KBPYTest::testError
	(	const char	*script,
		uint		lno,
		const char	*etext
	)
{
        const char	*testpath = getenv("TESTPATH") ;

	KBError		error	;
        QString 	path    ;

        if (testpath == 0)
                path    = locateFile("appdata", "test/script/dummy.rkl.frm") ;
        else    path    = QString("%1/test/script/dummy.rkl.frm").arg(testpath) ;

        KBLocation locn1
		   (	0,
			"script",
			KBLocation::m_pInline,
			"path",
			script
		   )	;
	bool	   rc	 = m_iface->compile (locn1, error) ;

	if (rc)
		CPPUNIT_FAIL (_STDSTR(QString("Compile unexpectedly succeeded"))) ;


	QString	   eMsg	 ;
	uint	   eLine ;
	QString	   eText ;
	KBLocation locn2 = m_iface->lastError (eMsg, eLine, eText) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Unexpected error line: got %1, expected %2").arg(eLine).arg(lno)),
		eLine == lno
	)	;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Unexpected error message: %1").arg(eMsg)),
		(bool)(eMsg == etext)
	)	;
}

void	KBPYTest::testError0 ()
{
	const char *script =
		"def eventFunc (self) :\n"
		"    return obj obj\n"
		"\n"
		;

	testError (script, 2, "path : 2: invalid syntax") ;
}

void	KBPYTest::testError1 ()
{
	const char *script =
		"\n"
		"\n"
		"\n"
		"defx eventFunc (self) :\n"
		"    return obj\n"
		"\n"
		;

	testError (script, 4, "path : 4: invalid syntax") ;
}


CppUnit::Test
	*KBPYTest::suite ()
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite ("PY") ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("import0",  &KBPYTest::testImport0)) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("import1",  &KBPYTest::testImport1)) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("import2",  &KBPYTest::testImport2)) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("import3",  &KBPYTest::testImport3)) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("import4",  &KBPYTest::testImport4)) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("import5",  &KBPYTest::testImport5)) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("import6",  &KBPYTest::testImport6)) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("error0",   &KBPYTest::testError0 )) ;
	s->addTest (new CppUnit::TestCaller<KBPYTest> ("error1",   &KBPYTest::testError1 )) ;
	return	s ;
}

CppUnit::Test EXPORT_API *pySetup
	(	const QMap<QString,QString> *
	)
{
	return	KBPYTest::suite () ;
}

