/***************************************************************************
    file	         : scriptif.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qfile.h>
#include	<qstring.h>
#include	<qmap.h>

#include	"kb_libloader.h"
#include	"kb_locator.h"
#include	"kb_desktop.h"
#include	"kb_form.h"
#include	"kb_script.h"
#include	"kb_locator.h"
#include	"kb_dom.h"
#include	"kb_test.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif

class	KBScriptIFTest : public CppUnit::TestCase
{
	uint			getArgs		(const QDomElement &, KBValue *) ;
	void			evaluate	(const QDomElement &)	   ;

	QDomElement		m_elem		;

public	:

	KBScriptIFTest	() ;
	KBScriptIFTest	(std::string name, const QDomElement &) ;

	void			testScript	() ;

	static CppUnit::Test	*suite		() ;
}	;

KBScriptIFTest::KBScriptIFTest ()
{
}

KBScriptIFTest::KBScriptIFTest
	(	std::string		name,
		const QDomElement 	&elem
	)
	:
	CppUnit::TestCase 	(name),
	m_elem			(elem)
{
}

uint	KBScriptIFTest::getArgs
	(	const QDomElement	&parent,
		KBValue			*argv
	)
{
	uint	argc	= 0 ;

	DOMITER_BEGIN(parent, "arg", elem)
	{
		QString	    typ	 = elem.attribute("type" ) ;
		QString	    val	 = elem.attribute("value") ;

		if (typ == "fixed")
		{
			argv[argc] = val.toInt() ;
			argc	  += 1 ;
			continue  ;
		}

		if (typ == "float")
		{
			argv[argc] = val.toDouble() ;
			argc	  += 1 ;
			continue  ;
		}

		argv[argc] = val ;
		argc	  += 1 ;
	}
	DOMITER_END

	return	argc	;
}

void	KBScriptIFTest::evaluate
	(	const QDomElement	&elem
	)
{
//	fprintf
//	(	stderr,
//		"KBScriptIFTest::evaluate: %s\n",
//		elem.attribute("id").latin1()
//	)	;

	if (elem.nodeName() == "inline")
	{
		QString		id	= elem.attribute("id") ;

		KBError	   error    ;
		KBScriptIF *iface  = LinkKBScript (elem.attribute("language"), error) ;
		if (iface == 0)
			CPPUNIT_FAIL (_STDSTR(QString("%1: %2").arg(id).arg(error.getMessage()))) ;

		uint		mode	= elem.attribute("testmode").toInt() ;
		QString		fname	= elem.attribute("fname") ;
		QString		erc	= elem.attribute("rc") ;
		QString		erv	= elem.attribute("rv") ;


		KBLocation	locn	 ;
		QDict<QString>	dict	 ;
		KBForm		*node	 = new KBForm (locn, dict) ;

		QString		text	= elem.text().stripWhiteSpace() + "\n" ;
		KBScriptCode	*code	= iface->compileFunc
					  (	node,
						text,
					  	QString::null,
						fname,
						QStringList(),
						0,
						error
		    			  )	;


		if (erc.toInt() < 0)
		{
			if (code != 0)
			{	delete	node ;
				CPPUNIT_FAIL (_STDSTR(QString("%1: compile unexpectedly OK").arg(id))) ;
			}
			return	;
		}

		if (code == 0)
		{
			delete	node ;
			CPPUNIT_FAIL (_STDSTR(QString("%1: compile failed: %2").arg(id).arg(error.getMessage()))) ;
		}

		KBValue		args[64] ;
		uint		argc	 = getArgs (elem, args) ;
		KBValue	        rv	 ;

		KBTest::setTestMode ((KBScriptIF::TestMode)mode) ;
		KBScript::ExeRC rc	 = code->execute (argc, args, rv) ;
		KBTest::setTestMode (KBScriptIF::testOff) ;

		delete		node	 ;

		if (rc != erc.toInt())
			CPPUNIT_FAIL (_STDSTR(QString("%1: %2: expected return code: %3").arg(id).arg(rc).arg(erc.toInt()))) ;
		if (rv.getRawText() != erv)
			CPPUNIT_FAIL (_STDSTR(QString("%1: %2: expected result: %3").arg(id).arg(rv.getRawText()).arg(erv))) ;

		return	;
	}

	if (elem.nodeName() == "expr")
	{
		QString		id	= elem.attribute("id") ;

		KBError	   error    ;
		KBScriptIF *iface  = LinkKBScript (elem.attribute("language"), error) ;
		if (iface == 0)
			CPPUNIT_FAIL (_STDSTR(QString("%1: %2").arg(id).arg(error.getMessage()))) ;

		QString		expr	= elem.attribute("expr") ;
		QString		erc	= elem.attribute("rc") ;
		QString		erv	= elem.attribute("rv") ;

		KBLocation	locn	 ;
		QDict<QString>	dict	 ;
		KBForm		*node	 = new KBForm (locn, dict) ;

		KBScriptCode	*code	= iface->compileExpr
					  (	node,
						expr,
					  	QString::null,
						QStringList(),
						error
		    			  )	;

		if (erc.toInt() < 0)
		{
			if (code != 0)
			{	delete	node ;
				CPPUNIT_FAIL (_STDSTR(QString("%1: compile unexpectedly OK").arg(id))) ;
			}
			return	;
		}

		if (code == 0)
		{	delete	node ;
			CPPUNIT_FAIL (_STDSTR(QString("%1: compile failed: %2").arg(id).arg(error.getMessage()))) ;
		}

		KBValue	        rv	 ;
		KBScript::ExeRC rc	 = code->execute (0, 0, rv) ;

		delete		node	 ;

		if (rc != erc.toInt())
			CPPUNIT_FAIL (_STDSTR(QString("%1: %2: expected return code: %3").arg(id).arg(rc).arg(erc.toInt()))) ;
		if (rv.getRawText() != erv)
			CPPUNIT_FAIL (_STDSTR(QString("%1: %2: expected result: %3").arg(id).arg(rv.getRawText()).arg(erv))) ;

		return	;
	}

}

void	KBScriptIFTest::testScript ()
{
	evaluate (m_elem) ;
}

static	void	addScriptTests
	(	CppUnit::TestSuite	*suite,
		const char		*spec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile (spec, "test/script", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"script: %s: cannot open: %s\n",
			spec,
			doc.lastError().getMessage().latin1()
		)	;
		exit	(1) ;
	}
	
	QDomElement  docElem	= doc.documentElement() ;

        CppUnit::TestSuite *s = new CppUnit::TestSuite(_STDSTR(docElem.attribute("name"))) ;

	for (QDomNode node = docElem.firstChild	() ;
			     !node  .isNull	() ;
		      node = node   .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull()) continue ;

		s->addTest
		(	new CppUnit::TestCaller<KBScriptIFTest>
			(	_STDSTR(elem.attribute("id")),
				&KBScriptIFTest::testScript,
				new KBScriptIFTest (_STDSTR(elem.attribute("id")), elem)
		)	)	;
	}

	suite->addTest (s) ;
}

CppUnit::Test
	*KBScriptIFTest::suite ()
{
	extern	void	rekallTestLoadScripting () ;
	rekallTestLoadScripting () ;

#ifdef	__NOTDEF
	QString		 dtDir	= locateDir
				  (	"appdata",
					"services/rekall_dummy.desktop"
				  )	;
	QPtrList<KBDesktop> dtDefs	;
#if	__KB_RUNTIME
	KBDesktop::scan (dtDir + "/services", "rekallrt_",   dtDefs) ;
#else
	KBDesktop::scan (dtDir + "/services", "rekall_", dtDefs) ;
#endif
	KBLibLoader	*loader	= KBLibLoader::self () ;

	for (unsigned int idx = 0 ; idx < dtDefs.count() ; idx += 1)
	{
		KBDesktop *desktop = dtDefs.at(idx) ;

		if (desktop->property ("ServiceTypes") != "Rekall/Script")
			continue	;

		QString	libname	 = desktop->property("X-KDE-Library") ;
		if (libname.isEmpty()) continue ;

		KBLibrary *lib 	 = loader->getGlobalLibrary (libname) ;
		if (lib == 0)
		{
			fprintf
			(	stderr,
				"script: %s: %s\n",
				(cchar *)libname,
				(cchar *)loader->lastErrorMessage()
			)	;
			continue;
		}

		KBFactory *factory = (KBFactory *)lib->factory () ;
		if (factory ==  0)
		{	
			fprintf	(stderr, "script: cannot create %s\n", (cchar *)libname) ;
			continue  ;
		}

		QString	lang	= desktop->property("X-KDE-RekallPart-Language") ;
		QString	options	= desktop->property("X-KDE-RekallPart-Options" ) ;

//		fprintf
//		(	stderr,
//			"script: got language [%s] f=%p\n",
//			(cchar *)lang,
//			(void  *)factory
//		)	;

		KBScriptIF::registerScript  (lang, factory, options) ;
	}
#endif
	CppUnit::TestSuite *s = new CppUnit::TestSuite("ScriptIF") ;
	addScriptTests (s, "script_py1.xml" ) ;
	addScriptTests (s, "script_py2.xml" ) ;
	addScriptTests (s, "script_kjs1.xml") ;
	addScriptTests (s, "script_kjs2.xml") ;
	return s ;
}

CppUnit::Test EXPORT_API *scriptIFSetup
	(	const QMap<QString,QString> *
	)
{
	return	KBScriptIFTest::suite () ;
}

