/***************************************************************************
    file	         : navigate.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#define		private	public

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#if		__KB_PLAT_WIN32
#define		PAUSE	_sleep (1)
#else
#include	<unistd.h>
#define		PAUSE	usleep(100000)
#endif

#include	"rk_application.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_dom.h"
#include	"kb_field.h"
#include	"kb_ascii.h"
#include	"kb_macro.h"

#include	"accessor.h"
#include	"tester.h"
#include	"tester_kbase.h"

#include	"api_if.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_REPORTS	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_REPORTS
#endif



class	KBTestNavigateTest : public KBAPIIF
{
	KBDBInfo		*m_dbInfo	;
	TestOpenForm		*m_form		;

	KBField			*field_id	;
	KBField			*field_t	;
	KBField			*field_n	;

	KBMacroExec		*m_macro	;

	QString			nullToEmpty	(const QString &) ;

public	:

	KBTestNavigateTest	() ;
	KBTestNavigateTest	(std::string name) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testNavigate0	() ;
	void			testNavigate1	() ;
	void			testNavigate2	() ;
	void			testNavigate3	() ;
	void			testNavigate4	() ;
	void			testNavigate5	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBTestNavigateTest::KBTestNavigateTest ()
{
}

KBTestNavigateTest::KBTestNavigateTest
	(	std::string		name
	)
	:
	KBAPIIF	(name)
{
}

void	KBTestNavigateTest::setUp ()
{
	KBError	 error	;

	m_dbInfo = rekallDBInfo ("Misc.rkl", "test/misc") ;
	m_form	 = rekallTestOpenForm (m_dbInfo, "misc", "Navigate", error) ;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(error.getMessage()),
		m_form != 0
	)	;

	m_docroot	= m_form->m_form	;

	field_id	= getField ("id", "id") ;
	field_t		= getField ("t",  "t" ) ;
	field_n		= getField ("n",  "n" ) ;

	m_macro		= new KBMacroExec (m_dbInfo, "!Files", "tests") ;
	m_macro->setName ("navigate") ;

	KBControl	*c_id_0	= getControl	(field_id, 0, 0) ;
	RKLineEdit	*e_id_0	= getRKLineEdit	(c_id_0) ;

	e_id_0->setActiveWindow() ;
	for (uint idx = 0 ; idx < 10 ; idx += 1)
		if (!e_id_0->isActiveWindow())
		{	qApp->syncX() ;
			PAUSE	;
			qApp->processEvents() ;
		}

	CPPUNIT_ASSERT
	(	e_id_0->isActiveWindow()
	)	;

}

void	KBTestNavigateTest::tearDown ()
{
	rekallTestCloseForm ()	;
	delete	m_dbInfo	;
	delete	m_macro		;
}

void	KBTestNavigateTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBTestNavigateTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

QString	KBTestNavigateTest::nullToEmpty
	(	const QString	&text
	)
{
	return	text.isNull() ? QString("") : text ;
}

void	KBTestNavigateTest::testNavigate0 ()
{
	KBControl  *ctrl_id_0	= getControl	(field_id, 0, 0) ;
	RKLineEdit *le_id_0	= getRKLineEdit	(ctrl_id_0)	 ;

	KBControl  *ctrl_id_1	= getControl	(field_id, 1, 1) ;
	RKLineEdit *le_id_1	= getRKLineEdit	(ctrl_id_1)	 ;

	le_id_0->setActiveWindow() ;
	for (uint idx = 0 ; idx < 10 ; idx += 1)
		if (!le_id_0->isActiveWindow())
		{	qApp->syncX() ;
			PAUSE ;
			qApp->processEvents() ;
		}

	CPPUNIT_ASSERT
	(	le_id_0->isActiveWindow()
	)	;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (le_id_0, &me) ;

	qApp->processEvents() ;
	if (!le_id_0->hasFocus())
	{	QWidget *fw = qApp->focusWidget() ;
		CPPUNIT_FAIL (_STDSTR(QString("Unexpected focus: %1").arg(fw == 0 ? "none" : fw->className()))) ;
	}

	KBError		error	;
	QStringList	args	;
	args.append	("/id")	;
	args.append	("id" )	;
	args.append	("0"  )	;
	args.append	(QString("%1:%2:%3").arg(Qt::Key_Down).arg(0).arg(0)) ;
	bool		rc	= m_macro->append
				  (	"KeyNavigation",
					args,
					QString::null,
					error
				  )	;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("%1: %2").arg(error.getMessage()).arg(error.getDetails())),
		rc
	)	;

	KBScriptError	*se	= m_macro->execute (m_docroot) ;
	if (se != 0)
		CPPUNIT_FAIL
		(	_STDSTR(QString("%1: %2").arg(error.getMessage()).arg(error.getDetails()))
		)	;

	qApp->processEvents() ;
	if (!le_id_1 ->hasFocus())
	{	QWidget *fw = qApp->focusWidget() ;
		CPPUNIT_FAIL (_STDSTR(QString("Unexpected focus: %1").arg(fw == 0 ? "none" : fw->className()))) ;
	}
}

void	KBTestNavigateTest::testNavigate1 ()
{
	KBControl  *ctrl_id_0	= getControl	(field_id, 0, 0) ;
	RKLineEdit *le_id_0	= getRKLineEdit	(ctrl_id_0)	 ;

	KBControl  *ctrl_t_0	= getControl	(field_t,  0, 0) ;
	RKLineEdit *le_t_0	= getRKLineEdit	(ctrl_t_0)	 ;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (le_id_0, &me) ;

	qApp->processEvents() ;
	if (!le_id_0->hasFocus())
	{	QWidget *fw = qApp->focusWidget() ;
		CPPUNIT_FAIL (_STDSTR(QString("Unexpected focus: %1").arg(fw == 0 ? "none" : fw->className()))) ;
	}

	KBError		error	;
	QStringList	args	;
	args.append	("/id")	;
	args.append	("id" )	;
	args.append	("0"  )	;
	args.append	(QString("%1:%2:%3").arg(Qt::Key_Tab).arg(0).arg(0)) ;
	bool		rc	= m_macro->append
				  (	"KeyNavigation",
					args,
					QString::null,
					error
				  )	;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("%1: %2").arg(error.getMessage()).arg(error.getDetails())),
		rc
	)	;

	KBScriptError	*se	= m_macro->execute (m_docroot) ;
	if (se != 0)
		CPPUNIT_FAIL
		(	_STDSTR(QString("%1: %2").arg(error.getMessage()).arg(error.getDetails()))
		)	;

	qApp->processEvents() ;
	if (!le_t_0 ->hasFocus())
	{	QWidget *fw = qApp->focusWidget() ;
		CPPUNIT_FAIL (_STDSTR(QString("Unexpected focus: %1").arg(fw == 0 ? "none" : fw->className()))) ;
	}
}

void	KBTestNavigateTest::testNavigate2 ()
{
	KBControl  *ctrl_id_0	= getControl	(field_id, 0, 0) ;
	RKLineEdit *le_id_0	= getRKLineEdit	(ctrl_id_0)	 ;

	KBControl  *ctrl_t_0	= getControl	(field_t,  0, 0) ;
	RKLineEdit *le_t_0	= getRKLineEdit	(ctrl_t_0)	 ;

	QMouseEvent me (QEvent::MouseButtonPress, QPoint(2,2), 0, 0) ;
	me.spont = true ;
	qApp->notify (le_t_0, &me) ;

	qApp->processEvents() ;
	if (!le_t_0->hasFocus())
	{	QWidget *fw = qApp->focusWidget() ;
		CPPUNIT_FAIL (_STDSTR(QString("Unexpected focus: %1").arg(fw == 0 ? "none" : fw->className()))) ;
	}

	KBError		error	;
	QStringList	args	;
	args.append	("/t")	;
	args.append	("t" )	;
	args.append	("0"  )	;
	args.append	(QString("%1:%2:%3").arg(Qt::Key_BackTab).arg(0).arg(0)) ;
	bool		rc	= m_macro->append
				  (	"KeyNavigation",
					args,
					QString::null,
					error
				  )	;

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("%1: %2").arg(error.getMessage()).arg(error.getDetails())),
		rc
	)	;

	KBScriptError	*se	= m_macro->execute (m_docroot) ;
	if (se != 0)
		CPPUNIT_FAIL
		(	_STDSTR(QString("%1: %2").arg(se->error().getMessage()).arg(se->error().getDetails()))
		)	;

	qApp->processEvents() ;
	if (!le_id_0 ->hasFocus())
	{	QWidget *fw = qApp->focusWidget() ;
		CPPUNIT_FAIL (_STDSTR(QString("Unexpected focus: %1").arg(fw == 0 ? "none" : fw->className()))) ;
	}
}

void	KBTestNavigateTest::testNavigate3 ()
{
}

void	KBTestNavigateTest::testNavigate4 ()
{
}

void	KBTestNavigateTest::testNavigate5 ()
{
}

CppUnit::Test
	*KBTestNavigateTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("navigate") ;

	if (rekallTestSkip ("navigate")) return s ;

	extern	void	rekallTestLoadScripting () ;
	extern	void	registerAllNodes	() ;

	registerAllNodes	() ;
	rekallTestLoadScripting () ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestNavigateTest> ("noDisplay", &KBTestNavigateTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBTestNavigateTest> ("withGUI",   &KBTestNavigateTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBTestNavigateTest>
			(	"testNavigate0",
				&KBTestNavigateTest::testNavigate0
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestNavigateTest>
			(	"testNavigate1",
				&KBTestNavigateTest::testNavigate1
		)	)	;
		s->addTest
		(	new CppUnit::TestCaller<KBTestNavigateTest>
			(	"testNavigate2",
				&KBTestNavigateTest::testNavigate2
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_REPORTS *testsNavigateTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBTestNavigateTest::suite (__settings) ;
}
