/***************************************************************************
    file	         : ctrl.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<cppunit/TestCase.h>
#include	<cppunit/TestCaller.h>
#include	<cppunit/TestSuite.h>
#include	<cppunit/extensions/HelperMacros.h>

#include	<qmainwindow.h>
#include	<qstring.h>
#include	<qmap.h>
#include	<qregexp.h>

#include	"rk_application.h"
#include	"rk_pushbutton.h"

#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_locator.h"
#include	"kb_dom.h"
#include	"kb_wizard.h"

#include	"tester.h"
#include	"tester_kbase.h"
#include	"accessor.h"

#if		defined(__WIN32__)
#define		_STDSTR(s)	((s).isNull() ? "" : (std::string)(s).latin1())
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		_STDSTR(s)	(s)
#define		EXPORT_API
#endif



class	KBWizardCtrlTest : public CppUnit::TestCase
{
	QString			m_pageName	;
	QDomElement		m_pageSpec	;
	KBDomDocument		*m_pages	;

public	:

	KBWizardCtrlTest	() ;
	KBWizardCtrlTest	(std::string name, const QString &) ;

	virtual	void		setUp		() ;
	virtual	void		tearDown	() ;

	void			noDisplay	() ;
	void			withGUI		() ;

	void			testCtrl1	() ;

	static CppUnit::Test	*suite		(const QMap<QString,QString> *) ;
}	;

KBWizardCtrlTest::KBWizardCtrlTest ()
{
	m_pages	= 0 ;
}

KBWizardCtrlTest::KBWizardCtrlTest
	(	std::string		name,
		const QString		&pageName
	)
	:
	CppUnit::TestCase (name),
	m_pageName	  (pageName)
{
	m_pages	= 0 ;
}

void	KBWizardCtrlTest::setUp ()
{
	m_pages	= new KBDomDocument ;
	if (!m_pages->loadFile ("pages.xml", "test/wizard", getenv("TESTPATH")))
	{
		fprintf
		(	stderr,
			"wizard: cannot open: %s\n",
			"pages.xml"
		)	;
		exit	(1) ;
	}

	DOMITER_BEGIN(m_pages->documentElement(), "page", spec)
	{
		if (spec.attribute("name") == m_pageName)
		{
			m_pageSpec = spec ;
			break	;
		}
	}
	DOMITER_END

	CPPUNIT_ASSERT_MESSAGE
	(	_STDSTR(QString("Page '%1' not found").arg(m_pageName)),
		!m_pageSpec.isNull()
	)	;
}

void	KBWizardCtrlTest::tearDown ()
{
	delete	m_pages	;
}

void	KBWizardCtrlTest::noDisplay ()
{
	CPPUNIT_FAIL ("No $DISPLAY X11 environment") ;
}

void	KBWizardCtrlTest::withGUI ()
{
	CPPUNIT_FAIL ("Running with --gui") ;
}

void	KBWizardCtrlTest::testCtrl1 ()
{
	KBWizard	*wiz	= new KBWizard (0, "testCtrl1") ;
	KBWizardPage	*page	= wiz->addNewPage (m_pageSpec ) ;

	page->setCtrl	("text1", "test_string") ;

	CPPUNIT_ASSERT_EQUAL_MESSAGE
	(	_STDSTR(QString("Unexpected control value")),
		_STDSTR(QString("test_string")),
		_STDSTR(wiz->ctrlValue("ctrl1", "text1"))
	)	;

	delete	wiz	;
}

CppUnit::Test
	*KBWizardCtrlTest::suite
	(	const QMap<QString,QString>	*settings
	)
{
	CppUnit::TestSuite *s = new CppUnit::TestSuite("Button") ;

	if (rekallTestSkip ("wizard")) return s ;

	bool	fail	= false	;

#if	__KB_PLAT_LINUX
	if (getenv("DISPLAY") == 0)
	{
		s->addTest (new CppUnit::TestCaller<KBWizardCtrlTest> ("noDisplay", &KBWizardCtrlTest::noDisplay)) ;
		fail	= true	;
	}
#endif
	if ((*settings)["gui"].toInt() > 0)
	{
		s->addTest (new CppUnit::TestCaller<KBWizardCtrlTest> ("withGUI",   &KBWizardCtrlTest::withGUI)) ;
		fail	= true	;
	}

	if (!fail)
	{
		s->addTest
		(	new CppUnit::TestCaller<KBWizardCtrlTest>
			(	"ctrl1",
				&KBWizardCtrlTest::testCtrl1,
				new KBWizardCtrlTest (_STDSTR(QString("ctrl1")), "ctrl1")
		)	)	;
	}

	return s ;
}

CppUnit::Test EXPORT_API *ctrlTestSuite
	(	const QMap<QString,QString> *__settings
	)
{
	return	KBWizardCtrlTest::suite (__settings) ;
}
